@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto & The Claws Mail Team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto & The Claws Mail Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -51,11 +52,16 @@
 #include "action.h"
 #include "compose.h"
 #include "procmsg.h"
+#include "msgcache.h"
 #include "textview.h"
 #include "matcher_parser.h" /* CLAWS */
 #include "filtering.h"
 #include "procheader.h"
 
+#ifdef G_OS_WIN32
+#include <windows.h>
+#endif
+
 typedef struct _Children		Children;
 typedef struct _ChildInfo		ChildInfo;
 typedef struct _UserStringDialog	UserStringDialog;
@@ -88,11 +94,11 @@ struct _ChildInfo
 {
 	Children	*children;
 	gchar		*cmd;
-	pid_t		 pid;
+	GPid		 pid;
+	gint		 next_sig;
 	gint		 chld_in;
 	gint		 chld_out;
 	gint		 chld_err;
-	gint		 chld_status;
 	gint		 tag_in;
 	gint		 tag_out;
 	gint		 tag_err;
@@ -179,13 +185,11 @@ static gint io_dialog_key_pressed_cb	(GtkWidget	*widget,
 
 static void catch_output		(gpointer		 data,
 					 gint			 source,
-					 GdkInputCondition	 cond);
+					 GIOCondition		 cond);
 static void catch_input			(gpointer		 data, 
 					 gint			 source,
-					 GdkInputCondition	 cond);
-static void catch_status		(gpointer		 data,
-					 gint			 source,
-					 GdkInputCondition	 cond);
+					 GIOCondition		 cond);
+static void catch_status		(GPid pid, gint status, gpointer data);
 
 static gchar *get_user_string		(const gchar	*action,
 					 ActionType	 type);
@@ -194,6 +198,7 @@ static gchar *get_user_string		(const gchar	*action,
 ActionType action_get_type(const gchar *action_str)
 {
 	const gchar *p;
+	gboolean in_filtering_action = FALSE;
 	ActionType action_type = ACTION_NONE;
 
 	cm_return_val_if_fail(action_str,  ACTION_ERROR);
@@ -216,55 +221,60 @@ ActionType action_get_type(const gchar *action_str)
 		return ACTION_ERROR;
 
 	while (*p && action_type != ACTION_ERROR) {
-		if (p[0] == '%' && p[1]) {
-			switch (p[1]) {
-			case 'a':
-				/* CLAWS: filtering action is a mutually exclusive
-				 * action. we can enable others if needed later. we
-				 * add ACTION_SINGLE | ACTION_MULTIPLE so it will
-				 * only be executed from the main window toolbar */
-				if (p[2] == 's')  /* source messages */
-					action_type = ACTION_FILTERING_ACTION 
-						    | ACTION_SINGLE 
-						    | ACTION_MULTIPLE;
-				break;
-			case 'f':
-				action_type |= ACTION_SINGLE;
-				break;
-			case 'F':
-				action_type |= ACTION_MULTIPLE;
-				break;
-			case 'p':
-				action_type |= ACTION_SINGLE;
-				break;
-			case 's':
-				action_type |= ACTION_SELECTION_STR;
-				break;
-			case 'u':
-				action_type |= ACTION_USER_STR;
-				break;
-			case 'h':
-				action_type |= ACTION_USER_HIDDEN_STR;
-				break;
-			case '%':
-				/* literal '%' */
-				break;
-			default:
-				action_type = ACTION_ERROR;
-				break;
+		if (!in_filtering_action) {
+			if (p[0] == '%' && p[1]) {
+				switch (p[1]) {
+				case 'a':
+					/* CLAWS: filtering action is a mutually exclusive
+					* action. we can enable others if needed later. we
+					* add ACTION_SINGLE | ACTION_MULTIPLE so it will
+					* only be executed from the main window toolbar */
+					if (p[2] == 's')  /* source messages */
+						action_type = ACTION_FILTERING_ACTION 
+								| ACTION_SINGLE 
+								| ACTION_MULTIPLE;
+					in_filtering_action = TRUE;
+					break;
+				case 'f':
+					action_type |= ACTION_SINGLE;
+					break;
+				case 'F':
+					action_type |= ACTION_MULTIPLE;
+					break;
+				case 'p':
+					action_type |= ACTION_SINGLE;
+					break;
+				case 's':
+					action_type |= ACTION_SELECTION_STR;
+					break;
+				case 'u':
+					action_type |= ACTION_USER_STR;
+					break;
+				case 'h':
+					action_type |= ACTION_USER_HIDDEN_STR;
+					break;
+				case '%':
+					/* literal '%' */
+					break;
+				default:
+					action_type = ACTION_ERROR;
+					break;
+				}
+				p++;
+			} else if (p[0] == '|') {
+				if (p[1] == '\0')
+					action_type |= ACTION_PIPE_OUT;
+			} else if (p[0] == '>') {
+				if (p[1] == '\0')
+					action_type |= ACTION_INSERT;
+			} else if (p[0] == '&') {
+				if (p[1] == '\0')
+					action_type |= ACTION_ASYNC;
+			} else if (p[0] == '}') {
+				in_filtering_action = FALSE;
 			}
-			p++;
-		} else if (p[0] == '|') {
-			if (p[1] == '\0')
-				action_type |= ACTION_PIPE_OUT;
-		} else if (p[0] == '>') {
-			if (p[1] == '\0')
-				action_type |= ACTION_INSERT;
-		} else if (p[0] == '&') {
-			if (p[1] == '\0')
-				action_type |= ACTION_ASYNC;
 		}
-		p++;
+			p++;
 	}
 
 	return action_type;
@@ -376,15 +386,21 @@ static gboolean parse_append_filename(GString *cmd, MsgInfo *msginfo)
 	}
 
 	p = filename;
-	while ((q = strpbrk(p, "$\"`'\\ \t*?[]&|;<>()!#~")) != NULL) {
+	g_string_append(cmd, "\"");
+#ifdef G_OS_UNIX
+	while ((q = strpbrk(p, "$\"`\\~")) != NULL) {
 		escape_ch[1] = *q;
 		*q = '\0';
 		g_string_append(cmd, p);
 		g_string_append(cmd, escape_ch);
 		p = q + 1;
 	}
-	g_string_append(cmd, p);
 
+	g_string_append(cmd, p);
+#else
+	g_string_append(cmd, filename);
+#endif
+	g_string_append(cmd, "\"");
 	g_free(filename);
 
 	return TRUE;
@@ -469,15 +485,22 @@ void action_update_compose_menu(GtkUIManager *ui_manager,
 
 static GtkWidget *find_item_in_menu(GtkWidget *menu, gchar *name)
 {
-	GList *amenu = GTK_MENU_SHELL(menu)->children;
+	GList *children = gtk_container_get_children(GTK_CONTAINER(GTK_MENU_SHELL(menu)));
+	GList *amenu = children;
 	const gchar *existing_name;
 	while (amenu) {
 		GtkWidget *item = GTK_WIDGET(amenu->data);
 		if ((existing_name = g_object_get_data(G_OBJECT(item), "s_name")) != NULL &&
 		    !strcmp2(name, existing_name))
+		{
+			g_list_free(children);
 			 return item;
+		}
 		amenu = amenu->next;
 	}
+
+	g_list_free(children);
+
 	return NULL;
 }
 
@@ -890,10 +913,7 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 			child_info->callback = callback;
 			child_info->data = data;
 			child_info->tag_status = 
-				claws_input_add(child_info->chld_status,
-					      GDK_INPUT_READ,
-					      catch_status, child_info,
-					      FALSE);
+				g_child_watch_add(child_info->pid, catch_status, child_info);
 		}
 
 		create_io_dialog(children);
@@ -904,157 +924,95 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 			     Children *children)
 {
-#ifdef G_OS_UNIX
-	gint chld_in[2], chld_out[2], chld_err[2], chld_status[2];
-	gchar *cmdline[4], *ret_str;
-	pid_t pid, gch_pid;
+	gint chld_in, chld_out, chld_err;
+	gchar **argv, *ret_str, *trim_cmd;
+	GPid pid;
 	ChildInfo *child_info;
-	gint sync;
+	gint follow_child;
 	gssize by_written = 0, by_read = 0;
+	gboolean result = FALSE;
+	GError *error = NULL;
 
-	sync = !(children->action_type & ACTION_ASYNC);
+	follow_child = !(children->action_type & ACTION_ASYNC);
 
-	chld_in[0] = chld_in[1] = chld_out[0] = chld_out[1] = chld_err[0]
-		= chld_err[1] = chld_status[0] = chld_status[1] = -1;
+	chld_in = chld_out = chld_err = -1;
 
-	if (sync) {
-		if (pipe(chld_status) || pipe(chld_in) || pipe(chld_out) ||
-		    pipe(chld_err)) {
-			alertpanel_error(_("Command could not be started. "
-					   "Pipe creation failed.\n%s"),
-					g_strerror(errno));
-			/* Closing fd = -1 fails silently */
-			(void)close(chld_in[0]);
-			(void)close(chld_in[1]);
-			(void)close(chld_out[0]);
-			(void)close(chld_out[1]);
-			(void)close(chld_err[0]);
-			(void)close(chld_err[1]);
-			(void)close(chld_status[0]);
-			(void)close(chld_status[1]);
-			return NULL; /* Pipe error */
-		}
+	ret_str = g_locale_from_utf8(cmd, strlen(cmd),
+				     &by_read, &by_written,
+				     NULL);
+	if (!ret_str || !by_written)
+		ret_str = g_strdup(cmd);
+
+	trim_cmd = ret_str;
+
+	while (g_ascii_isspace(trim_cmd[0]))
+		trim_cmd++;
+
+#ifdef G_OS_UNIX
+	argv = g_new0(gchar *, 4);
+	argv[0] = g_strdup("/bin/sh");
+	argv[1] = g_strdup("-c");
+	argv[2] = g_strdup(trim_cmd);
+	argv[3] = 0;
+#else
+	argv = strsplit_with_quote(trim_cmd, " ", 0);
+#endif
+	g_free(ret_str);
+
+	if (follow_child) {
+		result = g_spawn_async_with_pipes(NULL, argv, NULL,
+			  G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
+			  NULL, NULL, &pid, &chld_in, &chld_out,
+			  &chld_err, &error);
+	} else {
+		result = g_spawn_async(NULL, argv, NULL, 
+			  G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH, NULL, NULL,
+			  &pid, &error);
 	}
 
-	debug_print("Forking child and grandchild.\n");
-	debug_print("Executing: /bin/sh -c %s\n", cmd);
+	debug_print("spawning %s: %d\n", cmd, result);
 
-	pid = fork();
-	if (pid == 0) { /* Child */
-		if (setpgid(0, 0))
-			perror("setpgid");
+	g_strfreev(argv);
 
-#ifdef GDK_WINDOWING_X11
-		(void)close(ConnectionNumber(gdk_display));
-#endif /* GDK_WINDOWING_X11 */
-
-		gch_pid = fork();
-
-		if (gch_pid == 0) {
-			if (setpgid(0, getppid()))
-				perror("setpgid");
-
-			if (sync) {
-				if (children->action_type &
-				    (ACTION_PIPE_IN |
-				     ACTION_USER_IN |
-				     ACTION_USER_HIDDEN_IN)) {
-					(void)close(fileno(stdin));
-					(void)dup  (chld_in[0]);
-				}
-				(void)close(chld_in[0]);
-				(void)close(chld_in[1]);
-
-				(void)close(fileno(stdout));
-				(void)dup  (chld_out[1]);
-				(void)close(chld_out[0]);
-				(void)close(chld_out[1]);
-
-				(void)close(fileno(stderr));
-				(void)dup  (chld_err[1]);
-				(void)close(chld_err[0]);
-				(void)close(chld_err[1]);
-			}
-
-			cmdline[0] = "sh";
-			cmdline[1] = "-c";
-			ret_str = g_locale_from_utf8(cmd, strlen(cmd),
-						     &by_read, &by_written,
-						     NULL);
-			if (ret_str && by_written)
-				cmdline[2] = ret_str;
-			else
-				cmdline[2] = cmd;
-			cmdline[3] = NULL;
-			execvp("/bin/sh", cmdline);
-
-			perror("execvp");
-			g_free(ret_str);
-			_exit(1);
-		} else if (gch_pid < (pid_t) 0) { /* Fork error */
-			if (sync)
-				(void)write(chld_status[1], "1\n", 2);
-			perror("fork");
-			_exit(1);
-		} else { /* Child */
-			if (sync) {
-				(void)close(chld_in[0]);
-				(void)close(chld_in[1]);
-				(void)close(chld_out[0]);
-				(void)close(chld_out[1]);
-				(void)close(chld_err[0]);
-				(void)close(chld_err[1]);
-				(void)close(chld_status[0]);
-
-				debug_print("Child: waiting for grandchild\n");
-				waitpid(gch_pid, NULL, 0);
-				debug_print("Child: grandchild ended\n");
-				(void)write(chld_status[1], "0\n", 2);
-				(void)close(chld_status[1]);
-			}
-			_exit(0);
-		}
-	} else if (pid < 0) { /* Fork error */
+	if (!result) {
 		alertpanel_error(_("Could not fork to execute the following "
-				   "command:\n%s\n%s"),
-				 cmd, g_strerror(errno));
-		return NULL; 
-	}
-
-	/* Parent */
-
-	if (!sync) {
-		waitpid(pid, NULL, 0);
+				"command:\n%s\n%s"),
+				 cmd, error ? error->message : _("Unknown error"));
+		if (error)
+			g_error_free(error);
 		return NULL;
 	}
 
-	(void)close(chld_in[0]);
 	if (!(children->action_type &
 	      (ACTION_PIPE_IN | ACTION_USER_IN | ACTION_USER_HIDDEN_IN)))
-		(void)close(chld_in[1]);
-	(void)close(chld_out[1]);
-	(void)close(chld_err[1]);
-	(void)close(chld_status[1]);
+		(void)close(chld_in);
 
+	if (!follow_child) {
+		g_spawn_close_pid(pid);
+		return NULL;
+	}
 	child_info = g_new0(ChildInfo, 1);
 
 	child_info->children    = children;
 
 	child_info->pid         = pid;
+#ifdef G_OS_UNIX
+	child_info->next_sig	= SIGTERM;
+#endif
 	child_info->cmd         = g_strdup(cmd);
 	child_info->new_out     = FALSE;
 	child_info->output      = g_string_new(NULL);
 	child_info->chld_in     =
 		(children->action_type &
 		 (ACTION_PIPE_IN | ACTION_USER_IN | ACTION_USER_HIDDEN_IN))
-			? chld_in [1] : -1;
-	child_info->chld_out    = chld_out[0];
-	child_info->chld_err    = chld_err[0];
-	child_info->chld_status = chld_status[0];
+			? chld_in : -1;
+	child_info->chld_out    = chld_out;
+	child_info->chld_err    = chld_err;
+	child_info->tag_status  = -1;
 	child_info->tag_in      = -1;
-	child_info->tag_out     = claws_input_add(chld_out[0], GDK_INPUT_READ,
+	child_info->tag_out     = claws_input_add(chld_out, G_IO_IN | G_IO_HUP | G_IO_ERR,
 						catch_output, child_info, FALSE);
-	child_info->tag_err     = claws_input_add(chld_err[0], GDK_INPUT_READ,
+	child_info->tag_err     = claws_input_add(chld_err, G_IO_IN | G_IO_HUP | G_IO_ERR,
 						catch_output, child_info, FALSE);
 
 	if (!(children->action_type &
@@ -1062,28 +1020,27 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 		return child_info;
 
 	if ((children->action_type & ACTION_PIPE_IN) && msg_str) {
+		int r;
 		ret_str = g_locale_from_utf8(msg_str, strlen(msg_str),
 					     &by_read, &by_written, NULL);
 		if (ret_str && by_written) {
-			(void)write(chld_in[1], ret_str, strlen(ret_str));
+			r = write(chld_in, ret_str, strlen(ret_str));
 			g_free(ret_str);
 		} else
-			(void)write(chld_in[1], msg_str, strlen(msg_str));
+			r = write(chld_in, msg_str, strlen(msg_str));
 		if (!(children->action_type &
 		      (ACTION_USER_IN | ACTION_USER_HIDDEN_IN)))
-			(void)close(chld_in[1]);
+			r = close(chld_in);
 		child_info->chld_in = -1; /* No more input */
+		if (r != 0)
+			debug_print("%s(%d)", strerror(errno), errno);
 	}
 
 	return child_info;
-#else
-	return NULL;
-#endif /* G_OS_UNIX */
 }
 
 static void kill_children_cb(GtkWidget *widget, gpointer data)
 {
-#ifdef G_OS_UNIX
 	GSList *cur;
 	Children *children = (Children *) data;
 	ChildInfo *child_info;
@@ -1091,10 +1048,14 @@ static void kill_children_cb(GtkWidget *widget, gpointer data)
 	for (cur = children->list; cur; cur = cur->next) {
 		child_info = (ChildInfo *)(cur->data);
 		debug_print("Killing child group id %d\n", child_info->pid);
-		if (child_info->pid && kill(-child_info->pid, SIGTERM) < 0)
+#ifdef G_OS_UNIX
+		if (child_info->pid && kill(child_info->pid, child_info->next_sig) < 0)
 			perror("kill");
+		child_info->next_sig = SIGKILL;
+#else
+		TerminateProcess(child_info->pid, 0);
+#endif
 	}
-#endif /* G_OS_UNIX */
 }
 
 static gint wait_for_children(Children *children)
@@ -1102,23 +1063,18 @@ static gint wait_for_children(Children *children)
 	gboolean new_output;
 	ChildInfo *child_info;
 	GSList *cur;
-	gint nb = children->nb;
-
-	children->nb = 0;
 
 	cur = children->list;
 	new_output = FALSE;
 	while (cur) {
 		child_info = (ChildInfo *)cur->data;
-		if (child_info->pid)
-			children->nb++;
 		new_output |= child_info->new_out;
 		cur = cur->next;
 	}
 
 	children->output |= new_output;
 
-	if (new_output || (children->dialog && (nb != children->nb)))
+	if (new_output || (children->dialog && (children->initial_nb != children->nb)))
 		update_io_dialog(children);
 
 	if (children->nb)
@@ -1139,7 +1095,7 @@ static void send_input(GtkWidget *w, gpointer data)
 	ChildInfo *child_info = (ChildInfo *) children->list->data;
 
 	child_info->tag_in = claws_input_add(child_info->chld_in,
-					   GDK_INPUT_WRITE,
+					   G_IO_OUT | G_IO_ERR,
 					   catch_input, children, FALSE);
 }
 
@@ -1166,9 +1122,9 @@ static void hide_io_dialog_cb(GtkWidget *w, gpointer data)
 static gint io_dialog_key_pressed_cb(GtkWidget *widget, GdkEventKey *event,
 				     gpointer data)
 {
-	if (event && (event->keyval == GDK_Escape ||
-		      event->keyval == GDK_Return ||
-			  event->keyval == GDK_KP_Enter))
+	if (event && (event->keyval == GDK_KEY_Escape ||
+		      event->keyval == GDK_KEY_Return ||
+			  event->keyval == GDK_KEY_KP_Enter))
 		hide_io_dialog_cb(widget, data);
 	return TRUE;
 }
@@ -1193,8 +1149,6 @@ static void childinfo_close_pipes(ChildInfo *child_info)
 		(void)close(child_info->chld_out);
 	if (child_info->chld_err >= 0)
 		(void)close(child_info->chld_err);
-
-	(void)close(child_info->chld_status);
 }
 
 static void free_children(Children *children)
@@ -1230,17 +1184,23 @@ static void update_io_dialog(Children *children)
 
 	if (children->progress_bar) {
 		gchar *text;
+#ifdef GENERIC_UMPC
+		/* use a more compact format */
+		const gchar *format = "%s %d/%d";
+#else
+		const gchar *format = "%s %d / %d";
+#endif
 		
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(children->progress_bar),
 						  (children->initial_nb == 0) ? 0 :
 					      (gdouble) (children->initial_nb - children->nb) /
 					      (gdouble) children->initial_nb);
-		text = g_strdup_printf("%s %d/%d", _("Completed"), 
+		text = g_strdup_printf(format, _("Completed"), 
 				       children->initial_nb - children->nb,
 				       children->initial_nb);
 		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(children->progress_bar), text);
 		g_free(text);
-	}					      
+	}
 
 	if (!children->nb) {
 		gtk_widget_set_sensitive(children->abort_btn, FALSE);
@@ -1306,7 +1266,7 @@ static void create_io_dialog(Children *children)
 
 	dialog = gtk_dialog_new();
 	gtk_container_set_border_width
-		(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), 5);
+		(GTK_CONTAINER(gtk_dialog_get_action_area(GTK_DIALOG(dialog))), 5);
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
 	gtk_window_set_title(GTK_WINDOW(dialog), _("Action's input/output"));
 	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
@@ -1318,7 +1278,8 @@ static void create_io_dialog(Children *children)
 			 children);
 
 	vbox = gtk_vbox_new(FALSE, 8);
-	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
+	gtk_container_add(GTK_CONTAINER(
+				gtk_dialog_get_content_area(GTK_DIALOG(dialog))), vbox);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
 	gtk_widget_show(vbox);
 
@@ -1386,12 +1347,25 @@ static void create_io_dialog(Children *children)
 	}
 
 	if (children->initial_nb > 1) {
-		gchar * text;
+		gchar *text;
+#ifdef GENERIC_UMPC
+		/* use a more compact format */
+		const gchar *format = "%s 0/%d\n";
+#else
+		const gchar *format = "%s 0 / %d\n";
+#endif
 		
 		progress_bar = gtk_progress_bar_new();
+#if !GTK_CHECK_VERSION(3, 0, 0)
 		gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(progress_bar),
 				GTK_PROGRESS_LEFT_TO_RIGHT);
-		text = g_strdup_printf("%s 0/%d\n", _("Completed"), 
+#else
+		gtk_orientable_set_orientation(GTK_ORIENTABLE(progress_bar),
+				GTK_ORIENTATION_HORIZONTAL);
+		gtk_progress_bar_set_inverted(GTK_PROGRESS_BAR(progress_bar),
+				FALSE);
+#endif
+		text = g_strdup_printf(format, _("Completed"), 
 		                       children->initial_nb);
 		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress_bar),
 					  text);
@@ -1411,7 +1385,8 @@ static void create_io_dialog(Children *children)
 	if (children->nb)
 		gtk_widget_set_sensitive(close_button, FALSE);
 
-	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), hbox);
+	gtk_container_add(GTK_CONTAINER(
+			gtk_dialog_get_action_area(GTK_DIALOG(dialog))), hbox);
 
 	children->dialog       = dialog;
 	children->scrolledwin  = scrolledwin;
@@ -1425,21 +1400,14 @@ static void create_io_dialog(Children *children)
 	gtk_widget_show(dialog);
 }
 
-static void catch_status(gpointer data, gint source, GdkInputCondition cond)
+static void catch_status(GPid pid, gint status, gpointer data)
 {
 	ChildInfo *child_info = (ChildInfo *)data;
-	gchar buf;
-	gint c;
 
-	g_source_remove(child_info->tag_status);
+	debug_print("Child returned %d\n", status);
 
-	c = read(source, &buf, 1);
-	debug_print("Child returned %c\n", buf);
-
-#ifdef G_OS_UNIX
-	waitpid(-child_info->pid, NULL, 0);
-#endif
 	childinfo_close_pipes(child_info);
+	g_spawn_close_pid(child_info->pid);
 	child_info->pid = 0;
 
 	if (child_info->children->action_type & (ACTION_SINGLE | ACTION_MULTIPLE)
@@ -1487,19 +1455,22 @@ static void catch_status(gpointer data, gint source, GdkInputCondition cond)
 		child_info->msginfo_list = NULL;
 	}
 
+	if (!child_info->pid)
+		child_info->children->nb--;
+
 	wait_for_children(child_info->children);
 }
 	
-static void catch_input(gpointer data, gint source, GdkInputCondition cond)
+static void catch_input(gpointer data, gint source, GIOCondition cond)
 {
 	Children *children = (Children *)data;
 	ChildInfo *child_info = (ChildInfo *)children->list->data;
 	gchar *input, *ret_str;
-	gint c, count, len;
+	gint c, count, len, r;
 	gssize by_read = 0, by_written = 0;
 
 	debug_print("Sending input to grand child.\n");
-	if (!(cond && GDK_INPUT_WRITE))
+	if (!(cond & (G_IO_OUT | G_IO_ERR)))
 		return;
 
 	gtk_widget_set_sensitive(children->input_hbox, FALSE);
@@ -1527,16 +1498,19 @@ static void catch_input(gpointer data, gint source, GdkInputCondition cond)
 	} while (c >= 0 && count < len);
 
 	if (c >= 0)
-		(void)write(child_info->chld_in, "\n", 2);
+		r = write(child_info->chld_in, "\n", 2);
 
 	g_free(input);
 
-	(void)close(child_info->chld_in);
+	r = close(child_info->chld_in);
+	child_info->chld_in = -1;
+	if (r != 0)
+		debug_print("%s(%d)", strerror(errno), errno);
 	child_info->chld_in = -1;
 	debug_print("Input to grand child sent.\n");
 }
 
-static void catch_output(gpointer data, gint source, GdkInputCondition cond)
+static void catch_output(gpointer data, gint source, GIOCondition cond)
 {
 	ChildInfo *child_info = (ChildInfo *)data;
 	gint c;
@@ -1566,14 +1540,14 @@ static void catch_output(gpointer data, gint source, GdkInputCondition cond)
 				break;
 
 			ret_str = g_locale_to_utf8
-				(buf, c, &bytes_read, &bytes_written, NULL);
+				(buf, c - 1, &bytes_read, &bytes_written, NULL);
 			if (ret_str && bytes_written > 0) {
 				gtk_text_buffer_insert
 					(textbuf, &iter, ret_str,
 					 -1);
 				g_free(ret_str);
 			} else
-				gtk_text_buffer_insert(textbuf, &iter, buf, c);
+				gtk_text_buffer_insert(textbuf, &iter, buf, c - 1);
 		}
 
 		if (child_info->children->is_selection) {
