@@ -1,7 +1,7 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
  *
- * Copyright (C) 2000-2009 by Alfons Hoogervorst & The Claws Mail Team.
+ * Copyright (C) 2000-2012 by Alfons Hoogervorst & The Claws Mail Team.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 #include "defs.h"
 
@@ -35,13 +36,22 @@
 #  include <wctype.h>
 #endif
 
-#include "addrindex.h"
 #include "addr_compl.h"
+#include "addritem.h"
 #include "utils.h"
 #include "prefs_common.h"
 #include "claws.h"
+#include "hooks.h"
+#include "gtkutils.h"
+#include "stock_pixmap.h"
 #include <pthread.h>
 
+#ifndef USE_NEW_ADDRBOOK
+	#include "addrindex.h"
+#else
+	#include "addressbook-dbus.h"
+#endif
+
 /*!
  *\brief	For the GtkListStore
  */
@@ -69,18 +79,7 @@ enum {
  * addresses a little more (e.g. break up alfons@proteus.demon.nl into
  * something like alfons, proteus, demon, nl; and then completing on
  * any of those words).
- */ 
-	
-/**
- * address_entry - structure which refers to the original address entry in the
- * address book .
  */
-typedef struct
-{
-	gchar *name;
-	gchar *address;
-	GList *grp_emails;
-} address_entry;
 
 /**
  * completion_entry - structure used to complete addresses, with a reference
@@ -230,7 +229,7 @@ static void free_all(void)
  * \param str Index string value.
  * \param ae  Entry containing address data.
  */
-static void add_address1(const char *str, address_entry *ae)
+void addr_compl_add_address1(const char *str, address_entry *ae)
 {
 	completion_entry *ce1;
 	ce1 = g_new0(completion_entry, 1),
@@ -255,35 +254,32 @@ static void add_address1(const char *str, address_entry *ae)
 static gint add_address(const gchar *name, const gchar *address, 
 			const gchar *nick, const gchar *alias, GList *grp_emails)
 {
-	address_entry    *ae;
-	gboolean is_group = FALSE;
+	address_entry *ae;
 
-	if (!name || !address) {
-		if (!address && !nick && !alias && grp_emails) {
-			is_group = TRUE;
-		} else
-			return -1;
-	}
+	if (!address && !grp_emails)
+		return -1;
+
+	if (!name)
+		name = "";
 
 	ae = g_new0(address_entry, 1);
-
 	cm_return_val_if_fail(ae != NULL, -1);
 
-	ae->name    = g_strdup(name);
-	ae->address = g_strdup(address);		
+	ae->name = g_strdup(name);
+	ae->address = g_strdup(address);
 	ae->grp_emails = grp_emails;
 	g_address_list = g_list_prepend(g_address_list, ae);
 
-	add_address1(name, ae);
+	addr_compl_add_address1(name, ae);
+
 	if (address != NULL && *address != '\0')
-		add_address1(address, ae);
-	
+		addr_compl_add_address1(address, ae);
+
 	if (nick != NULL && *nick != '\0')
-		add_address1(nick, ae);
-	
-	if ( alias != NULL && *alias != '\0') {
-		add_address1(alias, ae);
-	}
+		addr_compl_add_address1(nick, ae);
+
+	if (alias != NULL && *alias != '\0')
+		addr_compl_add_address1(alias, ae);
 
 	return 0;
 }
@@ -295,14 +291,28 @@ static void read_address_book(gchar *folderpath) {
 	free_all_addresses();
 	free_completion_list();
 
+#ifndef USE_NEW_ADDRBOOK
 	addrindex_load_completion( add_address, folderpath );
+#else
+	GError* error = NULL;
+	
+	addrcompl_initialize();
+	if (! addrindex_dbus_load_completion(add_address, &error)) {
+		g_warning("Failed to populate address completion list");
+        g_error_free(error);
+		return;
+	}
+#endif
+	/* plugins may hook in here to modify/extend the completion list */
+	hooks_invoke(ADDDRESS_COMPLETION_BUILD_ADDRESS_LIST_HOOKLIST, &g_address_list);
+
 	g_address_list = g_list_reverse(g_address_list);
 	g_completion_list = g_list_reverse(g_completion_list);
 	/* merge the completion entry list into g_completion */
 	if (g_completion_list) {
 		g_completion_add_items(g_completion, g_completion_list);
 		if (debug_get_mode())
-			debug_print("read %d items in %s\n", 
+			debug_print("read %d items in %s\n",
 				g_list_length(g_completion_list),
 				folderpath?folderpath:"(null)");
 	}
@@ -758,8 +768,9 @@ static CompletionWindow *addrcompl_create_window( void ) {
  */
 static void addrcompl_destroy_window( CompletionWindow *cw ) {
 	/* Stop all searches currently in progress */
+#ifndef USE_NEW_ADDRBOOK
 	addrindex_stop_search( _queryID_ );
-
+#endif
 	/* Remove idler function... or application may not terminate */
 	if( _completionIdleID_ != 0 ) {
 		g_source_remove( _completionIdleID_ );
@@ -848,18 +859,35 @@ static void addrcompl_resize_window( CompletionWindow *cw ) {
 	gint x, y, width, height, depth;
 
 	/* Get current geometry of window */
-	gdk_window_get_geometry( cw->window->window, &x, &y, &width, &height, &depth );
+#if !GTK_CHECK_VERSION(3, 0, 0)
+	gdk_window_get_geometry( gtk_widget_get_window( cw->window ), &x, &y, &width, &height, &depth );
+#else
+	gdk_window_get_geometry( gtk_widget_get_window( cw->window ), &x, &y, &width, &height );
+#endif
 
+	/* simple _hide breaks size requisition !? */
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	gtk_widget_hide_all( cw->window );
 	gtk_widget_show_all( cw->window );
+#else
+	gtk_widget_hide( cw->window );
+	gtk_widget_show( cw->window );
+#endif
 	gtk_widget_size_request( cw->list_view, &r );
 
 	/* Adjust window height to available screen space */
-	if( ( y + r.height ) > gdk_screen_height() ) {
-		gtk_window_set_resizable(GTK_WINDOW(cw->window), FALSE);
-		gtk_widget_set_size_request( cw->window, width, gdk_screen_height() - y );
-	} else
-		gtk_widget_set_size_request(cw->window, width, r.height);
+	if( y + r.height > gdk_screen_height())
+		r.height = gdk_screen_height() - y;
+
+	gtk_widget_set_size_request(cw->window, width, r.height);
+
+	gdk_pointer_grab(gtk_widget_get_window(cw->window), TRUE,
+			 GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK |
+			 GDK_BUTTON_RELEASE_MASK,
+			 NULL, NULL, GDK_CURRENT_TIME);
+	gdk_keyboard_grab(gtk_widget_get_window(cw->window), FALSE, GDK_CURRENT_TIME);
+	gtk_grab_add(cw->window);
+
 }
 
 static GdkPixbuf *group_pixbuf = NULL;
@@ -972,6 +1000,7 @@ static gboolean addrcompl_idle( gpointer data ) {
  *                   criteria.
  * \param data       Query data.
  */
+#ifndef USE_NEW_ADDRBOOK
 static gint addrcompl_callback_entry(
 	gpointer sender, gint queryID, GList *listEMail, gpointer data )
 {
@@ -997,6 +1026,7 @@ static gint addrcompl_callback_entry(
 
 	return 0;
 }
+#endif
 
 /**
  * Clear the display queue.
@@ -1042,6 +1072,7 @@ static void addrcompl_load_local( void ) {
  * Start the search.
  */
 static void addrcompl_start_search( void ) {
+#ifndef USE_NEW_ADDRBOOK
 	gchar *searchTerm;
 
 	searchTerm = g_strdup( _compWindow_->searchTerm );
@@ -1050,6 +1081,7 @@ static void addrcompl_start_search( void ) {
 	_queryID_ = addrindex_setup_search(
 		searchTerm, NULL, addrcompl_callback_entry );
 	g_free( searchTerm );
+#endif
 	/* g_print( "addrcompl_start_search::queryID=%d\n", _queryID_ ); */
 
 	/* Load local stuff */
@@ -1057,10 +1089,14 @@ static void addrcompl_start_search( void ) {
 
 	/* Sit back and wait until something happens */
 	_completionIdleID_ =
-		g_idle_add( ( GtkFunction ) addrcompl_idle, NULL );
+		g_idle_add( (GSourceFunc) addrcompl_idle, NULL );
 	/* g_print( "addrindex_start_search::queryID=%d\n", _queryID_ ); */
 
+#ifndef USE_NEW_ADDRBOOK
 	addrindex_start_search( _queryID_ );
+#else
+	
+#endif
 }
 
 /**
@@ -1107,7 +1143,7 @@ static void completion_window_apply_selection(GtkTreeView *list_view,
 	g_free(text);
 
 	/* Move focus to next widget */
-	parent = GTK_WIDGET(entry)->parent;
+	parent = gtk_widget_get_parent(GTK_WIDGET(entry));
 	if( parent && move_focus) {
 		gtk_widget_child_focus( parent, GTK_DIR_TAB_FORWARD );
 	}
@@ -1162,7 +1198,7 @@ void address_completion_register_entry(GtkEntry *entry, gboolean allow_commas)
  */
 void address_completion_unregister_entry(GtkEntry *entry)
 {
-	GtkObject *entry_obj;
+	GObject *entry_obj;
 
 	cm_return_if_fail(entry != NULL);
 	cm_return_if_fail(GTK_IS_ENTRY(entry));
@@ -1217,14 +1253,14 @@ static gboolean address_completion_entry_key_pressed(GtkEntry    *entry,
 						     GdkEventKey *ev,
 						     gpointer     data)
 {
-	if (ev->keyval == GDK_Tab) {
+	if (ev->keyval == GDK_KEY_Tab) {
 		addrcompl_clear_queue();
 		_allowCommas_ = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(entry), ENTRY_DATA_ALLOW_COMMAS));
 		if( address_completion_complete_address_in_entry( entry, TRUE ) ) {
 			/* route a void character to the default handler */
 			/* this is a dirty hack; we're actually changing a key
 			 * reported by the system. */
-			ev->keyval = GDK_AudibleBell_Enable;
+			ev->keyval = GDK_KEY_AudibleBell_Enable;
 			ev->state &= ~GDK_SHIFT_MASK;
 
 			/* Create window */			
@@ -1238,16 +1274,16 @@ static gboolean address_completion_entry_key_pressed(GtkEntry    *entry,
 		else {
 			/* old behaviour */
 		}
-	} else if (ev->keyval == GDK_Shift_L
-		|| ev->keyval == GDK_Shift_R
-		|| ev->keyval == GDK_Control_L
-		|| ev->keyval == GDK_Control_R
-		|| ev->keyval == GDK_Caps_Lock
-		|| ev->keyval == GDK_Shift_Lock
-		|| ev->keyval == GDK_Meta_L
-		|| ev->keyval == GDK_Meta_R
-		|| ev->keyval == GDK_Alt_L
-		|| ev->keyval == GDK_Alt_R) {
+	} else if (ev->keyval == GDK_KEY_Shift_L
+		|| ev->keyval == GDK_KEY_Shift_R
+		|| ev->keyval == GDK_KEY_Control_L
+		|| ev->keyval == GDK_KEY_Control_R
+		|| ev->keyval == GDK_KEY_Caps_Lock
+		|| ev->keyval == GDK_KEY_Shift_Lock
+		|| ev->keyval == GDK_KEY_Meta_L
+		|| ev->keyval == GDK_KEY_Meta_R
+		|| ev->keyval == GDK_KEY_Alt_L
+		|| ev->keyval == GDK_KEY_Alt_R) {
 		/* these buttons should not clear the cache... */
 	} else
 		clear_completion_cache();
@@ -1266,7 +1302,7 @@ static gboolean address_completion_complete_address_in_entry(GtkEntry *entry,
 
 	cm_return_val_if_fail(entry != NULL, FALSE);
 
-	if (!GTK_WIDGET_HAS_FOCUS(entry)) return FALSE;
+	if (!gtk_widget_has_focus(GTK_WIDGET(entry))) return FALSE;
 
 	/* get an address component from the cursor */
 	searchTerm = get_address_from_edit( entry, &cursor_pos );
@@ -1317,6 +1353,7 @@ static void address_completion_create_completion_window( GtkEntry *entry_ )
 	GtkRequisition r;
 	GtkWidget *window;
 	GtkWidget *entry = GTK_WIDGET(entry_);
+	GdkWindow *gdkwin;
 
 	/* Create new window and list */
 	window = gtk_window_new(GTK_WINDOW_POPUP);
@@ -1340,8 +1377,13 @@ static void address_completion_create_completion_window( GtkEntry *entry_ )
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
 		GTK_SHADOW_OUT);
 	/* Use entry widget to create initial window */
-	gdk_window_get_geometry(entry->window, &x, &y, &width, &height, &depth);
-	gdk_window_get_origin (entry->window, &x, &y);
+	gdkwin = gtk_widget_get_window(entry),
+#if !GTK_CHECK_VERSION(3, 0, 0)
+	gdk_window_get_geometry(gdkwin, &x, &y, &width, &height, &depth);
+#else
+	gdk_window_get_geometry(gdkwin, &x, &y, &width, &height);
+#endif
+	gdk_window_get_origin (gdkwin, &x, &y);
 	y += height;
 	gtk_window_move(GTK_WINDOW(window), x, y);
 
@@ -1368,17 +1410,12 @@ static void address_completion_create_completion_window( GtkEntry *entry_ )
 			 "key-press-event",
 			 G_CALLBACK(completion_window_key_press),
 			 _compWindow_ );
-	gdk_pointer_grab(window->window, TRUE,
+	gdk_pointer_grab(gtk_widget_get_window(window), TRUE,
 			 GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK |
 			 GDK_BUTTON_RELEASE_MASK,
 			 NULL, NULL, GDK_CURRENT_TIME);
+	gdk_keyboard_grab(gtk_widget_get_window(window), FALSE, GDK_CURRENT_TIME);
 	gtk_grab_add( window );
-
-	/* XXX: GTK2 too??? 
-	 *
-	 * GTK1: this gets rid of the irritating focus rectangle that doesn't
-	 * follow the selection */
-	GTK_WIDGET_UNSET_FLAGS(list_view, GTK_CAN_FOCUS);
 }
 
 /**
@@ -1414,7 +1451,7 @@ static gboolean completion_window_button_press(GtkWidget *widget,
 				restore = FALSE;
 				break;
 			}
-			event_widget = event_widget->parent;
+			event_widget = gtk_widget_get_parent(event_widget);
 		}
 	}
 
@@ -1454,19 +1491,19 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 	cm_return_val_if_fail(entry != NULL, FALSE);
 
 	/* allow keyboard navigation in the alternatives tree view */
-	if (event->keyval == GDK_Up || event->keyval == GDK_Down ||
-	    event->keyval == GDK_Page_Up || event->keyval == GDK_Page_Down) {
+	if (event->keyval == GDK_KEY_Up || event->keyval == GDK_KEY_Down ||
+	    event->keyval == GDK_KEY_Page_Up || event->keyval == GDK_KEY_Page_Down) {
 		completion_window_advance_selection
 			(GTK_TREE_VIEW(list_view),
-			 event->keyval == GDK_Down ||
-			 event->keyval == GDK_Page_Down ? TRUE : FALSE);
+			 event->keyval == GDK_KEY_Down ||
+			 event->keyval == GDK_KEY_Page_Down ? TRUE : FALSE);
 		return FALSE;
 	}		
 
 	/* make tab move to next field */
-	if( event->keyval == GDK_Tab ) {
+	if( event->keyval == GDK_KEY_Tab ) {
 		/* Reference to parent */
-		parent = GTK_WIDGET(entry)->parent;
+		parent = gtk_widget_get_parent(GTK_WIDGET(entry));
 
 		/* Discard the window */
 		clear_completion_cache();
@@ -1480,9 +1517,9 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 	}
 
 	/* make backtab move to previous field */
-	if( event->keyval == GDK_ISO_Left_Tab ) {
+	if( event->keyval == GDK_KEY_ISO_Left_Tab ) {
 		/* Reference to parent */
-		parent = GTK_WIDGET(entry)->parent;
+		parent = gtk_widget_get_parent(GTK_WIDGET(entry));
 
 		/* Discard the window */
 		clear_completion_cache();
@@ -1497,17 +1534,17 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 	_allowCommas_ = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(entry), ENTRY_DATA_ALLOW_COMMAS));
 
 	/* look for presses that accept the selection */
-	if (event->keyval == GDK_Return || event->keyval == GDK_space ||
-			event->keyval == GDK_KP_Enter ||
-			(_allowCommas_ && event->keyval == GDK_comma)) {
+	if (event->keyval == GDK_KEY_Return || event->keyval == GDK_KEY_space ||
+			event->keyval == GDK_KEY_KP_Enter ||
+			(_allowCommas_ && event->keyval == GDK_KEY_comma)) {
 		/* User selected address with a key press */
 
 		/* Display selected address in entry field */		
 		completion_window_apply_selection(
 			GTK_TREE_VIEW(list_view), GTK_ENTRY(entry),
-			event->keyval != GDK_comma);
+			event->keyval != GDK_KEY_comma);
 
-		if (event->keyval == GDK_comma) {
+		if (event->keyval == GDK_KEY_comma) {
 			gint pos = gtk_editable_get_position(GTK_EDITABLE(entry));
 			gtk_editable_insert_text(GTK_EDITABLE(entry), ", ", 2, &pos);
 			gtk_editable_set_position(GTK_EDITABLE(entry), pos + 1);
@@ -1520,16 +1557,16 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 	}
 
 	/* key state keys should never be handled */
-	if (event->keyval == GDK_Shift_L
-		 || event->keyval == GDK_Shift_R
-		 || event->keyval == GDK_Control_L
-		 || event->keyval == GDK_Control_R
-		 || event->keyval == GDK_Caps_Lock
-		 || event->keyval == GDK_Shift_Lock
-		 || event->keyval == GDK_Meta_L
-		 || event->keyval == GDK_Meta_R
-		 || event->keyval == GDK_Alt_L
-		 || event->keyval == GDK_Alt_R) {
+	if (event->keyval == GDK_KEY_Shift_L
+		 || event->keyval == GDK_KEY_Shift_R
+		 || event->keyval == GDK_KEY_Control_L
+		 || event->keyval == GDK_KEY_Control_R
+		 || event->keyval == GDK_KEY_Caps_Lock
+		 || event->keyval == GDK_KEY_Shift_Lock
+		 || event->keyval == GDK_KEY_Meta_L
+		 || event->keyval == GDK_KEY_Meta_R
+		 || event->keyval == GDK_KEY_Alt_L
+		 || event->keyval == GDK_KEY_Alt_R) {
 		return FALSE;
 	}
 
@@ -1540,7 +1577,7 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 
 	/* make sure anything we typed comes in the edit box */
 	tmp_event.type       = event->type;
-	tmp_event.window     = entry->window;
+	tmp_event.window     = gtk_widget_get_window(GTK_WIDGET(entry));
 	tmp_event.send_event = TRUE;
 	tmp_event.time       = event->time;
 	tmp_event.state      = event->state;
@@ -1681,11 +1718,11 @@ static gboolean addr_compl_selected(GtkTreeSelection *selector,
 	 * not using a time out would result in a crash. if this doesn't work
 	 * safely, maybe we should set variables when receiving button presses
 	 * in the tree view. */
-	if (!window->destroying) {	 
-		window->destroying = TRUE;	 
+	if (!window->destroying) {
+		window->destroying = TRUE;
 		g_idle_add((GSourceFunc) addr_compl_defer_select_destruct, data);
-	}		
-	
+	}
+
 	return TRUE;
 }
 
