@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #define __SMTP_H__
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include <glib.h>
@@ -32,7 +32,7 @@ typedef struct _SMTPSession	SMTPSession;
 
 #define SMTP_SESSION(obj)	((SMTPSession *)obj)
 
-#define MSGBUFSIZE		8192
+#define MESSAGEBUFSIZE		8192
 
 typedef enum
 {
@@ -90,10 +90,6 @@ struct _SMTPSession
 
 	SMTPState state;
 
-#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
-	gboolean tls_init_done;
-#endif
-
 	gchar *hostname;
 
 	gchar *user;
@@ -118,9 +114,13 @@ struct _SMTPSession
 	ESMTPFlag esmtp_flags;
 	
 	void *dialog;
+
+#ifdef USE_GNUTLS
+	gboolean tls_init_done;
+#endif
 };
 
-Session *smtp_session_new	(void);
+Session *smtp_session_new	(void *prefs_account);
 gint smtp_from(SMTPSession *session);
 gint smtp_quit(SMTPSession *session);
 
