@@ -1,6 +1,6 @@
 /*
  * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2007 The Claws Mail Team
+ * Copyright (C) 2007-2012 The Claws Mail Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -90,12 +91,12 @@ static struct TagApplyWindow
 } applywindow;
 
 #define APPLYWINDOW_LOCK() {								\
-	gdk_window_set_cursor(applywindow.window->window, applywindow.watch_cursor);	\
+	gdk_window_set_cursor(gtk_widget_get_window(applywindow.window), applywindow.watch_cursor);	\
 	applywindow.busy = TRUE;							\
 }
 
 #define APPLYWINDOW_UNLOCK() {								\
-	gdk_window_set_cursor(applywindow.window->window, NULL);			\
+	gdk_window_set_cursor(gtk_widget_get_window(applywindow.window), NULL);			\
 	applywindow.busy = FALSE;							\
 }
 
@@ -197,10 +198,9 @@ static void apply_window_create_list_view_columns(GtkWidget *list_view)
 
 }
 
-static GtkItemFactory *apply_popup_factory = NULL;
 static GtkWidget *apply_popup_menu = NULL;
 
-static void apply_popup_delete (void *obj, guint action, void *data)
+static void apply_popup_delete (GtkAction *action, gpointer data)
 {
 	GtkTreeIter sel;
 	GtkTreeModel *model;
@@ -234,9 +234,9 @@ static void apply_popup_delete (void *obj, guint action, void *data)
 	APPLYWINDOW_UNLOCK();
 }
 
-static void apply_popup_delete_all (void *obj, guint action, void *data)
+static void apply_popup_delete_all (GtkAction *action, gpointer data)
 {
-	GSList *cur;
+	GSList *cur, *tags;
 	GtkTreeModel *model;
 	SummaryView *summaryview = NULL;
 	
@@ -252,27 +252,31 @@ static void apply_popup_delete_all (void *obj, guint action, void *data)
 
 	if (mainwindow_get_mainwindow() != NULL)
 		summaryview = mainwindow_get_mainwindow()->summaryview;
-	cur = tags_get_list();
+	cur = tags = tags_get_list();
 	for (; cur; cur = cur->next) {
 		gint id = GPOINTER_TO_INT(cur->data);
 		if (summaryview)
 			summary_set_tag(summaryview, -id, NULL);
 		tags_remove_tag(id);
 	}
+	g_slist_free(tags);
 	tags_write_tags();
 
 	APPLYWINDOW_UNLOCK();
 }
 
-static GtkItemFactoryEntry apply_popup_entries[] =
+static GtkActionEntry apply_popup_actions[] =
 {
-	{N_("/_Delete"),		NULL, apply_popup_delete, 0, NULL, NULL},
-	{N_("/Delete _all"),	NULL, apply_popup_delete_all, 0, NULL, NULL},
+	{ "EditTags", NULL, "EditTags" },
+	{ "EditTags/Delete", NULL, N_("_Delete"), NULL, "Delete tag", G_CALLBACK(apply_popup_delete) },
+	{ "EditTags/DeleteAll", NULL, N_("Delete _all"), NULL, "Delete all tags", G_CALLBACK(apply_popup_delete_all) }
 };
 
 static gint apply_list_btn_pressed(GtkWidget *widget, GdkEventButton *event,
 				    GtkTreeView *list_view)
 {
+	GtkUIManager *gui_manager = gtkut_ui_manager();
+
 	if (applywindow.busy)
 		return FALSE;
 
@@ -282,17 +286,23 @@ static gint apply_list_btn_pressed(GtkWidget *widget, GdkEventButton *event,
 		gboolean non_empty;
 
 		if (!apply_popup_menu) {
-			gint n_entries = sizeof(apply_popup_entries) /
-				sizeof(apply_popup_entries[0]);
-			apply_popup_menu = menu_create_items(apply_popup_entries, n_entries,
-						      "<TagPopupMenu>", &apply_popup_factory,
-						      list_view);
+			(void) cm_menu_create_action_group("EditTags", apply_popup_actions,
+					G_N_ELEMENTS(apply_popup_actions), (gpointer)list_view);
+
+			MENUITEM_ADDUI("/Menus", "EditTags", "EditTags", GTK_UI_MANAGER_MENU)
+			MENUITEM_ADDUI("/Menus/EditTags", "Delete", "EditTags/Delete",
+					GTK_UI_MANAGER_MENUITEM)
+			MENUITEM_ADDUI("/Menus/EditTags", "DeleteAll", "EditTags/DeleteAll",
+					GTK_UI_MANAGER_MENUITEM)
+
+			apply_popup_menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(
+						gtk_ui_manager_get_widget(gui_manager, "/Menus/EditTags")) );
 		}
 
 		/* grey out popup menu items if list is empty */
 		non_empty = gtk_tree_model_get_iter_first(model, &iter);
-		menu_set_sensitive(apply_popup_factory, "/Delete", non_empty);
-		menu_set_sensitive(apply_popup_factory, "/Delete all", non_empty);
+		cm_menu_set_sensitive("EditTags/Delete", non_empty);
+		cm_menu_set_sensitive("EditTags/DeleteAll", non_empty);
 
 		gtk_menu_popup(GTK_MENU(apply_popup_menu), 
 			       NULL, NULL, NULL, NULL, 
@@ -365,6 +375,7 @@ static void apply_window_close(void)
 	APPLYWINDOW_UNLOCK();
 	apply_window_list_view_clear_tags(applywindow.taglist);
 	gtk_widget_hide(applywindow.window);
+	gtk_window_set_modal(GTK_WINDOW(applywindow.window), FALSE);
 }
 
 static void apply_window_close_cb(GtkWidget *widget,
@@ -405,8 +416,14 @@ static void apply_window_add_tag(void)
 	gchar *new_tag = gtk_editable_get_chars(GTK_EDITABLE(applywindow.add_entry), 0, -1);
 	g_strstrip(new_tag);
 	if (new_tag && *new_tag) {
-		gint id = tags_get_id_for_str(new_tag);
+		gint id;
 		FindTagInStore fis;
+		if (!(IS_NOT_RESERVED_TAG(new_tag))) {
+			alertpanel_error(_("You entered a reserved tag name, please choose another instead."));
+			g_free(new_tag);
+			return;
+		}
+		id = tags_get_id_for_str(new_tag);
 		APPLYWINDOW_LOCK();
 		if (id == -1) {
 			id = tags_add_tag(new_tag);
@@ -464,7 +481,7 @@ static void apply_window_del_tag_cb(GtkWidget *widget,
 {
 	if (applywindow.busy)
 		return;
-	apply_popup_delete(NULL, 0, NULL);
+	apply_popup_delete(NULL, NULL);
 	gtk_widget_grab_focus(applywindow.taglist);
 }
 
@@ -472,10 +489,10 @@ static gboolean apply_window_key_pressed(GtkWidget *widget, GdkEventKey *event,
 {
 	if (applywindow.busy)
 		return FALSE;
-	if (event && event->keyval == GDK_Escape)
+	if (event && event->keyval == GDK_KEY_Escape)
 		apply_window_close();
-	else if (event && event->keyval == GDK_Delete)
-		apply_popup_delete(NULL, 0, NULL);
+	else if (event && event->keyval == GDK_KEY_Delete)
+		apply_popup_delete(NULL, NULL);
 	return FALSE;
 }
 
@@ -484,7 +501,7 @@ static gboolean apply_window_add_key_pressed(GtkWidget *widget, GdkEventKey *eve
 	if (applywindow.busy)
 		return FALSE;
 
-	if (event && (event->keyval == GDK_KP_Enter || event->keyval == GDK_Return)) {
+	if (event && (event->keyval == GDK_KEY_KP_Enter || event->keyval == GDK_KEY_Return)) {
 		apply_window_add_tag();
 		gtk_entry_set_text(GTK_ENTRY(applywindow.add_entry), "");
 		gtk_widget_grab_focus(applywindow.taglist);
@@ -509,7 +526,7 @@ static void apply_window_create(void)
 
 	window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "tag_apply_window");
 	gtk_window_set_title (GTK_WINDOW(window),
-			      Q_("Dialog title|Apply tags"));
+			      C_("Dialog title", "Apply tags"));
 
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
 	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
@@ -739,13 +756,14 @@ static void apply_window_list_view_insert_tag(GtkWidget *list_view,
 
 static void apply_window_load_tags (void) 
 {
-	GSList *cur;
+	GSList *cur, *tags;
 	gint id;
 	apply_window_list_view_clear_tags(applywindow.taglist);
 	
-	cur = tags_get_list();
+	cur = tags = tags_get_list();
 	for (; cur; cur = cur->next) {
 		id = GPOINTER_TO_INT(cur->data);
 		apply_window_list_view_insert_tag(applywindow.taglist, NULL, id);
 	}
+	g_slist_free(tags);
 }
