@@ -1,7 +1,6 @@
 static char *TEAM_LIST[] = {
 "Holger Berndt <berndth@users.sf.net>",
 "Tristan Chabredier <wwp@claws-mail.org>",
-"Hoà Viêt Dinh <hoa@users.sf.net>",
 "Andrej Kacian <andrej@kacian.sk>",
 "Darko Koruga <darko@users.sf.net>",
 "Ricardo Mones Lastra <ricardo@mones.org>",
@@ -14,6 +13,7 @@ NULL
 };
 
 static char *EX_TEAM_LIST[] = {
+"Hoà Viêt Dinh",
 "Keith Edmunds",
 "Match Grun",
 "Melvin Hadasht",
@@ -34,26 +34,21 @@ NULL
 
 static char *TRANS_TEAM_LIST[] = {
 "[bg] Yasen Pramatarov <yasen@lindeas.com>",
-"[ca] Miquel Oliete <ktalanet@yahoo.es>",
 "[cs] David Vachulka <david@konstrukce-cad.com>",
-"[de] Stephan Sachse <white@dev-zero.com>",
+"[de] Thomas Bellmann <ThomasBellmann@gmx.net>",
 "[en_GB] Paul Mangan <paul@claws-mail.org>",
 "[es] Ricardo Mones Lastra <ricardo@mones.org>",
 "[fi] Flammie Pirinen <flammie@iki.fi>",
 "[fr] Colin Leroy <colin@colino.net>",
 "[hu] Páder Rezső <rezso@rezso.net>",
 "[id] MSulchan Darmawan <bleketux@gmail.com>",
-"[it] Andrea Spadaccini <a.spadaccini@catania.linux.it>",
-"[ja] kazken3 <kazken3@gmail.com>",
-"[nl] Marcel Pol <mpol@gmx.net>",
-"[pl] Emilian Nowak <emil5@go2.pl>",
+"[lt] Mindaugas Baranauskas <embar@super.lt>",
 "[pt_BR] Frederico Goncalves Guimaraes <fggdebian@yahoo.com.br>",
-"[pt_PT] Tiago Faria <gouki@goukihq.org>",
 "[ru] Aleksei Miheev <aleksei@miheev.info>",
 "[sk] Slavko <slavino@slavino.sk>",
-"[sv] Lars Persson Fink <lars.p.fink@gmail.com>",
-"[uk] YUP <yupadmin@gmail.com>",
-"[zh_CN] Ralgh Young <bamanzi@gmail.com>",
+"[sv] Andreas Rönnquist <gusnan@gusnan.se>",
+"[zh_CN] Rob <rbnwmk@gmail.com>",
+"[zh_TW] Rob <rbnwmk@gmail.com>",
 NULL
 };
 
@@ -62,7 +57,6 @@ static char *DOC_TEAM_LIST[] = {
 "     Ricardo Mones Lastra <ricardo@mones.org>",
 "     Colin Leroy <colin@colino.net>",
 "     Paul Mangan <paul@claws-mail.org>",
-"[de] Volker Ribbert <thoraki@arcor.de>",
 "[es] Ricardo Mones Lastra <ricardo@mones.org>",
 "[fr] Tristan Chabredier <wwp@claws-mail.org>",
 "     Colin Leroy <colin@colino.net>",
@@ -97,6 +91,7 @@ static char *CONTRIBS_LIST[] = {
 "Pavlo Bohmat",
 "Eugene Brevdo",
 "André Filipe de Assunção e Brito",
+"Sean Buckheister",
 "Henrik Carlqvist",
 "Luca Cavalli",
 "David Chalmers",
@@ -105,10 +100,12 @@ static char *CONTRIBS_LIST[] = {
 "Tsu-Fan Cheng",
 "Manrat Chobchuen",
 "Rich Coe",
+"Natanael Copa",
 "Jonathan Corbet",
 "Christian Cornelssen",
 "George Danchev",
 "Matthieu Dazy",
+"Ben Deering",
 "Pierric Descamps",
 "Frank Dietrich",
 "Tim Dijkstra",
@@ -116,10 +113,13 @@ static char *CONTRIBS_LIST[] = {
 "Felix Eckhofer",
 "Charles A. Edwards",
 "Stefaan Eeckels",
+"Mikhail Efremov",
 "Willem van Engen",
 "Paul Evans",
 "Leonid Evdokimov",
 "Xavier FACQ",
+"Tiago Faria",
+"Lars Persson Fink",
 "Bob Forsman",
 "Matthias Förste",
 "Stoyan Genov",
@@ -128,6 +128,7 @@ static char *CONTRIBS_LIST[] = {
 "Marius Glauser",
 "Thierry Godefroy",
 "Alex Gorbachenko",
+"Bodo Graumann",
 "Yang Guilong",
 "Daniel Gustafson",
 "HIROSHIMA",
@@ -139,6 +140,7 @@ static char *CONTRIBS_LIST[] = {
 "Federico Heinz",
 "Geir Helland",
 "Jorge Van Hemelryck",
+"Christian Hesse",
 "Higuchi",
 "Hiramatu",
 "Marc Hoper",
@@ -159,10 +161,12 @@ static char *CONTRIBS_LIST[] = {
 "KUROSAWA",
 "Nicolas Kaiser",
 "Stanislav Karchebny",
+"kazken3",
 "WAKAI Kazunao",
 "Fabian Keil",
 "Martin Kluge",
 "IWAMOTO Kouichi",
+"Abhay S. Kushwaha",
 "Joshua M. Kwan",
 "Jean-Yves Lefort",
 "Jérôme Lelong",
@@ -173,6 +177,7 @@ static char *CONTRIBS_LIST[] = {
 "Alessandro Maestri",
 "Maki",
 "Jindrich Makovicka",
+"Igor Mammedov",
 "Tim Mann",
 "Ivan Francolin Martinez",
 "HIRAMATSU Masami",
@@ -182,6 +187,7 @@ static char *CONTRIBS_LIST[] = {
 "David Mehrmann",
 "Christian Mertes",
 "Bram Metsch",
+"Hanno Meyer-Thurow",
 "George Michaelson",
 "Florian Mickler",
 "Neill Miller",
@@ -199,21 +205,28 @@ static char *CONTRIBS_LIST[] = {
 "Nishiyama",
 "Masaaki Noro",
 "Ales Nosek",
+"Emilian Nowak",
 "James Noyes",
 "Jens Oberender",
 "Ofer",
 "Ohmasa",
+"Miquel Oliete",
+"Paul Ollis",
 "Fredrik Olofssen",
 "Reza Pakdel",
+"Marcel Pol",
 "Martin Pool",
 "Quar",
 "Don Quijote",
+"Steve Rader",
 "Jens Rantil",
 "Ravemax",
 "Ivan Rayner",
 "David Relson",
+"Volker Ribbert",
 "Chad Robinson",
 "Paul Rolland",
+"Andreas Rönnquist",
 "Luca Rosellini",
 "Guy Rouillier",
 "Stephan Sachse",
@@ -226,11 +239,13 @@ static char *CONTRIBS_LIST[] = {
 "Markus Schwarzenberg",
 "Rodrigo Dias Arruda Senra",
 "Setoh",
+"Michael Shell",
 "shigeri",
 "Gustavo Noronha Silva",
 "Jesse Skinner",
 "Ville Skyttä",
 "Dale P. Smith",
+"Andrea Spadaccini",
 "Bogdan Sumanariu",
 "TAKANO",
 "TOGAWA",
@@ -262,6 +277,7 @@ static char *CONTRIBS_LIST[] = {
 "Yanase",
 "Yasumatsu",
 "Ralgh Young",
+"YUP",
 "Gál Zoltán",
 "Marien Zwart",
 "Martin Zwickel",
