@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
+#include "claws-features.h"
 #endif 
 
 #include <glib.h>
@@ -168,7 +169,6 @@ static GList *filesel_create(const gchar *title, const gchar *path,
 			g_free(realpath); 
 			realpath = g_strdup(get_home_dir());
 		}
-		tmp = NULL;
 		if (g_utf8_validate(realpath, -1, NULL))
 			tmp = g_filename_from_utf8(realpath, -1, NULL, NULL, NULL);
 		if (tmp == NULL)
@@ -176,13 +176,9 @@ static GList *filesel_create(const gchar *title, const gchar *path,
 		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser), tmp);
 		g_free(tmp);
 		if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
-			tmp = NULL;
 			if (g_utf8_validate(filename, -1, NULL))
-				tmp = g_filename_from_utf8(filename, -1, NULL, NULL, NULL);
-			if (tmp == NULL)
-				tmp = g_strdup(filename);
-			gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(chooser), tmp);
-			g_free(tmp);
+				gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(chooser),
+								  filename);
 		}
 		g_free(realpath);
 	} else {
