@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto and the Claws Mail Team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include <glib.h>
@@ -232,21 +233,21 @@ static gboolean pluginwindow_key_pressed(GtkWidget *widget, GdkEventKey *event,
 {
 	if (event) {
 		switch (event->keyval) {
-			case GDK_Escape : 
-			case GDK_Return : 
-			case GDK_KP_Enter :
+			case GDK_KEY_Escape : 
+			case GDK_KEY_Return : 
+			case GDK_KEY_KP_Enter :
 				close_cb(NULL, pluginwindow);
 				break;
-			case GDK_Insert : 
-			case GDK_KP_Insert :
-			case GDK_KP_Add : 
-			case GDK_plus :
+			case GDK_KEY_Insert : 
+			case GDK_KEY_KP_Insert :
+			case GDK_KEY_KP_Add : 
+			case GDK_KEY_plus :
 				load_cb(NULL, pluginwindow);
 				break;
-			case GDK_Delete : 
-			case GDK_KP_Delete :
-			case GDK_KP_Subtract : 
-			case GDK_minus :
+			case GDK_KEY_Delete : 
+			case GDK_KEY_KP_Delete :
+			case GDK_KEY_KP_Subtract : 
+			case GDK_KEY_minus :
 				unload_cb(NULL, pluginwindow);
 				break;
 			default :
@@ -293,7 +294,6 @@ void pluginwindow_create()
 	GtkWidget *vbox3;
 	GtkWidget *hbox_info;
 	static GdkGeometry geometry;
-	CLAWS_TIP_DECL();	
 
 	debug_print("Creating plugins window...\n");
 
@@ -409,6 +409,7 @@ void pluginwindow_create()
 			   G_CALLBACK(pluginwindow_key_pressed), pluginwindow);
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 G_CALLBACK(pluginwindow_delete_cb), pluginwindow);
+	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 
 	CLAWS_SET_TIP(load_btn,
 			_("Click here to load one or more plugins"));
