@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2011 Hiroyuki Yamamoto and the Claws Mail Team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include <glib.h>
@@ -31,6 +32,7 @@
 #include "prefswindow.h"
 #include "gtkutils.h"
 #include "prefs_common.h"
+#include "gtk/manage_window.h"
 
 enum { 
 	PREFS_PAGE_TITLE,		/* page title */
@@ -222,11 +224,11 @@ static gboolean prefswindow_key_pressed(GtkWidget *widget, GdkEventKey *event,
 
 	if (event) {
 		switch (event->keyval) {
-			case GDK_Escape :
+			case GDK_KEY_Escape :
 				cancel_button_clicked(NULL, data);
 				break;
-			case GDK_Return : 
-			case GDK_KP_Enter :
+			case GDK_KEY_Return : 
+			case GDK_KEY_KP_Enter :
 				focused_child = gtkut_get_focused_child
 					(GTK_CONTAINER(data->notebook));
 				/* Press ok, if the focused child is not a text view
@@ -305,7 +307,9 @@ static void prefswindow_build_page(PrefsWindow *prefswindow, PrefsPage *page)
 
 		gtk_container_add(GTK_CONTAINER(prefswindow->notebook), scrolledwin);
 		tmp = gtk_bin_get_child(GTK_BIN(scrolledwin));
-		gtk_viewport_set_shadow_type(GTK_VIEWPORT(tmp), GTK_SHADOW_NONE);
+
+		gtk_widget_realize(tmp);
+		gtk_widget_realize(page->widget);
 		
 		page->widget	= scrolledwin;
 		page->page_open	= TRUE;
@@ -457,6 +461,8 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 	gtk_window_set_position (GTK_WINDOW(prefswindow->window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal (GTK_WINDOW (prefswindow->window), TRUE);
 	gtk_window_set_resizable (GTK_WINDOW(prefswindow->window), TRUE);
+	gtk_window_set_transient_for (GTK_WINDOW(prefswindow->window),
+			GTK_WINDOW(mainwindow_get_mainwindow()->window));
 	gtk_container_set_border_width(GTK_CONTAINER(prefswindow->window), 4);
 
 	prefswindow->vbox = gtk_vbox_new(FALSE, 6);
@@ -566,12 +572,14 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 			   G_CALLBACK(prefswindow_key_pressed), &(prefswindow->window));
 #endif
 
-	/* connect to callback only if we hhave non-NULL pointers to store size to */
+	/* connect to callback only if we have non-NULL pointers to store size to */
 	if (prefswindow->save_width && prefswindow->save_height) {
 		g_signal_connect(G_OBJECT(prefswindow->window), "size_allocate",
 				 G_CALLBACK(prefs_size_allocate_cb), prefswindow);
 	}
 
+	MANAGE_WINDOW_SIGNALS_CONNECT(prefswindow->window);
+
 	if (!geometry.min_height) {
 		
 		if (x < 800 && y < 600) {
@@ -598,7 +606,7 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 #endif
 	adj = gtk_scrolled_window_get_vadjustment(
 			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow1));
-	gtk_adjustment_set_value(adj, adj->lower);
+	gtk_adjustment_set_value(adj, gtk_adjustment_get_lower(adj));
 	gtk_adjustment_changed(adj);
 }
 
@@ -684,13 +692,12 @@ static gboolean prefswindow_row_selected(GtkTreeSelection *selector,
 	PrefsTreeNode *prefsnode;
 	PrefsPage *page;
 	PrefsWindow *prefswindow = (PrefsWindow *) data;
+	gfloat lower;
 	gchar *labeltext;
 	gint pagenum, i;
 	GtkTreeIter iter;
 	GtkAdjustment *adj;
 
-	if (gtk_tree_path_get_depth(path) < 2)
-		return FALSE;
 #ifndef GENERIC_UMPC
 	if (currently_selected) 
 		return TRUE;
@@ -728,11 +735,12 @@ static gboolean prefswindow_row_selected(GtkTreeSelection *selector,
 
 	adj = gtk_scrolled_window_get_vadjustment(
 			GTK_SCROLLED_WINDOW(page->widget));
-	gtk_adjustment_set_value(adj, adj->lower);
+	lower = gtk_adjustment_get_lower(adj);
+	gtk_adjustment_set_value(adj, lower);
 	gtk_adjustment_changed(adj);
 	adj = gtk_scrolled_window_get_hadjustment(
 			GTK_SCROLLED_WINDOW(page->widget));
-	gtk_adjustment_set_value(adj, adj->lower);
+	gtk_adjustment_set_value(adj, lower);
 	gtk_adjustment_changed(adj);
 
 #ifdef GENERIC_UMPC
