@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,16 +19,14 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkprogressbar.h>
+#include <gtk/gtk.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -60,6 +58,7 @@
 #include "filtering.h"
 #include "log.h"
 #include "hooks.h"
+#include "logwindow.h"
 
 #ifdef MAEMO
 #ifdef CHINOOK
@@ -80,16 +79,19 @@ static gboolean maemo_warned_offline = FALSE;
 #endif
 #endif
 
+extern SessionStats session_stats;
+
 static GList *inc_dialog_list = NULL;
 
-static guint inc_lock_count = 0;
+guint inc_lock_count = 0;
 
 static GdkPixbuf *currentpix;
 static GdkPixbuf *errorpix;
 static GdkPixbuf *okpix;
 
-#define MSGBUFSIZE	8192
+#define MESSAGEBUFSIZE	8192
 
+static void inc_update_stats(gint new_msgs);
 static void inc_finished		(MainWindow		*mainwin,
 					 gboolean		 new_messages,
 					 gboolean		 autocheck);
@@ -136,6 +138,8 @@ static gint inc_drop_message		(Pop3Session	*session,
 static void inc_put_error		(IncState	 istate,
 					 Pop3Session 	*session);
 
+static void inc_showlog_cb		(GtkWidget	*widget,
+					 gpointer	 data);
 static void inc_cancel_cb		(GtkWidget	*widget,
 					 gpointer	 data);
 static gint inc_dialog_delete_cb	(GtkWidget	*widget,
@@ -153,7 +157,13 @@ static gint inc_autocheck_func			(gpointer	 data);
 
 static void inc_notify_cmd		(gint new_msgs, 
  					 gboolean notify);
-	
+
+static void inc_update_stats(gint new_msgs)
+{
+	/* update session statistics */
+	session_stats.received += new_msgs;
+}
+
 /**
  * inc_finished:
  * @mainwin: Main window.
@@ -168,12 +178,12 @@ static void inc_finished(MainWindow *mainwin, gboolean new_messages, gboolean au
 	if (prefs_common.scan_all_after_inc)
 		folderview_check_new(NULL);
 
-	if (!autocheck && prefs_common.open_inbox_on_inc) {
+	if (!autocheck && new_messages && prefs_common.open_inbox_on_inc) {
 		FolderItem *item = NULL;
 
 		if (cur_account && cur_account->inbox)
 			item = folder_find_item_from_identifier(cur_account->inbox);
-		if (item == NULL && cur_account->folder)
+		if (item == NULL && cur_account && cur_account->folder)
 			item = cur_account->folder->inbox;
 		if (item == NULL)
 			item = folder_get_default_inbox();
@@ -205,6 +215,7 @@ void inc_mail(MainWindow *mainwin, gboolean notify)
 		if (execute_command_line(prefs_common.extinc_cmd, FALSE) < 0) {
 			main_window_unlock(mainwin);
 			inc_autocheck_timer_set();
+			inc_unlock();
 			return;
 		}
 	} else {
@@ -213,6 +224,7 @@ void inc_mail(MainWindow *mainwin, gboolean notify)
 			new_msgs += account_new_msgs;
 	}
 
+	inc_update_stats(new_msgs);
 	inc_finished(mainwin, new_msgs > 0, FALSE);
 	main_window_unlock(mainwin);
  	inc_notify_cmd(new_msgs, notify);
@@ -306,6 +318,7 @@ gint inc_account_mail(MainWindow *mainwin, PrefsAccount *account)
 
 	new_msgs = inc_account_mail_real(mainwin, account);
 
+	inc_update_stats(new_msgs);
 	inc_finished(mainwin, new_msgs > 0, FALSE);
 	main_window_unlock(mainwin);
 	inc_autocheck_timer_set();
@@ -334,6 +347,7 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck,
 
 	list = account_get_list();
 	if (!list) {
+		inc_update_stats(new_msgs);
 		inc_finished(mainwin, new_msgs > 0, autocheck);
 		main_window_unlock(mainwin);
  		inc_notify_cmd(new_msgs, notify);
@@ -389,6 +403,7 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck,
 		new_msgs += inc_start(inc_dialog);
 	}
 
+	inc_update_stats(new_msgs);
 	inc_finished(mainwin, new_msgs > 0, autocheck);
 	main_window_unlock(mainwin);
  	inc_notify_cmd(new_msgs, notify);
@@ -398,7 +413,7 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck,
 static void inc_progress_dialog_size_allocate_cb(GtkWidget *widget,
 					 GtkAllocation *allocation)
 {
-	g_return_if_fail(allocation != NULL);
+	cm_return_if_fail(allocation != NULL);
 
 	prefs_common.receivewin_width = allocation->width;
 	prefs_common.receivewin_height = allocation->height;
@@ -415,6 +430,8 @@ static IncProgressDialog *inc_progress_dialog_create(gboolean autocheck)
 	progress = progress_dialog_create();
 	gtk_window_set_title(GTK_WINDOW(progress->window),
 			     _("Retrieving new messages"));
+	g_signal_connect(G_OBJECT(progress->showlog_btn), "clicked",
+			 G_CALLBACK(inc_showlog_cb), dialog);
 	g_signal_connect(G_OBJECT(progress->cancel_btn), "clicked",
 			 G_CALLBACK(inc_cancel_cb), dialog);
 	g_signal_connect(G_OBJECT(progress->window), "delete_event",
@@ -487,7 +504,7 @@ static void inc_progress_dialog_clear(IncProgressDialog *inc_dialog)
 
 static void inc_progress_dialog_destroy(IncProgressDialog *inc_dialog)
 {
-	g_return_if_fail(inc_dialog != NULL);
+	cm_return_if_fail(inc_dialog != NULL);
 
 	inc_dialog_list = g_list_remove(inc_dialog_list, inc_dialog);
 
@@ -502,7 +519,7 @@ static IncSession *inc_session_new(PrefsAccount *account)
 {
 	IncSession *session;
 
-	g_return_val_if_fail(account != NULL, NULL);
+	cm_return_val_if_fail(account != NULL, NULL);
 
 	if (account->protocol != A_POP3)
 		return NULL;
@@ -527,7 +544,7 @@ static IncSession *inc_session_new(PrefsAccount *account)
 
 static void inc_session_destroy(IncSession *session)
 {
-	g_return_if_fail(session != NULL);
+	cm_return_if_fail(session != NULL);
 
 	session_destroy(session->session);
 	g_free(session);
@@ -557,9 +574,6 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		if (pop3_session->ac_prefs->passwd)
 			pop3_session->pass =
 				g_strdup(pop3_session->ac_prefs->passwd);
-		else if (pop3_session->ac_prefs->tmp_pass)
-			pop3_session->pass =
-				g_strdup(pop3_session->ac_prefs->tmp_pass);
 		else {
 			gchar *pass;
 
@@ -568,9 +582,10 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 					(inc_dialog->dialog->window,
 					 NULL, NULL);
 
-			pass = input_dialog_query_password
+			pass = input_dialog_query_password_keep
 				(pop3_session->ac_prefs->recv_server,
-				 pop3_session->user);
+				 pop3_session->user,
+				 &(pop3_session->ac_prefs->session_passwd));
 
 			if (inc_dialog->show_dialog)
 				manage_window_focus_out
@@ -578,8 +593,6 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 					 NULL, NULL);
 
 			if (pass) {
-				pop3_session->ac_prefs->tmp_pass =
-					g_strdup(pass);
 				pop3_session->pass = pass;
 			}
 		}
@@ -626,7 +639,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 						 "Done (%d messages (%s) received)",
 					 pop3_session->cur_total_num),
 					 pop3_session->cur_total_num,
-					 to_human_readable(pop3_session->cur_total_recv_bytes));
+					 to_human_readable((goffset)pop3_session->cur_total_recv_bytes));
 			else
 				msg = g_strdup_printf(_("Done (no new messages)"));
 			SET_PIXMAP_AND_TEXT(okpix, msg);
@@ -637,6 +650,10 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 			break;
 		case INC_AUTH_FAILED:
 			SET_PIXMAP_AND_TEXT(errorpix, _("Auth failed"));
+			if (pop3_session->ac_prefs->session_passwd) {
+				g_free(pop3_session->ac_prefs->session_passwd);
+				pop3_session->ac_prefs->session_passwd = NULL;
+			}
 			break;
 		case INC_LOCKED:
 			SET_PIXMAP_AND_TEXT(errorpix, _("Locked"));
@@ -709,12 +726,6 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 
 		new_msgs += pop3_session->cur_total_num;
 
-		if (pop3_session->error_val == PS_AUTHFAIL &&
-		    pop3_session->ac_prefs->tmp_pass) {
-			g_free(pop3_session->ac_prefs->tmp_pass);
-			pop3_session->ac_prefs->tmp_pass = NULL;
-		}
-
 		pop3_write_uidl_list(pop3_session);
 
 		if (inc_state != INC_SUCCESS && inc_state != INC_CANCEL) {
@@ -776,6 +787,7 @@ static IncState inc_pop3_session_do(IncSession *session)
 	Pop3Session *pop3_session = POP3_SESSION(session->session);
 	IncProgressDialog *inc_dialog = (IncProgressDialog *)session->data;
 	gchar *server;
+	gchar *account_name;
 	gushort port;
 	gchar *buf;
 
@@ -790,7 +802,8 @@ static IncState inc_pop3_session_do(IncSession *session)
 	g_free(buf);
 
 	server = pop3_session->ac_prefs->recv_server;
-#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
+	account_name = pop3_session->ac_prefs->account_name;
+#ifdef USE_GNUTLS
 	port = pop3_session->ac_prefs->set_popport ?
 		pop3_session->ac_prefs->popport :
 		pop3_session->ac_prefs->ssl_pop == SSL_TUNNEL ? 995 : 110;
@@ -816,7 +829,8 @@ static IncState inc_pop3_session_do(IncSession *session)
 		pop3_session->ac_prefs->popport : 110;
 #endif
 
-	buf = g_strdup_printf(_("Connecting to POP3 server: %s..."), server);
+	buf = g_strdup_printf(_("Account '%s': Connecting to POP3 server: %s:%d..."),
+				account_name, server, port);
 	statuswindow_print_all("%s", buf);
 	log_message(LOG_PROTOCOL, "%s\n", buf);
 
@@ -845,7 +859,7 @@ static IncState inc_pop3_session_do(IncSession *session)
 		return INC_CONNECT_ERROR;
 	}
 
-	while (session_is_connected(SESSION(pop3_session)) &&
+	while (session_is_running(SESSION(pop3_session)) &&
 	       session->inc_state != INC_CANCEL)
 		gtk_main_iteration();
 
@@ -907,7 +921,7 @@ static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 	ProgressDialog *dialog = inc_dialog->dialog;
 	Pop3Session *session;
 
-	g_return_if_fail(inc_session != NULL);
+	cm_return_if_fail(inc_session != NULL);
 
 	session = POP3_SESSION(inc_session->session);
 
@@ -919,7 +933,7 @@ static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 	case POP3_GETAUTH_APOP:
 		progress_dialog_set_label(dialog, _("Authenticating..."));
 		statuswindow_pop_all();
-		statusbar_print_all(_("Retrieving messages from %s (%s) ..."),
+		statusbar_print_all(_("Retrieving messages from %s (%s)..."),
 				    SESSION(session)->server,
 				    session->ac_prefs->account_name);
 		break;
@@ -954,7 +968,7 @@ static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 					     IncSession *inc_session)
 {
-	gchar buf[MSGBUFSIZE];
+	gchar buf[MESSAGEBUFSIZE];
 	Pop3Session *pop3_session = POP3_SESSION(inc_session->session);
 	gchar *total_size_str;
 	gint cur_total;
@@ -967,24 +981,18 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 	if (pop3_session->state == POP3_RETR ||
 	    pop3_session->state == POP3_RETR_RECV ||
 	    pop3_session->state == POP3_DELETE) {
-		Xstrdup_a(total_size_str, to_human_readable(total), return);
+		Xstrdup_a(total_size_str, to_human_readable((goffset)total), return);
 		g_snprintf(buf, sizeof(buf),
 			   _("Retrieving message (%d / %d) (%s / %s)"),
 			   pop3_session->cur_msg, pop3_session->count,
-			   to_human_readable(cur_total), total_size_str);
+			   to_human_readable((goffset)cur_total), total_size_str);
 		progress_dialog_set_label(inc_dialog->dialog, buf);
 	}
 
 	progress_dialog_set_fraction
 		(inc_dialog->dialog, (total == 0) ? 0: (gfloat)cur_total / (gfloat)total);
 
-	g_snprintf(buf, sizeof(buf), "%d / %d",
-		   pop3_session->cur_msg, pop3_session->count);
-	gtk_progress_bar_set_text
-		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar), buf);
-	gtk_progress_bar_set_fraction
-		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar),
-		 (total == 0) ? 0 : (gfloat)cur_total / (gfloat)total);
+	statusbar_progress_all(pop3_session->cur_msg, pop3_session->count, 1);
 
 	if (pop3_session->cur_total_num > 0) {
 		g_snprintf(buf, sizeof(buf),
@@ -993,7 +1001,7 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 				    pop3_session->cur_total_num),
 			   pop3_session->cur_total_num,
 			   to_human_readable
-			   (pop3_session->cur_total_recv_bytes));
+			   ((goffset)pop3_session->cur_total_recv_bytes));
 		progress_dialog_list_set_status(inc_dialog->dialog,
 						inc_dialog->cur_row,
 						buf);
@@ -1032,7 +1040,7 @@ static gint inc_recv_data_progressive(Session *session, guint cur_len,
 	IncProgressDialog *inc_dialog;
 	gint cur_total;
 
-	g_return_val_if_fail(inc_session != NULL, -1);
+	cm_return_val_if_fail(inc_session != NULL, -1);
 
 	if (pop3_session->state != POP3_RETR &&
 	    pop3_session->state != POP3_RETR_RECV &&
@@ -1057,7 +1065,7 @@ static gint inc_recv_data_finished(Session *session, guint len, gpointer data)
 	IncSession *inc_session = (IncSession *)data;
 	IncProgressDialog *inc_dialog;
 
-	g_return_val_if_fail(inc_session != NULL, -1);
+	cm_return_val_if_fail(inc_session != NULL, -1);
 
 	inc_dialog = (IncProgressDialog *)inc_session->data;
 
@@ -1075,7 +1083,7 @@ static gint inc_recv_message(Session *session, const gchar *msg, gpointer data)
 	IncSession *inc_session = (IncSession *)data;
 	IncProgressDialog *inc_dialog;
 
-	g_return_val_if_fail(inc_session != NULL, -1);
+	cm_return_val_if_fail(inc_session != NULL, -1);
 
 	inc_dialog = (IncProgressDialog *)inc_session->data;
 
@@ -1109,7 +1117,7 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	IncSession *inc_session = (IncSession *)(SESSION(session)->data);
 	gint msgnum;
 
-	g_return_val_if_fail(inc_session != NULL, -1);
+	cm_return_val_if_fail(inc_session != NULL, -1);
 
 	if (session->ac_prefs->inbox) {
 		inbox = folder_find_item_from_identifier
@@ -1119,7 +1127,7 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	} else
 		inbox = folder_get_default_inbox();
 	if (!inbox) {
-		g_unlink(file);
+		claws_unlink(file);
 		return -1;
 	}
 
@@ -1129,7 +1137,7 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	/* add msg file to drop folder */
 	if ((msgnum = folder_item_add_msg(
 			dropfolder, file, NULL, TRUE)) < 0) {
-		g_unlink(file);
+		claws_unlink(file);
 		return -1;
 	}
 
@@ -1144,7 +1152,6 @@ static void inc_put_error(IncState istate, Pop3Session *session)
 
 	switch (istate) {
 	case INC_CONNECT_ERROR:
-		log_msg = _("Connection failed.");
 		fatal_error = TRUE;
 		if (prefs_common.no_recv_err_panel)
 			break;
@@ -1212,7 +1219,9 @@ static void inc_put_error(IncState istate, Pop3Session *session)
 			err_msg = g_strdup(log_msg);
 		break;
 	case INC_TIMEOUT:
-		log_msg = _("Session timed out.");
+		log_msg = _("Session timed out. You may be able to "
+			    "recover by increasing the timeout value in "
+			    "Preferences/Other/Miscellaneous.");
 		if (prefs_common.no_recv_err_panel)
 			break;
 		err_msg = g_strdup_printf(_("Connection to %s:%d timed out."), 
@@ -1242,7 +1251,7 @@ static void inc_cancel(IncProgressDialog *dialog)
 {
 	IncSession *session;
 
-	g_return_if_fail(dialog != NULL);
+	cm_return_if_fail(dialog != NULL);
 
 	if (dialog->queue_list == NULL) {
 		inc_progress_dialog_destroy(dialog);
@@ -1269,6 +1278,13 @@ void inc_cancel_all(void)
 		inc_cancel((IncProgressDialog *)cur->data);
 }
 
+static void inc_showlog_cb(GtkWidget *widget, gpointer data)
+{
+	MainWindow *mainwin = mainwindow_get_mainwindow();
+
+	log_window_show(mainwin->logwin);
+}
+
 static void inc_cancel_cb(GtkWidget *widget, gpointer data)
 {
 	inc_cancel((IncProgressDialog *)data);
@@ -1351,9 +1367,9 @@ static gint get_spool(FolderItem *dest, const gchar *mbox, PrefsAccount *account
 	gint lockfd;
 	gchar tmp_mbox[MAXPATHLEN + 1];
 
-	g_return_val_if_fail(dest != NULL, -1);
-	g_return_val_if_fail(mbox != NULL, -1);
-	g_return_val_if_fail(account != NULL, -1);
+	cm_return_val_if_fail(dest != NULL, -1);
+	cm_return_val_if_fail(mbox != NULL, -1);
+	cm_return_val_if_fail(account != NULL, -1);
 
 	if (!is_file_exist(mbox) || (size = get_file_size(mbox)) == 0) {
 		debug_print("%s: no messages in local mailbox.\n", mbox);
@@ -1377,19 +1393,19 @@ static gint get_spool(FolderItem *dest, const gchar *mbox, PrefsAccount *account
 
 	msgs = proc_mbox(dest, tmp_mbox, account->filter_on_recv, account);
 
-	g_unlink(tmp_mbox);
+	claws_unlink(tmp_mbox);
 	if (msgs >= 0) empty_mbox(mbox);
 	unlock_mbox(mbox, lockfd, LOCK_FLOCK);
 
 	return msgs;
 }
 
-void inc_lock(void)
+void inc_lock_real(void)
 {
 	inc_lock_count++;
 }
 
-void inc_unlock(void)
+void inc_unlock_real(void)
 {
 	if (inc_lock_count > 0)
 		inc_lock_count--;
@@ -1404,7 +1420,6 @@ osso_context_t *get_osso_context(void);
 
 static void inc_notify_cmd(gint new_msgs, gboolean notify)
 {
-
 #ifndef MAEMO
 	gchar *buf, *numpos, *ret_str;
 	gssize by_read = 0, by_written = 0;
@@ -1501,6 +1516,12 @@ static void inc_autocheck_timer_set_interval(guint interval)
 	   the common preferences */
 	if (prefs_common.autochk_newmail && autocheck_data
 	    && prefs_common.work_offline == FALSE) {
+#if GLIB_CHECK_VERSION(2,14,0)
+		if (interval % 1000 == 0)
+			autocheck_timer =
+				g_timeout_add_seconds(interval/1000, inc_autocheck_func, autocheck_data);
+		else
+#endif
 		autocheck_timer = g_timeout_add
 			(interval, inc_autocheck_func, autocheck_data);
 		debug_print("added timer = %d\n", autocheck_timer);
