@@ -1,10 +1,10 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2003-2007 Match Grun and the Claws Mail team
+ * Copyright (C) 2003-2012 Match Grun and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -23,6 +23,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #ifdef USE_LDAP
@@ -30,8 +31,6 @@
 #include <glib.h>
 #include <sys/time.h>
 #include <string.h>
-#include <ldap.h>
-#include <lber.h>
 
 #include "mgutils.h"
 #include "addritem.h"
@@ -66,7 +65,6 @@ LdapServer *ldapsvr_create_noctl( void ) {
  */
 LdapServer *ldapsvr_create( void ) {
 	LdapServer *server;
-
 	server = ldapsvr_create_noctl();
 	server->control = ldapctl_create();
 	return server;
@@ -78,7 +76,7 @@ LdapServer *ldapsvr_create( void ) {
  * \return Name for server.
  */
 gchar *ldapsvr_get_name( LdapServer *server ) {
-	g_return_val_if_fail( server != NULL, NULL );
+	cm_return_val_if_fail( server != NULL, NULL );
 	return addrcache_get_name( server->addressCache );
 }
 
@@ -88,8 +86,9 @@ gchar *ldapsvr_get_name( LdapServer *server ) {
  * \param value      Name for server.
  */
 void ldapsvr_set_name( LdapServer* server, const gchar *value ) {
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 	addrcache_set_name( server->addressCache, value );
+	debug_print("setting name: %s\n", value?value:"null");
 }
 
 /**
@@ -97,6 +96,7 @@ void ldapsvr_set_name( LdapServer* server, const gchar *value ) {
  * \param server Server object.
  */
 void ldapsvr_force_refresh( LdapServer *server ) {
+	cm_return_if_fail( server != NULL );
 	addrcache_refresh( server->addressCache );
 }
 
@@ -106,7 +106,7 @@ void ldapsvr_force_refresh( LdapServer *server ) {
  * \return Status/error code.
  */
 gint ldapsvr_get_status( LdapServer *server ) {
-	g_return_val_if_fail( server != NULL, -1 );
+	cm_return_val_if_fail( server != NULL, -1 );
 	return server->retVal;
 }
 
@@ -116,11 +116,11 @@ gint ldapsvr_get_status( LdapServer *server ) {
  * \return Root level folder.
  */
 ItemFolder *ldapsvr_get_root_folder( LdapServer *server ) {
-	g_return_val_if_fail( server != NULL, NULL );
+	cm_return_val_if_fail( server != NULL, NULL );
 	/*
-	printf( "ldapsvr_get_root_folder/start\n" );
+	g_print( "ldapsvr_get_root_folder/start\n" );
 	ldapsvr_print_data( server, stdout );
-	printf( "ldapsvr_get_root_folder/done\n" );
+	g_print( "ldapsvr_get_root_folder/done\n" );
 	*/
 	return addrcache_get_root_folder( server->addressCache );
 }
@@ -131,7 +131,7 @@ ItemFolder *ldapsvr_get_root_folder( LdapServer *server ) {
  * \return <i>TRUE</i> if data was accessed.
  */
 gboolean ldapsvr_get_accessed( LdapServer *server ) {
-	g_return_val_if_fail( server != NULL, FALSE );
+	cm_return_val_if_fail( server != NULL, FALSE );
 	return server->addressCache->accessFlag;
 }
 
@@ -141,8 +141,9 @@ gboolean ldapsvr_get_accessed( LdapServer *server ) {
  * \param value      Value for flag.
  */
 void ldapsvr_set_accessed( LdapServer *server, const gboolean value ) {
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 	server->addressCache->accessFlag = value;
+	debug_print("setting accessFlag: %d\n", value);
 }
 
 /**
@@ -151,7 +152,7 @@ void ldapsvr_set_accessed( LdapServer *server, const gboolean value ) {
  * \return <i>TRUE</i> if data was modified.
  */
 gboolean ldapsvr_get_modified( LdapServer *server ) {
-	g_return_val_if_fail( server != NULL, FALSE );
+	cm_return_val_if_fail( server != NULL, FALSE );
 	return server->addressCache->modified;
 }
 
@@ -161,8 +162,9 @@ gboolean ldapsvr_get_modified( LdapServer *server ) {
  * \param value      Value for flag.
  */
 void ldapsvr_set_modified( LdapServer *server, const gboolean value ) {
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 	server->addressCache->modified = value;
+	debug_print("setting modified: %d\n", value);
 }
 
 /**
@@ -171,7 +173,7 @@ void ldapsvr_set_modified( LdapServer *server, const gboolean value ) {
  * \return <i>TRUE</i> if data was read.
  */
 gboolean ldapsvr_get_read_flag( LdapServer *server ) {
-	g_return_val_if_fail( server != NULL, FALSE );
+	cm_return_val_if_fail( server != NULL, FALSE );
 	return server->addressCache->dataRead;
 }
 
@@ -181,7 +183,7 @@ gboolean ldapsvr_get_read_flag( LdapServer *server ) {
  * \return <i>TRUE</i> if server is used for dynamic searches.
  */
 gboolean ldapsvr_get_search_flag( LdapServer *server ) {
-	g_return_val_if_fail( server != NULL, FALSE );
+	cm_return_val_if_fail( server != NULL, FALSE );
 	return server->searchFlag;
 }
 
@@ -191,8 +193,9 @@ gboolean ldapsvr_get_search_flag( LdapServer *server ) {
  * \param value      Name for server.
  */
 void ldapsvr_set_search_flag( LdapServer *server, const gboolean value ) {
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 	server->searchFlag = value;
+	debug_print("setting searchFlag: %d\n", value);
 }
 
 /**
@@ -202,7 +205,7 @@ void ldapsvr_set_search_flag( LdapServer *server, const gboolean value ) {
  * \param ctl    Control data.
  */
 void ldapsvr_set_control( LdapServer *server, LdapControl *ctl ) {
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 	addrcache_refresh( server->addressCache );
 	server->control = ctl;
 }
@@ -213,7 +216,7 @@ void ldapsvr_set_control( LdapServer *server, LdapControl *ctl ) {
  */
 void ldapsvr_free_all_query( LdapServer *server ) {
 	GList *node;	
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 
 	node = server->listQuery;
 	while( node ) {
@@ -232,8 +235,8 @@ void ldapsvr_free_all_query( LdapServer *server ) {
  * \param qry    Query object.
  */
 void ldapsvr_add_query( LdapServer *server, LdapQuery *qry ) {
-	g_return_if_fail( server != NULL );
-	g_return_if_fail( qry != NULL );
+	cm_return_if_fail( server != NULL );
+	cm_return_if_fail( qry != NULL );
 
 	server->listQuery = g_list_append( server->listQuery, qry );
 	qry->server = server;
@@ -244,7 +247,7 @@ void ldapsvr_add_query( LdapServer *server, LdapQuery *qry ) {
  * \param server Server object.
  */
 void ldapsvr_free( LdapServer *server ) {
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 
 	/* Stop and cancel any queries that may be active */
 	ldapsvr_stop_all_query( server );
@@ -281,7 +284,7 @@ void ldapsvr_print_data( LdapServer *server, FILE *stream ) {
 	GList *node;
 	gint  i;
 
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 
 	fprintf( stream, "LdapServer:\n" );
 	fprintf( stream, "  ret val: %d\n", server->retVal );
@@ -313,7 +316,7 @@ void ldapsvr_print_data( LdapServer *server, FILE *stream ) {
  * \return List of persons.
  */
 GList *ldapsvr_get_list_person( LdapServer *server ) {
-	g_return_val_if_fail( server != NULL, NULL );
+	cm_return_val_if_fail( server != NULL, NULL );
 	return addrcache_get_list_person( server->addressCache );
 }
 
@@ -324,7 +327,7 @@ GList *ldapsvr_get_list_person( LdapServer *server ) {
  * \return List of folders.
  */
 GList *ldapsvr_get_list_folder( LdapServer *server ) {
-	g_return_val_if_fail( server != NULL, NULL );
+	cm_return_val_if_fail( server != NULL, NULL );
 	/* return addrcache_get_list_folder( server->addressCache ); */
 	return NULL;
 }
@@ -337,8 +340,8 @@ GList *ldapsvr_get_list_folder( LdapServer *server ) {
 void ldapsvr_execute_query( LdapServer *server, LdapQuery *qry ) {
 	LdapControl *ctlCopy;
 
-	g_return_if_fail( server != NULL );
-	g_return_if_fail( qry != NULL );
+	cm_return_if_fail( server != NULL );
+	cm_return_if_fail( qry != NULL );
 
 	/* Copy server's control data to the query */
 	ctlCopy = ldapctl_create();
@@ -347,17 +350,15 @@ void ldapsvr_execute_query( LdapServer *server, LdapQuery *qry ) {
 	ldapqry_initialize();
 
 	/* Perform query */	
-	/* printf( "ldapsvr_execute_query::checking query...\n" ); */
+	debug_print("ldapsvr_execute_query::checking query...\n");
 	if( ldapqry_check_search( qry ) ) {
-		/* printf( "ldapsvr_execute_query::reading with thread...\n" ); */
+		debug_print("ldapsvr_execute_query::reading with thread...\n");
 		ldapqry_read_data_th( qry );
-		/*
-		if( qry->retVal == LDAPRC_SUCCESS ) {
-			printf( "ldapsvr_execute_query::SUCCESS with thread...\n" );
+		if(qry->server->retVal == LDAPRC_SUCCESS) {
+			debug_print("ldapsvr_execute_query::SUCCESS with thread...\n");
 		}
-		*/
 	}
-	/* printf( "ldapsvr_execute_query... terminated\n" ); */
+	debug_print("ldapsvr_execute_query... terminated\n");
 }
 
 /**
@@ -367,7 +368,7 @@ void ldapsvr_execute_query( LdapServer *server, LdapQuery *qry ) {
  */
 void ldapsvr_stop_query_id( LdapServer *server, const gint queryID ) {
 	GList *node;	
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 
 	node = server->listQuery;
 	while( node ) {
@@ -386,7 +387,7 @@ void ldapsvr_stop_query_id( LdapServer *server, const gint queryID ) {
  */
 void ldapsvr_stop_all_query( LdapServer *server ) {
 	GList *node;	
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 
 	node = server->listQuery;
 	while( node ) {
@@ -402,7 +403,7 @@ void ldapsvr_stop_all_query( LdapServer *server ) {
  */
 void ldapsvr_cancel_all_query( LdapServer *server ) {
 	GList *node;	
-	g_return_if_fail( server != NULL );
+	cm_return_if_fail( server != NULL );
 
 	node = server->listQuery;
 	while( node ) {
@@ -428,7 +429,7 @@ static LdapQuery *ldapsvr_locate_query(
 {
 	LdapQuery *incomplete = NULL;
 	GList *node;	
-	g_return_val_if_fail( server != NULL, NULL );
+	cm_return_val_if_fail( server != NULL, NULL );
 
 	node = server->listQuery;
 	node = g_list_last( node );
@@ -467,8 +468,8 @@ void ldapsvr_retire_query( LdapServer *server ) {
 	LdapControl *ctl;
 	ItemFolder *folder;
 
-	/* printf( "ldapsvr_retire_query\n" ); */
-	g_return_if_fail( server != NULL );
+	debug_print("ldapsvr_retire_query\n");
+	cm_return_if_fail( server != NULL );
 	ctl = server->control;
 	maxAge = ctl->maxQueryAge;
 
@@ -489,9 +490,8 @@ void ldapsvr_retire_query( LdapServer *server ) {
 		ldapqry_age( qry, maxAge );
 		if( qry->agedFlag ) {
 			/* Delete folder associated with query */
-			/*
-			printf( "deleting folder... ::%s::\n", ADDRQUERY_NAME(qry) );
-			*/
+			debug_print("deleting folder... ::%s::\n",
+					ADDRQUERY_NAME(qry)?ADDRQUERY_NAME(qry):"null");
 			ldapqry_delete_folder( qry );
 			listDelete = g_list_append( listDelete, qry );
 		}
@@ -568,8 +568,8 @@ gboolean ldapsvr_reuse_previous( const LdapServer *server, const QueryRequest *r
 	gchar *searchTerm;
 	ItemFolder *folder;
 
-	g_return_val_if_fail( server != NULL, FALSE );
-	g_return_val_if_fail( req != NULL, FALSE );
+	cm_return_val_if_fail( server != NULL, FALSE );
+	cm_return_val_if_fail( req != NULL, FALSE );
 
 	searchTerm = req->searchTerm;
 
@@ -602,8 +602,8 @@ LdapQuery *ldapsvr_new_dynamic_search( LdapServer *server, QueryRequest *req )
 	gchar *searchTerm;
 	ItemFolder *folder;
 
-	g_return_val_if_fail( server != NULL, NULL );
-	g_return_val_if_fail( req != NULL, NULL );
+	cm_return_val_if_fail( server != NULL, NULL );
+	cm_return_val_if_fail( req != NULL, NULL );
 
 	/* Retire any aged queries */
 	/* // ldapsvr_retire_query( server ); */
@@ -660,9 +660,9 @@ LdapQuery *ldapsvr_new_explicit_search(
 	gchar *searchTerm;
 	gchar *name;
 
-	g_return_val_if_fail( server != NULL, NULL );
-	g_return_val_if_fail( req != NULL, NULL );
-	g_return_val_if_fail( folder != NULL, NULL );
+	cm_return_val_if_fail( server != NULL, NULL );
+	cm_return_val_if_fail( req != NULL, NULL );
+	cm_return_val_if_fail( folder != NULL, NULL );
 
 	/* Retire any aged queries */
 	/* // ldapsvr_retire_query( server ); */
@@ -698,11 +698,16 @@ LdapQuery *ldapsvr_new_explicit_search(
 
 gint ldapsvr_read_data( LdapServer *server )
 {
-	g_return_val_if_fail( server != NULL, -1 );
+	gchar *name;
 
-	//printf( "...addrbook_read_data :%s:\n", addrcache_get_name( server->addressCache ) );
+	cm_return_val_if_fail( server != NULL, -1 );
+
+	name = addrcache_get_name(server->addressCache);
+	debug_print("...addrbook_read_data :%s:\n", name?name:"null");
 	
 	addrcache_clear(server->addressCache);
+	ldapsvr_free_all_query( server );
+	server->listQuery = NULL;
 	server->addressCache->modified = FALSE;
 	server->addressCache->accessFlag = FALSE;
 	server->addressCache->dataRead = TRUE;
@@ -710,6 +715,167 @@ gint ldapsvr_read_data( LdapServer *server )
 	return 0;
 }
 
+void ldapsrv_set_options (gint secs, LDAP *ld)
+{
+	static struct timeval timeout;
+	int rc;
+	int i;
+	timeout.tv_sec = secs;
+	timeout.tv_usec = 0;
+#ifdef G_OS_UNIX
+	i = LDAP_OPT_X_TLS_ALLOW;
+	rc = ldap_set_option(NULL, LDAP_OPT_X_TLS_REQUIRE_CERT, &i);
+	if (ld)
+		debug_print("cert %s\n", ldaputil_get_error(ld));
+	else
+		debug_print("cert %s\n", ldap_err2string(rc));
+	/* can crash old libldaps... */
+	rc = ldap_set_option(NULL, LDAP_OPT_NETWORK_TIMEOUT, &timeout);
+	if (ld)
+		debug_print("tm %s\n", ldaputil_get_error(ld));
+	else
+		debug_print("tm %s\n", ldap_err2string(rc));
+#endif
+}
+
+#ifdef G_OS_WIN32
+#if LDAP_UNICODE
+#define LDAP_START_TLS_S "ldap_start_tls_sW"
+typedef ULONG (* PFldap_start_tls_s) (LDAP *, PULONG, LDAPMessage **, PLDAPControlW *, PLDAPControlW *);
+#else
+#define LDAP_START_TLS_S "ldap_start_tls_sA"
+typedef ULONG (* PFldap_start_tls_s) (LDAP *, PULONG, LDAPMessage **, PLDAPControlA *, PLDAPControlA *);
+#endif /* LDAP_UNICODE */
+PFldap_start_tls_s Win32_ldap_start_tls_s = NULL;
+#endif
+
+/**
+ * Connect to LDAP server.
+ * \param  ctl Control object to process.
+ * \return LDAP Resource to LDAP.
+ */
+LDAP *ldapsvr_connect(LdapControl *ctl) {
+	LDAP *ld = NULL;
+	gint rc;
+	gint version;
+	gchar *uri = NULL;
+	gchar *pwd;
+
+	cm_return_val_if_fail(ctl != NULL, NULL);
+
+	ldapsrv_set_options (ctl->timeOut, NULL);
+	uri = g_strdup_printf("ldap%s://%s:%d",
+				ctl->enableSSL?"s":"",
+				ctl->hostName, ctl->port);
+#ifdef G_OS_UNIX
+	ldap_initialize(&ld, uri);
+#else
+	ld = ldap_sslinit(ctl->hostName, ctl->port, ctl->enableSSL);
+	if (ld && ctl->enableSSL) {
+		version = LDAP_VERSION3;
+		debug_print("Setting version 3\n");
+		rc = ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, (void *)&version);
+		if (rc == LDAP_SUCCESS)
+			ctl->version = LDAP_VERSION3;
+		else
+			debug_print("Failed: %s\n", ldaputil_get_error(ld));
+
+		if (ldap_get_option(ld,LDAP_OPT_SSL,(void*)&rc) != LDAP_SUCCESS)
+			debug_print("Can't get SSL state\n");
+
+		if ((void *)rc != LDAP_OPT_ON) {
+			debug_print("Enabling SSL\n");
+			if (ldap_set_option(ld,LDAP_OPT_SSL,LDAP_OPT_ON) != LDAP_SUCCESS)
+				debug_print("Failed: %s\n", ldaputil_get_error(ld));
+			else {
+				ldap_get_option(ld,LDAP_OPT_SSL,(void*)&rc);
+				debug_print("SSL now %d\n", rc);
+			}
+
+		}
+		if (!ld || (rc = ldap_connect(ld, NULL)) != LDAP_SUCCESS)
+			debug_print("ldap_connect failed: %d %s\n", rc, ldaputil_get_error(ld));
+	}
+#endif
+	g_free(uri);
+
+	if (ld == NULL)
+		return NULL;
+
+
+	debug_print("Got handle to LDAP host %s on port %d\n", ctl->hostName, ctl->port);
+
+	version = LDAP_VERSION3;
+	debug_print("Setting version 3\n");
+	rc = ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version);
+	if (rc == LDAP_OPT_SUCCESS) {
+		ctl->version = LDAP_VERSION3;
+	} else
+		g_printerr("LDAP: Error %d (%s)\n",
+			rc, ldaputil_get_error(ld));
+
+#if (defined USE_LDAP_TLS || defined G_OS_WIN32)
+	/* Handle TLS */
+	if (ctl->version == LDAP_VERSION3) {
+		if (ctl->enableTLS && !ctl->enableSSL) {
+#ifdef G_OS_WIN32
+			ULONG serv_rc;
+			if (Win32_ldap_start_tls_s == NULL) {
+				void *lib = LoadLibrary("wldap32.dll");
+				if (!lib || (Win32_ldap_start_tls_s = (PFldap_start_tls_s) GetProcAddress(lib, LDAP_START_TLS_S)) == NULL) {
+					g_printerr("LDAP Error(tls): ldap_start_tls_s: not supported on this platform");
+					if (lib)
+						FreeLibrary(lib);
+					return NULL;
+				}
+			}
+			debug_print("Setting TLS\n");
+			rc = Win32_ldap_start_tls_s(ld, &serv_rc, NULL, NULL, NULL);
+			debug_print("ldap_start_tls_s: %d server %d %s\n",
+					rc, serv_rc, ldaputil_get_error(ld));
+#else
+			debug_print("Setting TLS\n");
+			rc = ldap_start_tls_s(ld, NULL, NULL);
+#endif
+			if (rc != LDAP_SUCCESS) {
+				g_printerr("LDAP Error(tls): ldap_start_tls_s: %d %s\n",
+					rc, ldaputil_get_error(ld));
+				return NULL;
+			} else {
+				debug_print("Done\n");
+			}
+		}
+	}
+#endif
+
+	/* Bind to the server, if required */
+	if (ctl->bindDN) {
+		if (* ctl->bindDN != '\0') {
+			pwd = ldapctl_get_bind_password(ctl);
+			rc = claws_ldap_simple_bind_s(ld, ctl->bindDN, pwd);
+			if (rc != LDAP_SUCCESS) {
+				g_printerr("bindDN: %s, bindPass xxx\n", ctl->bindDN);
+				g_printerr("LDAP Error(bind): ldap_simple_bind_s: %s\n",
+					ldaputil_get_error(ld));
+				g_free(pwd);
+				return NULL;
+			}
+			g_free(pwd);
+		}
+	}
+	return ld;
+}
+
+/**
+ * Disconnect to LDAP server.
+ * \param ld Resource to LDAP.
+ */
+void ldapsvr_disconnect(LDAP *ld) {
+	/* Disconnect */
+	cm_return_if_fail(ld != NULL);
+	ldap_unbind_ext(ld, NULL, NULL);
+}
+
 #endif	/* USE_LDAP */
 
 /*
