@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2002-2004 by the Claws Mail Team and Hiroyuki Yamamoto
+ * Copyright (C) 2002-2012 by the Claws Mail Team and Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include <glib.h>
@@ -46,6 +47,8 @@
 #include "prefs_common.h"
 #include "log.h"
 #include "tags.h"
+#include "folder_item_prefs.h"
+#include "procmsg.h"
 
 /*!
  *\brief	Keyword lookup element
@@ -85,6 +88,10 @@ static const MatchParser matchparser_tab[] = {
 	{MATCHCRITERIA_NOT_WATCH_THREAD, "~watch_thread"},
 	{MATCHCRITERIA_SPAM, "spam"},
 	{MATCHCRITERIA_NOT_SPAM, "~spam"},
+	{MATCHCRITERIA_HAS_ATTACHMENT, "has_attachment"},
+	{MATCHCRITERIA_HAS_NO_ATTACHMENT, "~has_attachment"},
+	{MATCHCRITERIA_SIGNED, "signed"},
+	{MATCHCRITERIA_NOT_SIGNED, "~signed"},
 
 	/* msginfo headers */
 	{MATCHCRITERIA_SUBJECT, "subject"},
@@ -103,6 +110,8 @@ static const MatchParser matchparser_tab[] = {
 	{MATCHCRITERIA_NOT_TAGGED, "~tagged"},
 	{MATCHCRITERIA_AGE_GREATER, "age_greater"},
 	{MATCHCRITERIA_AGE_LOWER, "age_lower"},
+	{MATCHCRITERIA_AGE_GREATER_HOURS, "age_greater_hours"},
+	{MATCHCRITERIA_AGE_LOWER_HOURS, "age_lower_hours"},
 	{MATCHCRITERIA_NEWSGROUPS, "newsgroups"},
 	{MATCHCRITERIA_NOT_NEWSGROUPS, "~newsgroups"},
 	{MATCHCRITERIA_INREPLYTO, "inreplyto"},
@@ -175,6 +184,50 @@ enum {
 	MATCH_ONE = 2
 };
 
+enum {
+	CONTEXT_SUBJECT,
+	CONTEXT_FROM,
+	CONTEXT_TO,
+	CONTEXT_CC,
+	CONTEXT_NEWSGROUPS,
+	CONTEXT_IN_REPLY_TO,
+	CONTEXT_REFERENCES,
+	CONTEXT_HEADER,
+	CONTEXT_HEADER_LINE,
+	CONTEXT_BODY_LINE,
+	CONTEXT_TAG,
+	N_CONTEXT_STRS
+};
+
+static gchar *context_str[N_CONTEXT_STRS];
+
+void matcher_init(void)
+{
+	if (context_str[CONTEXT_SUBJECT] != NULL)
+		return;
+
+	context_str[CONTEXT_SUBJECT] = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("Subject:"));
+	context_str[CONTEXT_FROM] = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("From:"));
+	context_str[CONTEXT_TO] = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("To:"));
+	context_str[CONTEXT_CC] = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("Cc:"));
+	context_str[CONTEXT_NEWSGROUPS] = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("Newsgroups:"));
+	context_str[CONTEXT_IN_REPLY_TO] = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("In-Reply-To:"));
+	context_str[CONTEXT_REFERENCES] = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("References:"));
+	context_str[CONTEXT_HEADER] = g_strdup(_("header"));
+	context_str[CONTEXT_HEADER_LINE] = g_strdup(_("header line"));
+	context_str[CONTEXT_BODY_LINE] = g_strdup(_("body line"));
+	context_str[CONTEXT_TAG]  = g_strdup(_("tag"));
+}
+
+void matcher_done(void)
+{
+	int i;
+	for (i = 0; i < N_CONTEXT_STRS; i++) {
+		g_free(context_str[i]);
+		context_str[i] = NULL;
+	}
+}
+
 extern gboolean debug_filtering_session;
 
 /*!
@@ -267,41 +320,6 @@ MatcherProp *matcherprop_new(gint criteria, const gchar *header,
 	return prop;
 }
 
-/*!
- *\brief	Allocate a structure for a filtering / scoring
- *		"condition" (a matcher structure)
- *		Same as matcherprop_new, except it doesn't change the expr's 
- *		case.
- *
- *\param	criteria Criteria ID (MATCHCRITERIA_XXXX)
- *\param	header Header string (if criteria is MATCHCRITERIA_HEADER
-			or MATCHCRITERIA_FOUND_IN_ADDRESSBOOK)
- *\param	matchtype Type of action (MATCHTYPE_XXX)
- *\param	expr String value or expression to check
- *\param	value Integer value to check
- *
- *\return	MatcherProp * Pointer to newly allocated structure
- */
-MatcherProp *matcherprop_new_create(gint criteria, const gchar *header,
-			      gint matchtype, const gchar *expr,
-			      int value)
-{
-	MatcherProp *prop;
-
- 	prop = g_new0(MatcherProp, 1);
-	prop->criteria = criteria;
-	prop->header = header != NULL ? g_strdup(header) : NULL;
-
-	prop->expr = expr != NULL ? g_strdup(expr) : NULL;
-
-	prop->matchtype = matchtype;
-	prop->preg = NULL;
-	prop->value = value;
-	prop->error = 0;
-
-	return prop;
-}
-
 /*!
  *\brief	Free a matcher structure
  *
@@ -595,48 +613,19 @@ const gchar *debug_context)
 	return FALSE;
 }
 
-/* FIXME body search is a hack. */
-static gboolean matcherprop_string_decode_match(MatcherProp *prop, const gchar *str,
-												const gchar *debug_context)
+static gboolean matcherprop_header_line_match(MatcherProp *prop, const gchar *hdr,
+					      const gchar *str, const gchar *debug_context)
 {
-	gchar *utf = NULL;
-	gchar tmp[BUFFSIZE];
+	gchar *line = NULL;
 	gboolean res = FALSE;
 
-	if (str == NULL)
+	if (hdr == NULL || str == NULL)
 		return FALSE;
 
-	/* we try to decode QP first, because it's faster than base64 */
-	qp_decode_const(tmp, BUFFSIZE-1, str);
-	if (!g_utf8_validate(tmp, -1, NULL)) {
-		utf = conv_codeset_strdup
-			(tmp, conv_get_locale_charset_str_no_utf8(),
-			 CS_INTERNAL);
-		res = matcherprop_string_match(prop, utf, debug_context);
-		g_free(utf);
-	} else {
-		res = matcherprop_string_match(prop, tmp, debug_context);
-	}
-	
-	if (res == FALSE && (strchr(prop->expr, '=') || strchr(prop->expr, '_')
-			    || strchr(str, '=') || strchr(str, '_'))) {
-		/* if searching for something with an equal char, maybe 
-		 * we should try to match the non-decoded string. 
-		 * In case it was not qp-encoded. */
-		if (!g_utf8_validate(str, -1, NULL)) {
-			utf = conv_codeset_strdup
-				(str, conv_get_locale_charset_str_no_utf8(),
-				 CS_INTERNAL);
-			res = matcherprop_string_match(prop, utf, debug_context);
-			g_free(utf);
-		} else {
-			res = matcherprop_string_match(prop, str, debug_context);
-		}
-	}
-
-	/* FIXME base64 decoding is too slow, especially since text can 
-	 * easily be handled as base64. Don't even try now. */
-
+	line = g_strdup_printf("%s %s", hdr, str);
+	res = matcherprop_string_match(prop, line, debug_context);
+	g_free(line);
+       
 	return res;
 }
 
@@ -685,13 +674,21 @@ static gboolean matcherprop_match_test(const MatcherProp *prop,
 #endif
 
 	file = procmsg_get_message_file(info);
-	if (file == NULL)
+	if (file == NULL) {
+#ifdef USE_PTHREAD
+		g_free(td);
+#endif
 		return FALSE;
+	}
 	g_free(file);		
 
 	cmd = matching_build_command(prop->expr, info);
-	if (cmd == NULL)
+	if (cmd == NULL) {
+#ifdef USE_PTHREAD
+		g_free(td);
+#endif	
 		return FALSE;
+}
 
 #ifdef USE_PTHREAD
 	/* debug output */
@@ -758,10 +755,11 @@ static gboolean matcherprop_match_test(const MatcherProp *prop,
  *
  *\return	gboolean TRUE if a match
  */
-gboolean matcherprop_match(MatcherProp *prop, 
-			   MsgInfo *info)
+static gboolean matcherprop_match(MatcherProp *prop, 
+				  MsgInfo *info)
 {
 	time_t t;
+	gint age_mult_hours = 1;
 
 	switch(prop->criteria) {
 	case MATCHCRITERIA_ALL:
@@ -798,6 +796,14 @@ gboolean matcherprop_match(MatcherProp *prop,
 		return MSG_IS_SPAM(info->flags);
 	case MATCHCRITERIA_NOT_SPAM:
 		return !MSG_IS_SPAM(info->flags);
+	case MATCHCRITERIA_HAS_ATTACHMENT:
+		return MSG_IS_WITH_ATTACHMENT(info->flags);
+	case MATCHCRITERIA_HAS_NO_ATTACHMENT:
+		return !MSG_IS_WITH_ATTACHMENT(info->flags);
+	case MATCHCRITERIA_SIGNED:
+		return MSG_IS_SIGNED(info->flags);
+	case MATCHCRITERIA_NOT_SIGNED:
+		return !MSG_IS_SIGNED(info->flags);
 	case MATCHCRITERIA_COLORLABEL:
 	{
 		gint color = MSG_GET_COLORLABEL_VALUE(info->flags);
@@ -847,94 +853,46 @@ gboolean matcherprop_match(MatcherProp *prop,
 	case MATCHCRITERIA_NOT_WATCH_THREAD:
 		return !MSG_IS_WATCH_THREAD(info->flags);
 	case MATCHCRITERIA_SUBJECT:
-		return matcherprop_string_match(prop, info->subject,
-						prefs_common_translated_header_name("Subject:"));
+		return matcherprop_string_match(prop, info->subject, context_str[CONTEXT_SUBJECT]);
 	case MATCHCRITERIA_NOT_SUBJECT:
-		return !matcherprop_string_match(prop, info->subject,
-						prefs_common_translated_header_name("Subject:"));
+		return !matcherprop_string_match(prop, info->subject, context_str[CONTEXT_SUBJECT]);
 	case MATCHCRITERIA_FROM:
+		return matcherprop_string_match(prop, info->from, context_str[CONTEXT_FROM]);
 	case MATCHCRITERIA_NOT_FROM:
-	{
-		gchar *context;
-		gboolean ret;
-
-		context = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("From:"));
-		ret = matcherprop_string_match(prop, info->from, context);
-		g_free(context);
-		return (prop->criteria == MATCHCRITERIA_FROM)? ret : !ret;
-	}
+		return !matcherprop_string_match(prop, info->from, context_str[CONTEXT_FROM]);
 	case MATCHCRITERIA_TO:
+		return matcherprop_string_match(prop, info->to, context_str[CONTEXT_TO]);
 	case MATCHCRITERIA_NOT_TO:
-	{
-		gchar *context;
-		gboolean ret;
-
-		context = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("To:"));
-		ret = matcherprop_string_match(prop, info->to, context);
-		g_free(context);
-		return (prop->criteria == MATCHCRITERIA_TO)? ret : !ret;
-	}
+		return !matcherprop_string_match(prop, info->to, context_str[CONTEXT_TO]);
 	case MATCHCRITERIA_CC:
+		return matcherprop_string_match(prop, info->cc, context_str[CONTEXT_CC]);
 	case MATCHCRITERIA_NOT_CC:
-	{
-		gchar *context;
-		gboolean ret;
-
-		context = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("Cc:"));
-		ret = matcherprop_string_match(prop, info->cc, context);
-		g_free(context);
-		return (prop->criteria == MATCHCRITERIA_CC)? ret : !ret;
-	}
+		return !matcherprop_string_match(prop, info->cc, context_str[CONTEXT_CC]);
 	case MATCHCRITERIA_TO_OR_CC:
-	{
-		gchar *context1, *context2;
-		gboolean ret;
-
-		context1 = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("To:"));
-		context2 = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("Cc:"));
-		ret = matcherprop_string_match(prop, info->to, context1)
-			|| matcherprop_string_match(prop, info->cc, context2);
-		g_free(context1);
-		g_free(context2);
-		return ret;
-	}
+		return matcherprop_string_match(prop, info->to, context_str[CONTEXT_TO])
+		     || matcherprop_string_match(prop, info->cc, context_str[CONTEXT_CC]);
 	case MATCHCRITERIA_NOT_TO_AND_NOT_CC:
-	{
-		gchar *context1, *context2;
-		gboolean ret;
-
-		context1 = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("To:"));
-		context2 = g_strdup_printf(_("%s header"), prefs_common_translated_header_name("Cc:"));
-		ret = !(matcherprop_string_match(prop, info->to, context1)
-			|| matcherprop_string_match(prop, info->cc, context2));
-		g_free(context1);
-		g_free(context2);
-		return ret;
-	}
+		return !matcherprop_string_match(prop, info->to, context_str[CONTEXT_TO])
+		     && !matcherprop_string_match(prop, info->cc, context_str[CONTEXT_CC]);
 	case MATCHCRITERIA_TAG:
+		return matcherprop_tag_match(prop, info, context_str[CONTEXT_TAG]);
 	case MATCHCRITERIA_NOT_TAG:
-	{
-		gboolean ret;
-
-		ret = matcherprop_tag_match(prop, info, _("Tag"));
-		return (prop->criteria == MATCHCRITERIA_TAG)? ret : !ret;
-	}
+		return !matcherprop_tag_match(prop, info, context_str[CONTEXT_TAG]);
 	case MATCHCRITERIA_TAGGED:
+		return info->tags != NULL;
 	case MATCHCRITERIA_NOT_TAGGED:
-	{
-		gboolean ret;
-
-		ret = (info->tags != NULL);
-		return (prop->criteria == MATCHCRITERIA_TAGGED)? ret : !ret;
-	}
+		return info->tags == NULL;
 	case MATCHCRITERIA_AGE_GREATER:
+		age_mult_hours = 24;
+		/* Fallthrough intended */
+	case MATCHCRITERIA_AGE_GREATER_HOURS:
 	{
 		gboolean ret;
 		gint age;
 
 		t = time(NULL);
-		age = ((t - info->date_t) / (60 * 60 * 24));
-		ret = (age > prop->value);
+		age = ((t - info->date_t) / (60 * 60 * age_mult_hours));
+		ret = (age >= prop->value);
 
 		/* debug output */
 		if (debug_filtering_session
@@ -952,12 +910,15 @@ gboolean matcherprop_match(MatcherProp *prop,
 		return ret;
 	}
 	case MATCHCRITERIA_AGE_LOWER:
+		age_mult_hours = 24;
+		/* Fallthrough intended */
+	case MATCHCRITERIA_AGE_LOWER_HOURS:
 	{
 		gboolean ret;
 		gint age;
 
 		t = time(NULL);
-		age = ((t - info->date_t) / (60 * 60 * 24));
+		age = ((t - info->date_t) / (60 * 60 * age_mult_hours));
 		ret = (age < prop->value);
 
 		/* debug output */
@@ -1131,41 +1092,17 @@ gboolean matcherprop_match(MatcherProp *prop,
 		return ret;
 	}
 	case MATCHCRITERIA_NEWSGROUPS:
+		return matcherprop_string_match(prop, info->newsgroups, context_str[CONTEXT_NEWSGROUPS]);
 	case MATCHCRITERIA_NOT_NEWSGROUPS:
-	{
-		gchar *context;
-		gboolean ret;
-
-		context = g_strdup_printf(_("%s header"),
-						prefs_common_translated_header_name("Newsgroups:"));
-		ret = matcherprop_string_match(prop, info->newsgroups, context);
-		g_free(context);
-		return (prop->criteria == MATCHCRITERIA_NEWSGROUPS)? ret : !ret;
-	}
+		return !matcherprop_string_match(prop, info->newsgroups, context_str[CONTEXT_NEWSGROUPS]);
 	case MATCHCRITERIA_INREPLYTO:
+		return matcherprop_string_match(prop, info->inreplyto, context_str[CONTEXT_IN_REPLY_TO]);
 	case MATCHCRITERIA_NOT_INREPLYTO:
-	{
-		gchar *context;
-		gboolean ret;
-
-		context = g_strdup_printf(_("%s header"),
-						prefs_common_translated_header_name("In-Reply-To:"));
-		ret = matcherprop_string_match(prop, info->inreplyto, context);
-		g_free(context);
-		return (prop->criteria == MATCHCRITERIA_INREPLYTO)? ret : !ret;
-	}
+		return !matcherprop_string_match(prop, info->inreplyto, context_str[CONTEXT_IN_REPLY_TO]);
 	case MATCHCRITERIA_REFERENCES:
+		return matcherprop_list_match(prop, info->references, context_str[CONTEXT_REFERENCES]);
 	case MATCHCRITERIA_NOT_REFERENCES:
-	{
-		gchar *context;
-		gboolean ret;
-
-		context = g_strdup_printf(_("%s header"),
-						prefs_common_translated_header_name("References:"));
-		ret = matcherprop_list_match(prop, info->references, context);
-		g_free(context);
-		return (prop->criteria == MATCHCRITERIA_REFERENCES)? ret : !ret;
-	}
+		return !matcherprop_list_match(prop, info->references, context_str[CONTEXT_REFERENCES]);
 	case MATCHCRITERIA_TEST:
 		return matcherprop_match_test(prop, info);
 	case MATCHCRITERIA_NOT_TEST:
@@ -1197,6 +1134,107 @@ MatcherList *matcherlist_new(GSList *matchers, gboolean bool_and)
 	return cond;
 }
 
+#ifdef G_OS_UNIX
+/*!
+ *\brief	Builds a single regular expresion from an array of srings.
+ *
+ *\param	strings The lines containing the different sub-regexp.
+ *
+ *\return	The newly allocated regexp string.
+ */
+static gchar *build_complete_regexp(gchar **strings)
+{
+	int i = 0;
+	gchar *expr = NULL;
+	while (strings && strings[i] && *strings[i]) {
+		int old_len = expr ? strlen(expr):0;
+		int new_len = 0;
+		gchar *tmpstr = NULL;
+
+		if (g_utf8_validate(strings[i], -1, NULL))
+			tmpstr = g_strdup(strings[i]);
+		else
+			tmpstr = conv_codeset_strdup(strings[i], 
+					conv_get_locale_charset_str_no_utf8(),
+				 	CS_INTERNAL);
+
+		if (strstr(tmpstr, "\n"))
+			*(strstr(tmpstr, "\n")) = '\0';
+
+		new_len = strlen(tmpstr);
+
+		expr = g_realloc(expr, 
+			expr ? (old_len + strlen("|()") + new_len + 1)
+			     : (strlen("()") + new_len + 1));
+		
+		if (old_len) {
+			strcpy(expr + old_len, "|(");
+			strcpy(expr + old_len + 2, tmpstr);
+			strcpy(expr + old_len + 2 + new_len, ")");
+		} else {
+			strcpy(expr+old_len, "(");
+			strcpy(expr+old_len + 1, tmpstr);
+			strcpy(expr+old_len + 1 + new_len, ")");
+		}
+		g_free(tmpstr);
+		i++;
+	}
+	return expr;
+}
+#endif
+
+/*!
+ *\brief	Create a new list of matchers from a multi-line string
+ *
+ *\param	lines String with "\n"-separated expressions
+ *\param	bool_and Operator
+ *\param	case_sensitive If the matching is case sensitive or not
+ *
+ *\return	MatcherList * New matcher list
+ */
+MatcherList *matcherlist_new_from_lines(gchar *lines, gboolean bool_and, 
+					gboolean case_sensitive)
+{
+	MatcherProp *m = NULL;
+	GSList *matchers = NULL;
+	gchar **strings = g_strsplit(lines, "\n", -1);
+
+#ifdef G_OS_UNIX
+	gchar *expr = NULL;
+	expr = build_complete_regexp(strings);
+	debug_print("building matcherprop for expr '%s'\n", expr?expr:"NULL");
+	
+	m = matcherprop_new(MATCHCRITERIA_SUBJECT, NULL,
+			case_sensitive? MATCHTYPE_REGEXP: MATCHTYPE_REGEXPCASE,
+			expr, 0);
+	if (m == NULL) {
+		/* print error message */
+		debug_print("failed to allocate memory for matcherprop\n");
+	} else {
+		matchers = g_slist_append(matchers, m);
+	}
+
+	g_free(expr);
+#else
+	int i = 0;
+	while (strings && strings[i] && *strings[i]) {
+		m = matcherprop_new(MATCHCRITERIA_SUBJECT, NULL,
+			case_sensitive? MATCHTYPE_MATCH: MATCHTYPE_MATCHCASE
+			strings[i], 0);
+		if (m == NULL) {
+			/* print error message */
+			debug_print("failed to allocate memory for matcherprop\n");
+		} else {
+			matchers = g_slist_append(matchers, m);
+		}
+		i++;
+	}
+#endif
+	g_strfreev(strings);
+
+	return matcherlist_new(matchers, bool_and);
+}
+
 /*!
  *\brief	Frees a list of matchers
  *
@@ -1250,9 +1288,9 @@ static gboolean matcherprop_match_one_header(MatcherProp *matcher,
 		if (procheader_headername_equal(header->name,
 						matcher->header)) {
 			if (matcher->criteria == MATCHCRITERIA_HEADER)
-				result = matcherprop_string_match(matcher, header->body, _("header"));
+				result = matcherprop_string_match(matcher, header->body, context_str[CONTEXT_HEADER]);
 			else
-				result = !matcherprop_string_match(matcher, header->body, _("header"));
+				result = !matcherprop_string_match(matcher, header->body, context_str[CONTEXT_HEADER]);
 			procheader_header_free(header);
 			return result;
 		}
@@ -1261,13 +1299,23 @@ static gboolean matcherprop_match_one_header(MatcherProp *matcher,
 		}
 		break;
 	case MATCHCRITERIA_HEADERS_PART:
-		return matcherprop_string_match(matcher, buf, _("header line"));
-	case MATCHCRITERIA_NOT_HEADERS_PART:
-		return !matcherprop_string_match(matcher, buf, _("headers line"));
 	case MATCHCRITERIA_MESSAGE:
-		return matcherprop_string_decode_match(matcher, buf, _("message line"));
+		header = procheader_parse_header(buf);
+		if (!header)
+			return FALSE;
+		result = matcherprop_header_line_match(matcher, 
+			       header->name, header->body, context_str[CONTEXT_HEADER_LINE]);
+		procheader_header_free(header);
+		return result;
+	case MATCHCRITERIA_NOT_HEADERS_PART:
 	case MATCHCRITERIA_NOT_MESSAGE:
-		return !matcherprop_string_decode_match(matcher, buf, _("message line"));
+		header = procheader_parse_header(buf);
+		if (!header)
+			return FALSE;
+		result = !matcherprop_header_line_match(matcher, 
+			       header->name, header->body, context_str[CONTEXT_HEADER_LINE]);
+		procheader_header_free(header);
+		return result;
 	case MATCHCRITERIA_FOUND_IN_ADDRESSBOOK:
 	case MATCHCRITERIA_NOT_FOUND_IN_ADDRESSBOOK:
 		{
@@ -1470,29 +1518,129 @@ static gboolean matcherprop_criteria_body(const MatcherProp *matcher)
 		return FALSE;
 	}
 }
-
-/*!
- *\brief	Check if a (line) string matches the criteria
- *		described by a matcher structure
- *
- *\param	matcher Matcher structure
- *\param	line String
- *
- *\return	gboolean TRUE if string matches criteria
- */
-static gboolean matcherprop_match_line(MatcherProp *matcher, const gchar *line)
+	
+static gboolean matcherlist_match_binary_content(MatcherList *matchers, MimeInfo *partinfo)
 {
-	switch (matcher->criteria) {
-	case MATCHCRITERIA_BODY_PART:
-	case MATCHCRITERIA_MESSAGE:
-		return matcherprop_string_decode_match(matcher, line, _("body line"));
-	case MATCHCRITERIA_NOT_BODY_PART:
-	case MATCHCRITERIA_NOT_MESSAGE:
-		return !matcherprop_string_decode_match(matcher, line, _("body line"));
+	FILE *outfp;
+	gchar buf[BUFFSIZE];
+	GSList *l;
+
+	if (partinfo->type == MIMETYPE_TEXT)
+		return FALSE;
+	else
+		outfp = procmime_get_binary_content(partinfo);
+
+	if (!outfp)
+		return FALSE;
+
+	while (fgets(buf, sizeof(buf), outfp) != NULL) {
+		strretchomp(buf);
+
+		for (l = matchers->matchers ; l != NULL ; l = g_slist_next(l)) {
+			MatcherProp *matcher = (MatcherProp *) l->data;
+
+			if (matcher->done) 
+				continue;
+
+			/* Don't scan non-text parts when looking in body, only
+			 * when looking in whole message
+			 */
+			if (partinfo && partinfo->type != MIMETYPE_TEXT &&
+			(matcher->criteria == MATCHCRITERIA_NOT_BODY_PART ||
+			matcher->criteria == MATCHCRITERIA_BODY_PART))
+				continue;
+
+			/* if the criteria is ~body_part or ~message, ZERO lines
+			 * must match for the rule to match.
+			 */
+			if (matcher->criteria == MATCHCRITERIA_NOT_BODY_PART ||
+			    matcher->criteria == MATCHCRITERIA_NOT_MESSAGE) {
+				if (matcherprop_string_match(matcher, buf, 
+							context_str[CONTEXT_BODY_LINE])) {
+					matcher->result = FALSE;
+					matcher->done = TRUE;
+				} else
+					matcher->result = TRUE;
+			/* else, just one line has to match */
+			} else if (matcherprop_criteria_body(matcher) ||
+				   matcherprop_criteria_message(matcher)) {
+				if (matcherprop_string_match(matcher, buf,
+							context_str[CONTEXT_BODY_LINE])) {
+					matcher->result = TRUE;
+					matcher->done = TRUE;
+				}
+			}
+
+			/* if the matchers are OR'ed and the rule matched,
+			 * no need to check the others. */
+			if (matcher->result && matcher->done) {
+				if (!matchers->bool_and) {
+					fclose(outfp);
+					return TRUE;
+				}
+			}
+		}
 	}
+
+	fclose(outfp);
 	return FALSE;
 }
 
+static gboolean match_content_cb(const gchar *buf, gpointer data)
+{
+	MatcherList *matchers = (MatcherList *)data;
+	gboolean all_done = TRUE;
+	GSList *l;
+
+	for (l = matchers->matchers ; l != NULL ; l = g_slist_next(l)) {
+		MatcherProp *matcher = (MatcherProp *) l->data;
+
+		if (matcher->done) 
+			continue;
+
+		/* if the criteria is ~body_part or ~message, ZERO lines
+		 * must match for the rule to match.
+		 */
+		if (matcher->criteria == MATCHCRITERIA_NOT_BODY_PART ||
+		    matcher->criteria == MATCHCRITERIA_NOT_MESSAGE) {
+			if (matcherprop_string_match(matcher, buf, 
+						context_str[CONTEXT_BODY_LINE])) {
+				matcher->result = FALSE;
+				matcher->done = TRUE;
+			} else
+				matcher->result = TRUE;
+		/* else, just one line has to match */
+		} else if (matcherprop_criteria_body(matcher) ||
+			   matcherprop_criteria_message(matcher)) {
+			if (matcherprop_string_match(matcher, buf,
+						context_str[CONTEXT_BODY_LINE])) {
+				matcher->result = TRUE;
+				matcher->done = TRUE;
+			}
+		}
+
+		/* if the matchers are OR'ed and the rule matched,
+		 * no need to check the others. */
+		if (matcher->result && matcher->done) {
+			if (!matchers->bool_and) {
+				return TRUE;
+			}
+		}
+
+		if (!matcher->done)
+			all_done = FALSE;
+	}
+	return all_done;
+}
+
+static gboolean matcherlist_match_text_content(MatcherList *matchers, MimeInfo *partinfo)
+{
+	if (partinfo->type != MIMETYPE_TEXT)
+		return FALSE;
+
+	return procmime_scan_text_content(partinfo, match_content_cb, matchers);
+}
+
 /*!
  *\brief	Check if a line in a message file's body matches
  *		the criteria
@@ -1502,45 +1650,40 @@ static gboolean matcherprop_match_line(MatcherProp *matcher, const gchar *line)
  *
  *\return	gboolean TRUE if succesful match
  */
-static gboolean matcherlist_match_body(MatcherList *matchers, FILE *fp)
+static gboolean matcherlist_match_body(MatcherList *matchers, gboolean body_only, MsgInfo *info)
 {
-	GSList *l;
-	gchar buf[BUFFSIZE];
-	
-	while (fgets(buf, sizeof(buf), fp) != NULL) {
-		for (l = matchers->matchers ; l != NULL ; l = g_slist_next(l)) {
-			MatcherProp *matcher = (MatcherProp *) l->data;
-			
-			if (matcher->done) 
-				continue;
+	MimeInfo *mimeinfo = NULL;
+	MimeInfo *partinfo = NULL;
+	gboolean first_text_found = FALSE;
 
-			/* if the criteria is ~body_part or ~message, ZERO lines
-			 * must NOT match for the rule to match. */
-			if (matcher->criteria == MATCHCRITERIA_NOT_BODY_PART ||
-			    matcher->criteria == MATCHCRITERIA_NOT_MESSAGE) {
-				if (matcherprop_match_line(matcher, buf)) {
-					matcher->result = TRUE;
-				} else {
-					matcher->result = FALSE;
-					matcher->done = TRUE;
-				}
-			/* else, just one line has to match */
-			} else if (matcherprop_criteria_body(matcher) ||
-			           matcherprop_criteria_message(matcher)) {
-				if (matcherprop_match_line(matcher, buf)) {
-					matcher->result = TRUE;
-					matcher->done = TRUE;
-				}
-			}
+	cm_return_val_if_fail(info != NULL, FALSE);
 
-			/* if the matchers are OR'ed and the rule matched,
-			 * no need to check the others. */
-			if (matcher->result && matcher->done) {
-				if (!matchers->bool_and)
-					return TRUE;
+	mimeinfo = procmime_scan_message(info);
+
+	/* Skip headers */
+	partinfo = procmime_mimeinfo_next(mimeinfo);
+
+	for (; partinfo != NULL; partinfo = procmime_mimeinfo_next(partinfo)) {
+
+		if (partinfo->type != MIMETYPE_TEXT && body_only)
+			continue;
+
+		if (partinfo->type == MIMETYPE_TEXT) {
+			first_text_found = TRUE;
+			if (matcherlist_match_text_content(matchers, partinfo)) {
+				procmime_mimeinfo_free_all(mimeinfo);
+				return TRUE;
 			}
+		} else if (matcherlist_match_binary_content(matchers, partinfo)) {
+			procmime_mimeinfo_free_all(mimeinfo);
+			return TRUE;
 		}
+
+		if (body_only && first_text_found)
+			break;
 	}
+	procmime_mimeinfo_free_all(mimeinfo);
+
 	return FALSE;
 }
 
@@ -1558,6 +1701,7 @@ static gboolean matcherlist_match_file(MatcherList *matchers, MsgInfo *info,
 {
 	gboolean read_headers;
 	gboolean read_body;
+	gboolean body_only;
 	GSList *l;
 	FILE *fp;
 	gchar *file;
@@ -1566,6 +1710,7 @@ static gboolean matcherlist_match_file(MatcherList *matchers, MsgInfo *info,
 
 	read_headers = FALSE;
 	read_body = FALSE;
+	body_only = TRUE;
 	for (l = matchers->matchers ; l != NULL ; l = g_slist_next(l)) {
 		MatcherProp *matcher = (MatcherProp *) l->data;
 
@@ -1576,6 +1721,7 @@ static gboolean matcherlist_match_file(MatcherList *matchers, MsgInfo *info,
 		if (matcherprop_criteria_message(matcher)) {
 			read_headers = TRUE;
 			read_body = TRUE;
+			body_only = FALSE;
 		}
 		matcher->result = FALSE;
 		matcher->done = FALSE;
@@ -1605,7 +1751,7 @@ static gboolean matcherlist_match_file(MatcherList *matchers, MsgInfo *info,
 
 	/* read the body */
 	if (read_body) {
-		matcherlist_match_body(matchers, fp);
+		matcherlist_match_body(matchers, body_only, info);
 	}
 	
 	for (l = matchers->matchers; l != NULL; l = g_slist_next(l)) {
@@ -1686,6 +1832,10 @@ gboolean matcherlist_match(MatcherList *matchers, MsgInfo *info)
 		case MATCHCRITERIA_NOT_LOCKED:
 		case MATCHCRITERIA_SPAM:
 		case MATCHCRITERIA_NOT_SPAM:
+		case MATCHCRITERIA_HAS_ATTACHMENT:
+		case MATCHCRITERIA_HAS_NO_ATTACHMENT:
+		case MATCHCRITERIA_SIGNED:
+		case MATCHCRITERIA_NOT_SIGNED:
 		case MATCHCRITERIA_COLORLABEL:
 		case MATCHCRITERIA_NOT_COLORLABEL:
 		case MATCHCRITERIA_IGNORE_THREAD:
@@ -1708,6 +1858,8 @@ gboolean matcherlist_match(MatcherList *matchers, MsgInfo *info)
 		case MATCHCRITERIA_NOT_TAGGED:
 		case MATCHCRITERIA_AGE_GREATER:
 		case MATCHCRITERIA_AGE_LOWER:
+		case MATCHCRITERIA_AGE_GREATER_HOURS:
+		case MATCHCRITERIA_AGE_LOWER_HOURS:
 		case MATCHCRITERIA_NEWSGROUPS:
 		case MATCHCRITERIA_NOT_NEWSGROUPS:
 		case MATCHCRITERIA_INREPLYTO:
@@ -1855,6 +2007,8 @@ gchar *matcherprop_to_string(MatcherProp *matcher)
 	switch (matcher->criteria) {
 	case MATCHCRITERIA_AGE_GREATER:
 	case MATCHCRITERIA_AGE_LOWER:
+	case MATCHCRITERIA_AGE_GREATER_HOURS:
+	case MATCHCRITERIA_AGE_LOWER_HOURS:
 	case MATCHCRITERIA_SCORE_GREATER:
 	case MATCHCRITERIA_SCORE_LOWER:
 	case MATCHCRITERIA_SCORE_EQUAL:
@@ -1881,6 +2035,10 @@ gchar *matcherprop_to_string(MatcherProp *matcher)
 	case MATCHCRITERIA_NOT_LOCKED:
 	case MATCHCRITERIA_SPAM:
 	case MATCHCRITERIA_NOT_SPAM:
+	case MATCHCRITERIA_HAS_ATTACHMENT:
+	case MATCHCRITERIA_HAS_NO_ATTACHMENT:
+	case MATCHCRITERIA_SIGNED:
+	case MATCHCRITERIA_NOT_SIGNED:
 	case MATCHCRITERIA_PARTIAL:
 	case MATCHCRITERIA_NOT_PARTIAL:
 	case MATCHCRITERIA_IGNORE_THREAD:
@@ -2372,6 +2530,7 @@ static void matcher_add_rulenames(const gchar *rcpath)
 	FILE *dst = g_fopen(newpath, "wb");
 	gchar buf[BUFFSIZE];
 	int r;
+
 	if (src == NULL) {
 		perror("fopen");
 		if (dst)
@@ -2394,8 +2553,14 @@ static void matcher_add_rulenames(const gchar *rcpath)
 		&& strncmp(buf, "disabled rulename \"", 18)) {
 			r = fwrite("enabled rulename \"\" ",
 				strlen("enabled rulename \"\" "), 1, dst);
+			if (r != 1) {
+				g_message("cannot fwrite rulename\n");
+			}
 		}
 		r = fwrite(buf, strlen(buf), 1, dst);
+		if (r != 1) {
+			g_message("cannot fwrite rule\n");
+		}
 	}
 	fclose(dst);
 	fclose(src);
