@@ -1,6 +1,6 @@
 /*
  * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Colin Leroy <colin@colino.net> and 
+ * Copyright (C) 1999-2012 Colin Leroy <colin@colino.net> and 
  * the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -73,6 +74,9 @@
 #ifdef HAVE_PWD_H
 #include <pwd.h>
 #endif
+#ifdef USE_PTHREAD
+#include <pthread.h>
+#endif
 
 #define PLUGIN_NAME (_("Bogofilter"))
 
@@ -216,7 +220,7 @@ static void bogofilter_do_filter(BogoFilterData *data)
 			gchar *ab_folderpath;
 
 			if (*config.whitelist_ab_folder == '\0' ||
-				strcasecmp(config.whitelist_ab_folder, _("Any")) == 0) {
+				strcasecmp(config.whitelist_ab_folder, "Any") == 0) {
 				/* match the whole addressbook */
 				ab_folderpath = NULL;
 			} else {
@@ -272,8 +276,8 @@ static void bogofilter_do_filter(BogoFilterData *data)
 					    FOLDER_TYPE(msginfo->folder->folder) == F_MH &&
 					    config.insert_header) {
 						gchar *tmpfile = get_tmp_file();
-						FILE *input = fopen(file, "r");
-						FILE *output = fopen(tmpfile, "w");
+						FILE *input = g_fopen(file, "r");
+						FILE *output = g_fopen(tmpfile, "w");
 						if (strstr(parts[2], "\n"))
 							*(strstr(parts[2], "\n")) = '\0';
 						if (input && !output) 
