@@ -1,6 +1,6 @@
 /*
  * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 the Claws Mail team
+ * Copyright (C) 1999-2012 the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #ifdef USE_GPGME
@@ -364,6 +365,7 @@ static MimeInfo *pgpmime_decrypt(MimeInfo *mimeinfo)
 	if (len > 0) {
 		if (fwrite(chars, 1, len, dstfp) < len) {
         		FILE_OP_ERROR(fname, "fwrite");
+			g_free(chars);
 			fclose(dstfp);
 			privacy_set_error(_("Couldn't write to decrypted file %s"), fname);
         		g_free(fname);
@@ -373,6 +375,8 @@ static MimeInfo *pgpmime_decrypt(MimeInfo *mimeinfo)
 			return NULL;
 		}
 	}
+	g_free(chars);
+
 	if (fclose(dstfp) == EOF) {
         	FILE_OP_ERROR(fname, "fclose");
 		privacy_set_error(_("Couldn't close decrypted file %s"), fname);
@@ -425,7 +429,7 @@ static MimeInfo *pgpmime_decrypt(MimeInfo *mimeinfo)
 gboolean pgpmime_sign(MimeInfo *mimeinfo, PrefsAccount *account, const gchar *from_addr)
 {
 	MimeInfo *msgcontent, *sigmultipart, *newinfo;
-	gchar *textstr, *micalg;
+	gchar *textstr, *micalg = NULL;
 	FILE *fp;
 	gchar *boundary = NULL;
 	gchar *sigcontent;
@@ -566,6 +570,7 @@ gboolean pgpmime_sign(MimeInfo *mimeinfo, PrefsAccount *account, const gchar *fr
 	if (sigcontent == NULL || len <= 0) {
 		g_warning("sgpgme_data_release_and_get_mem failed");
 		privacy_set_error(_("Data signing failed, no contents."));
+		g_free(micalg);
 		return FALSE;
 	}
 
@@ -637,6 +642,7 @@ gboolean pgpmime_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 	if ((err = gpgme_new(&ctx)) != GPG_ERR_NO_ERROR) {
 		debug_print(("Couldn't initialize GPG context, %s"), gpgme_strerror(err));
 		privacy_set_error(_("Couldn't initialize GPG context, %s"), gpgme_strerror(err));
+		g_free(kset);
 		return FALSE;
 	}
 	i = 0;
@@ -646,6 +652,7 @@ gboolean pgpmime_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 		if (err) {
 			debug_print("can't add key '%s'[%d] (%s)\n", fprs[i],i, gpgme_strerror(err));
 			privacy_set_error(_("Couldn't add GPG key %s, %s"), fprs[i], gpgme_strerror(err));
+			g_free(kset);
 			return FALSE;
 		}
 		debug_print("found %s at %d\n", fprs[i], i);
@@ -674,6 +681,7 @@ gboolean pgpmime_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 	fp = my_tmpfile();
 	if (fp == NULL) {
 		privacy_set_error(_("Couldn't create temporary file, %s"), strerror(errno));
+		g_free(kset);
 		return FALSE;
 	}
 	procmime_write_mimeinfo(encmultipart, fp);
@@ -688,13 +696,14 @@ gboolean pgpmime_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 	gpgme_data_new_from_mem(&gpgtext, textstr, (size_t)strlen(textstr), 0);
 	gpgme_data_new(&gpgenc);
 	gpgme_set_armor(ctx, 1);
-	gpgme_data_rewind(gpgtext);
+	cm_gpgme_data_rewind(gpgtext);
 	
 	err = gpgme_op_encrypt(ctx, kset, GPGME_ENCRYPT_ALWAYS_TRUST, gpgtext, gpgenc);
 
 	enccontent = sgpgme_data_release_and_get_mem(gpgenc, &len);
 	gpgme_data_release(gpgtext);
 	g_free(textstr);
+	g_free(kset);
 
 	if (enccontent == NULL || len <= 0) {
 		g_warning("sgpgme_data_release_and_get_mem failed");
