@@ -1,6 +1,6 @@
 /* 
  * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Colin Leroy <colin@colino.net> and 
+ * Copyright (C) 1999-2012 Colin Leroy <colin@colino.net> and 
  * the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #ifdef USE_GPGME
@@ -256,7 +257,7 @@ static gint smime_check_signature(MimeInfo *mimeinfo)
 				sgpgme_verify_signature	(data->ctx, textdata, NULL, cipher);
 			gpgme_data_release(textdata);
 			g_free(textstr);
-			gpgme_data_rewind(cipher);
+			cm_gpgme_data_rewind(cipher);
 			textstr = sgpgme_data_release_and_get_mem(cipher, &len);
 			fclose(fp);
 			if (textstr && len > 0)
@@ -265,8 +266,7 @@ static gint smime_check_signature(MimeInfo *mimeinfo)
 			if (textstr && len) {
 				gchar *tmp_file = get_tmp_file();
 				MimeInfo *newinfo = NULL, *decinfo = NULL, *parentinfo = NULL;
-				gint childnumber = 0;
-								
+
 				str_write_to_file(textstr, tmp_file);
 				newinfo = procmime_scan_file(tmp_file);
 				decinfo = g_node_first_child(newinfo->node) != NULL ?
@@ -279,7 +279,7 @@ static gint smime_check_signature(MimeInfo *mimeinfo)
 				procmime_mimeinfo_free_all(newinfo);
 				decinfo->tmp = TRUE;
 				parentinfo = procmime_mimeinfo_parent(mimeinfo);
-				childnumber = g_node_child_index(parentinfo->node, mimeinfo);
+
 				if (parentinfo->type == MIMETYPE_MESSAGE && 
 				    !strcmp(parentinfo->subtype, "rfc822")) {
 					procmime_decode_content(parentinfo);
@@ -420,7 +420,7 @@ static MimeInfo *smime_decrypt(MimeInfo *mimeinfo)
 	encinfo = mimeinfo;
 
 	cipher = sgpgme_data_from_mimeinfo(encinfo);
-	gpgme_data_set_encoding(cipher, GPGME_DATA_ENCODING_BASE64);
+	
 	plain = sgpgme_decrypt_verify(cipher, &sigstat, ctx);
 
 	gpgme_data_release(cipher);
@@ -459,6 +459,7 @@ static MimeInfo *smime_decrypt(MimeInfo *mimeinfo)
 	if (len > 0) {
 		if (fwrite(chars, 1, len, dstfp) < len) {
         		FILE_OP_ERROR(fname, "fwrite");
+        		fclose(dstfp);
         		g_free(fname);
         		g_free(chars);
         		gpgme_data_release(plain);
@@ -524,7 +525,7 @@ static MimeInfo *smime_decrypt(MimeInfo *mimeinfo)
 gboolean smime_sign(MimeInfo *mimeinfo, PrefsAccount *account, const gchar *from_addr)
 {
 	MimeInfo *msgcontent, *sigmultipart, *newinfo;
-	gchar *textstr, *micalg;
+	gchar *textstr, *micalg = NULL;
 	FILE *fp;
 	gchar *boundary = NULL;
 	gchar *sigcontent;
@@ -647,12 +648,14 @@ gboolean smime_sign(MimeInfo *mimeinfo, PrefsAccount *account, const gchar *from
 
 	if (!sigcontent) {
 		gpgme_release(ctx);
+		g_free(micalg);
 		return FALSE;
 	}
 	real_content = sigcontent+strlen("-----BEGIN SIGNED MESSAGE-----\n");
 	if (!strstr(real_content, "-----END SIGNED MESSAGE-----")) {
 		debug_print("missing end\n");
 		gpgme_release(ctx);
+		g_free(micalg);
 		return FALSE;
 	}
 	*strstr(real_content, "-----END SIGNED MESSAGE-----") = '\0';
@@ -791,6 +794,9 @@ gboolean smime_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 	encmultipart->subtype = g_strdup("x-pkcs7-mime");
 	g_hash_table_insert(encmultipart->typeparameters, g_strdup("name"),
                             g_strdup("smime.p7m"));
+	g_hash_table_insert(encmultipart->typeparameters,
+			    g_strdup("smime-type"),
+			    g_strdup("enveloped-data"));
 	
 	encmultipart->disposition = DISPOSITIONTYPE_ATTACHMENT;
 	g_hash_table_insert(encmultipart->dispositionparameters, g_strdup("filename"),
@@ -803,6 +809,7 @@ gboolean smime_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 	fp = g_fopen(tmpfile, "wb");
 	if (fp == NULL) {
 		perror("get_tmp_file");
+		g_free(kset);
 		return FALSE;
 	}
 	procmime_decode_content(msgcontent);
@@ -813,6 +820,7 @@ gboolean smime_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 	fp = g_fopen(tmpfile, "rb");
 	if (fp == NULL) {
 		perror("get_tmp_file");
+		g_free(kset);
 		return FALSE;
 	}
 	g_free(tmpfile);
@@ -825,12 +833,12 @@ gboolean smime_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 	/* encrypt data */
 	gpgme_data_new_from_mem(&gpgtext, textstr, textstr?strlen(textstr):0, 0);
 	gpgme_data_new(&gpgenc);
-	gpgme_data_rewind(gpgtext);
+	cm_gpgme_data_rewind(gpgtext);
 	
-	gpgme_data_set_encoding(gpgenc, GPGME_DATA_ENCODING_BASE64);
 	gpgme_op_encrypt(ctx, kset, GPGME_ENCRYPT_ALWAYS_TRUST, gpgtext, gpgenc);
 
 	gpgme_release(ctx);
+	g_free(kset);
 	enccontent = sgpgme_data_release_and_get_mem(gpgenc, &len);
 
 	if (!enccontent) {
