@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #define __PREFS_COMMON_H__
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include <glib.h>
@@ -95,6 +95,13 @@ typedef enum
 	OPENMSG_WHEN_VIEW_VISIBLE
 } ShowMsgPolicy;
 
+typedef enum
+{
+	SHOW_NAME,
+	SHOW_ADDR,
+	SHOW_BOTH
+} SummaryFromShow;
+
 struct _PrefsCommon
 {
 #ifdef MAEMO
@@ -150,11 +157,14 @@ struct _PrefsCommon
 	gboolean show_ruler;
 	gboolean autosave;
 	gint autosave_length;
+	gboolean warn_large_insert;
+	gint warn_large_insert_size;
 	gboolean compose_no_markup;
 	ComposeDndInsertOrAttach compose_dnd_mode;
 	gboolean compose_with_format;
 	gchar *compose_subject_format;
 	gchar *compose_body_format;
+	gboolean show_compose_margin;
 
 	/* Quote */
 	gboolean reply_with_quote;
@@ -212,11 +222,11 @@ struct _PrefsCommon
 	gboolean swap_from;
 	gboolean use_addr_book;
 	gchar *date_format;
+	gboolean *msgview_date_format;
 
 	gboolean use_stripes_everywhere;
 	gboolean use_stripes_in_summaries; /* overrides if use_stripes_everywhere is set to TRUE */
 	gint stripes_color_offset;
-	gboolean enable_dotted_lines;
 	gboolean enable_hscrollbar;
 	gboolean bold_unread;
 	gboolean enable_thread;
@@ -282,11 +292,14 @@ struct _PrefsCommon
 	gint msgwin_width;
 	gint msgwin_height;
 
+	gint mimeview_tree_height;
+	
 	gint sourcewin_width;
 	gint sourcewin_height;
 
 	gint compose_width;
 	gint compose_height;
+	gint compose_notebook_height;
 	gint compose_x;
 	gint compose_y;
 
@@ -315,7 +328,9 @@ struct _PrefsCommon
 	gboolean enable_smooth_scroll;
 	gint scroll_step;
 	gboolean scroll_halfpage;
+	gboolean hide_quoted;
 	gboolean respect_flowed_format;
+	gboolean show_all_headers;
 
 	gboolean show_other_header;
 	GSList *disphdr_list;
@@ -329,6 +344,7 @@ struct _PrefsCommon
 	gchar *attach_load_dir;
 
 	GList *mime_open_cmd_history;
+	gboolean show_inline_attachments;
 
 	/* Addressbook */
 	gboolean addressbook_use_editaddress_dialog;
@@ -351,6 +367,7 @@ struct _PrefsCommon
 	EntryAction summary_select_prio[SUMMARY_OPEN_ACTIONS-1];
 
 	NextUnreadMsgDialogShow next_unread_msg_dialog;
+	SummaryFromShow summary_from_show;
 	gboolean add_address_by_click;
 	gchar *pixmap_theme_path;
 	int hover_timeout; /* msecs mouse hover timeout */
@@ -361,7 +378,8 @@ struct _PrefsCommon
 	/* Other */
 #ifndef G_OS_WIN32
 	gchar *uri_cmd;
-	gchar *print_cmd;
+#else
+	gchar *gtk_theme;
 #endif
 	gchar *ext_editor_cmd;
 	gboolean cmds_use_system_default;
@@ -389,10 +407,10 @@ struct _PrefsCommon
 	gboolean enable_filtering_debug_folder_proc;
 	gboolean enable_filtering_debug_pre_proc;
 	gboolean enable_filtering_debug_post_proc;
-    gboolean filtering_debug_cliplog;
-    guint filtering_debug_loglength;
+	gboolean filtering_debug_cliplog;
+	guint filtering_debug_loglength;
 
-    gboolean confirm_on_exit;
+	gboolean confirm_on_exit;
 	gboolean session_passwords;
 	gboolean clean_on_exit;
 	gboolean ask_on_clean;
@@ -465,6 +483,12 @@ struct _PrefsCommon
 	gint addressaddwin_height;
 	gint addressbook_folderselwin_width;
 	gint addressbook_folderselwin_height;
+	gint aboutwin_width;
+	gint aboutwin_height;
+	gint addrgather_width;
+	gint addrgather_height;
+	gint news_subscribe_width;
+	gint news_subscribe_height;
 
 	gint warn_dnd;
 	gint broken_are_utf8;
@@ -478,10 +502,10 @@ struct _PrefsCommon
 	
 	gchar *print_paper_type;
 	gint print_paper_orientation;
-  gint print_margin_top;
-  gint print_margin_bottom;
-  gint print_margin_left;
-  gint print_margin_right;
+	gint print_margin_top;
+	gint print_margin_bottom;
+	gint print_margin_left;
+	gint print_margin_right;
   
 	gint print_use_color;
 	gint print_use_collate;
@@ -495,7 +519,13 @@ struct _PrefsCommon
 	gboolean use_shred;
 	gboolean two_line_vert;
 	gboolean inherit_folder_props;
+	gboolean flush_metadata;
 
+	gint nav_history_length;
+
+	gulong diff_added_color;
+	gulong diff_deleted_color;
+	gulong diff_hunk_color;
 };
 
 extern PrefsCommon prefs_common;
