@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2011 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -274,9 +275,17 @@ static void prefs_display_header_create(void)
 	gtk_widget_show (hdr_label);
 	gtk_box_pack_start (GTK_BOX (hbox1), hdr_label, FALSE, FALSE, 0);
 
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	hdr_combo = gtk_combo_box_entry_new_text();
+#else
+	hdr_combo = gtk_combo_box_text_new_with_entry();
+#endif
 	for(i=0; i < 9 ; i++)
+#if !GTK_CHECK_VERSION(2, 24, 0)
 		gtk_combo_box_append_text(GTK_COMBO_BOX (hdr_combo),
+#else
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT (hdr_combo),
+#endif
 			(*defaults[i] == '-') ? defaults[i]+1 : defaults[i]);
 	gtk_combo_box_set_active(GTK_COMBO_BOX(hdr_combo), 0);
 	gtk_widget_show (hdr_combo);
@@ -578,7 +587,11 @@ static void prefs_display_header_list_view_set_row(gboolean hidden)
 	gchar *entry_text;
 	GtkTreeModel *model;
 
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	entry_text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(dispheader.hdr_combo));
+#else
+	entry_text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(dispheader.hdr_combo));
+#endif
 	if (!entry_text)
 		entry_text = gtk_editable_get_chars(
 				GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(dispheader.hdr_combo))),0,-1);
@@ -703,7 +716,7 @@ static gboolean prefs_display_header_key_pressed(GtkWidget *widget,
 					     GdkEventKey *event,
 					     gpointer data)
 {
-	if (event && event->keyval == GDK_Escape)
+	if (event && event->keyval == GDK_KEY_Escape)
 		prefs_display_header_cancel();
 	return FALSE;
 }
