@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2011 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -45,11 +46,13 @@
 #include "gtkutils.h"
 #include "alertpanel.h"
 #include "folder.h"
+#include "folder_item_prefs.h"
 #include "filtering.h"
 #include "addr_compl.h"
 #include "manual.h"
 #include "combobox.h"
 #include "menu.h"
+#include "account.h"
 
 #include "matcher_parser.h"
 #include "matcher.h"
@@ -114,6 +117,8 @@ static void prefs_filtering_bottom	(gpointer action, gpointer data);
 static gint prefs_filtering_deleted	(GtkWidget	*widget,
 					 GdkEventAny	*event,
 					 gpointer	 data);
+static void prefs_filtering_row_selected(GtkTreeSelection *selection,
+					 GtkTreeView *list_view);
 static gboolean prefs_filtering_key_pressed(GtkWidget	*widget,
 					 GdkEventKey	*event,
 					 gpointer	 data);
@@ -258,7 +263,7 @@ static void prefs_filtering_account_option_menu_populate(void)
 
 	cm_return_if_fail(accounts != NULL);
 
-	COMBOBOX_ADD(filtering.account_combobox_list, Q_("Filtering Account Menu|All"), 0);
+	COMBOBOX_ADD(filtering.account_combobox_list, C_("Filtering Account Menu", "All"), 0);
 	COMBOBOX_ADD(filtering.account_combobox_list, NULL, 0);
 	for (; accounts != NULL; accounts = accounts->next) {
 		PrefsAccount *ac = (PrefsAccount *)accounts->data;
@@ -330,7 +335,6 @@ static void prefs_filtering_create(void)
 	GtkWidget *bottom_btn;
 	GtkWidget *table;
 	static GdkGeometry geometry;
-	CLAWS_TIP_DECL();
 
 	debug_print("Creating filtering configuration window...\n");
 
@@ -645,97 +649,13 @@ void prefs_filtering_rename_path(const gchar *old_path, const gchar *new_path)
 static void rename_path(GSList * filters,
 			const gchar * old_path, const gchar * new_path)
 {
-	gchar *base;
-	gchar *prefix;
-	gchar *suffix;
-	gchar *dest_path;
-	gchar *old_path_with_sep;
-	gint destlen;
-	gint prefixlen;
-	gint oldpathlen;
-        GSList * action_cur;
         GSList * cur;
-	const gchar *separator=G_DIR_SEPARATOR_S;
-	gboolean matched = FALSE;
-#ifdef G_OS_WIN32
-again:
-#endif
-	oldpathlen = strlen(old_path);
-	old_path_with_sep = g_strconcat(old_path,separator,NULL);
 
 	for (cur = filters; cur != NULL; cur = cur->next) {
 		FilteringProp   *filtering = (FilteringProp *)cur->data;
-                
-                for(action_cur = filtering->action_list ; action_cur != NULL ;
-                    action_cur = action_cur->next) {
-
-                        FilteringAction *action = action_cur->data;
-                        
-                        if (action->type == MATCHACTION_SET_TAG ||
-			    action->type == MATCHACTION_UNSET_TAG)
-				continue;
-                        if (!action->destination) 
-				continue;
-                        
-                        destlen = strlen(action->destination);
-                        
-                        if (destlen > oldpathlen) {
-                                prefixlen = destlen - oldpathlen;
-                                suffix = action->destination + prefixlen;
-                                
-                                if (!strncmp(old_path, suffix, oldpathlen)) {
-                                        prefix = g_malloc0(prefixlen + 1);
-                                        strncpy2(prefix, action->destination, prefixlen);
-                                        
-                                        base = suffix + oldpathlen;
-                                        while (*base == G_DIR_SEPARATOR) base++;
-                                        if (*base == '\0')
-                                                dest_path = g_strconcat(prefix,
-                                                    separator,
-                                                    new_path, NULL);
-                                        else
-                                                dest_path = g_strconcat(prefix,
-                                                    separator,
-                                                    new_path,
-                                                    separator,
-                                                    base, NULL);
-                                        
-                                        g_free(prefix);
-                                        g_free(action->destination);
-                                        action->destination = dest_path;
-					matched = TRUE;
-                                } else { /* for non-leaf folders */
-                                        /* compare with trailing slash */
-                                        if (!strncmp(old_path_with_sep, action->destination, oldpathlen+1)) {
-                                                
-                                                suffix = action->destination + oldpathlen + 1;
-                                                dest_path = g_strconcat(new_path,
-                                                    separator,
-                                                    suffix, NULL);
-                                                g_free(action->destination);
-                                                action->destination = dest_path;
-						matched = TRUE;
-                                        }
-                                }
-                        } else {
-                                /* folder-moving a leaf */
-                                if (!strcmp(old_path, action->destination)) {
-                                        dest_path = g_strdup(new_path);
-                                        g_free(action->destination);
-                                        action->destination = dest_path;
-					matched = TRUE;
-                                }
-                        }
-                }
+		filtering_action_list_rename_path(filtering->action_list,
+						  old_path, new_path);
         }
-	
-	g_free(old_path_with_sep);
-#ifdef G_OS_WIN32
-	if (!strcmp(separator, G_DIR_SEPARATOR_S) && !matched) {
-		separator = "/";
-		goto again;
-	}
-#endif
 }
 
 static gboolean prefs_filtering_rename_path_func(GNode *node, gpointer data)
@@ -941,7 +861,7 @@ static void prefs_filtering_set_dialog(const gchar *header, const gchar *key)
 				account_name = ac_prefs->account_name;
 		}
 		if (account_name == NULL)
-			account_name = (gchar *)Q_("Filtering Account Menu|All");
+			account_name = (gchar *)C_("Filtering Account Menu", "All");
 
 		cond_str = filteringprop_to_string(prop);
 		subst_char(cond_str, '\t', ':');
@@ -993,7 +913,7 @@ static gboolean prefs_filtering_search_func_cb (GtkTreeModel *model, gint column
 	if (!store_string || !key) return FALSE;
 
 
-	retval = (strncmp (key, store_string, strlen(key)) != 0);
+	retval = (g_ascii_strncasecmp (key, store_string, strlen(key)) != 0);
 
 	g_free(store_string);
 	debug_print("selecting row\n");
@@ -1055,7 +975,7 @@ static gint prefs_filtering_list_view_set_row(gint row, FilteringProp * prop)
 	GtkListStore *list_store;
 	gchar *name = NULL;
 	gint account_id = 0;
-	gchar *account_name = (gchar *)Q_("Filtering Account Menu|All");
+	gchar *account_name = (gchar *)C_("Filtering Account Menu", "All");
 	gboolean enabled = TRUE;
 
 	if (prop)
@@ -1262,7 +1182,9 @@ static void prefs_filtering_substitute_cb(gpointer action, gpointer data)
 
 	filteringprop_free(prop);
 
-	prefs_filtering_reset_dialog();
+	prefs_filtering_row_selected(gtk_tree_view_get_selection(
+				GTK_TREE_VIEW(filtering.cond_list_view)),
+				GTK_TREE_VIEW(filtering.cond_list_view));
 	modified = TRUE;
 }
 
@@ -1367,14 +1289,13 @@ static void prefs_filtering_top(gpointer action, gpointer data)
 
 static void prefs_filtering_page_up(gpointer action, gpointer data)
 {
-	gint row, target_row, n_rows;
+	gint row, target_row;
 	GtkTreeIter selected, target;
 	GtkTreeModel *model;
 	GtkTreePath *path;
 	GdkRectangle cell_rect, view_rect;
 
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(filtering.cond_list_view));	
-	n_rows = gtk_tree_model_iter_n_children(model, NULL);
 	row = gtkut_list_view_get_selected_row(filtering.cond_list_view);
 	if (row <= 1)
 		return;
@@ -1532,7 +1453,7 @@ static gint prefs_filtering_deleted(GtkWidget *widget, GdkEventAny *event,
 static gboolean prefs_filtering_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				     gpointer data)
 {
-	if (event && event->keyval == GDK_Escape) {
+	if (event && event->keyval == GDK_KEY_Escape) {
 		prefs_filtering_cancel(NULL, NULL);
 		return TRUE;			
 	}
@@ -1778,6 +1699,21 @@ static GtkActionEntry prefs_filtering_popup_entries[] =
 #endif
 };
 
+static void prefs_filtering_row_selected(GtkTreeSelection *selection,
+					 GtkTreeView *list_view)
+{
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	
+	if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+		return;
+	
+	path = gtk_tree_model_get_path(model, &iter);
+	prefs_filtering_select_row(list_view, path);
+	gtk_tree_path_free(path);
+}
+
 static gint prefs_filtering_list_btn_pressed(GtkWidget *widget, GdkEventButton *event,
 				    GtkTreeView *list_view)
 {
@@ -1881,6 +1817,8 @@ static GtkWidget *prefs_filtering_list_view_create(void)
 	
 	selector = gtk_tree_view_get_selection(list_view);
 	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
+	g_signal_connect(G_OBJECT(selector), "changed",
+			 G_CALLBACK(prefs_filtering_row_selected), list_view);
 
 	/* create the columns */
 	prefs_filtering_create_list_view_columns(GTK_WIDGET(list_view));
