@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -32,7 +33,9 @@
 #include <string.h>
 #include <errno.h>
 
+#if !GTK_CHECK_VERSION(3, 0, 0)
 #include "gtkcmoptionmenu.h"
+#endif
 #include "main.h"
 #include "prefs_gtk.h"
 #include "prefs_matcher.h"
@@ -52,8 +55,13 @@
 #include "matcher_parser.h"
 #include "colorlabel.h"
 #include "tags.h"
+#ifndef USE_NEW_ADDRBOOK
+	#include "addressbook.h"
+#endif
 
+#ifndef USE_NEW_ADDRBOOK
 static void prefs_matcher_addressbook_select(void);
+#endif
 static void prefs_matcher_test_info(GtkWidget *widget, GtkWidget *parent);
 
 enum {
@@ -93,7 +101,9 @@ static struct Matcher {
 #ifndef G_OS_WIN32
 	GtkWidget *regexp_checkbtn;
 #endif
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	GtkWidget *color_optmenu;
+#endif
 
 	GtkWidget *test_btn;
 	GtkWidget *addressbook_select_btn;
@@ -173,7 +183,10 @@ enum {
 	CRITERIA_TAGGED = 36,
 
 	CRITERIA_HAS_ATTACHMENT = 37,
-	CRITERIA_SIGNED = 38
+	CRITERIA_SIGNED = 38,
+
+	CRITERIA_AGE_GREATER_HOURS = 39,
+	CRITERIA_AGE_LOWER_HOURS = 40
 };
 
 enum {
@@ -193,8 +206,9 @@ enum {
 };
 
 enum {
-	AGE_DAYS  = 0,
-	AGE_WEEKS = 1
+	AGE_HOURS = 0,
+	AGE_DAYS  = 1,
+	AGE_WEEKS = 2
 };
 
 enum {
@@ -316,6 +330,7 @@ static void prefs_matcher_models_create(void)
 	matcher.model_age = GTK_TREE_MODEL(store);
 
 	store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN);
+	COMBOBOX_ADD(store, _("hours"), AGE_HOURS);
 	COMBOBOX_ADD(store, _("days"), AGE_DAYS);
 	COMBOBOX_ADD(store, _("weeks"), AGE_WEEKS);
 	matcher.model_age_units = GTK_TREE_MODEL(store);
@@ -429,9 +444,11 @@ void prefs_matcher_open(MatcherList *matchers, PrefsMatcherSignal *cb)
 		prefs_matcher_models_create();
 		prefs_matcher_create();
 	} else {
+#if !GTK_CHECK_VERSION(3, 0, 0)
 		/* update color label menu */
 		gtk_cmoption_menu_set_menu(GTK_CMOPTION_MENU(matcher.color_optmenu),
 				colorlabel_create_color_menu());
+#endif
 	}
 
 	manage_window_set_transient(GTK_WINDOW(matcher.window));
@@ -522,8 +539,9 @@ static void prefs_matcher_create(void)
 
 	GtkWidget *test_btn;
 	GtkWidget *addressbook_select_btn;
-
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	GtkWidget *color_optmenu;
+#endif
 
 	static GdkGeometry geometry;
 	GtkSizeGroup *size_group;
@@ -630,7 +648,15 @@ static void prefs_matcher_create(void)
 	gtk_box_pack_start(GTK_BOX(upper_hbox), criteria_label2, FALSE, FALSE, 0);
 
 	/* headers combo box entry */
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	headers_combo = gtk_combo_box_entry_new_with_model(matcher.model_headers, 0);
+#else
+	headers_combo = gtk_combo_box_new_with_model_and_entry(matcher.model_headers);
+	GtkCellRenderer *cell = gtk_cell_renderer_text_new();
+	gtk_cell_renderer_set_alignment(cell, 0.0, 0.5);
+	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(headers_combo), cell, TRUE);
+	gtk_combo_box_set_entry_text_column(headers_combo, 0);
+#endif
 	gtk_widget_set_size_request(headers_combo, 100, -1);
 	gtk_box_pack_start(GTK_BOX(upper_hbox), headers_combo, TRUE, TRUE, 0);
 	header_entry = gtk_bin_get_child(GTK_BIN((headers_combo)));
@@ -647,12 +673,15 @@ static void prefs_matcher_create(void)
 	gtk_widget_set_size_request(addressbook_folder_combo, 150, -1);
 	gtk_box_pack_start(GTK_BOX(upper_hbox), addressbook_folder_combo, TRUE, TRUE, 0);			 
 
-	addressbook_select_btn = gtk_button_new_with_label(_("Select ..."));
+	addressbook_select_btn = gtk_button_new_with_label(_("Select..."));
 	gtk_box_pack_start(GTK_BOX(upper_hbox), addressbook_select_btn, FALSE, FALSE, 0);
+#ifndef USE_NEW_ADDRBOOK
 	g_signal_connect(G_OBJECT (addressbook_select_btn), "clicked",
 			 G_CALLBACK(prefs_matcher_addressbook_select),
 			 NULL);
-
+#else
+	gtk_widget_set_sensitive(GTK_WIDGET(addressbook_select_btn), FALSE);
+#endif
 	match_label = gtk_label_new("");
 	gtk_misc_set_alignment(GTK_MISC(match_label), 1, 0.5);
 	gtk_table_attach(GTK_TABLE(table), match_label, 0, 1, 1, 2,
@@ -665,15 +694,17 @@ static void prefs_matcher_create(void)
 	match_combo = gtkut_sc_combobox_create(NULL, TRUE);
 	gtk_box_pack_start(GTK_BOX(match_hbox), match_combo, TRUE, TRUE, 0);
 	
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	/* color labels combo */
 	color_optmenu = gtk_cmoption_menu_new();
 	gtk_cmoption_menu_set_menu(GTK_CMOPTION_MENU(color_optmenu),
 				 colorlabel_create_color_menu());
 	gtk_box_pack_start(GTK_BOX(match_hbox), color_optmenu, FALSE, FALSE, 0);
+#endif
 	
 	/* address header name */
 	header_addr_combo = combobox_text_new(TRUE,
-			      Q_("Filtering Matcher Menu|All"), _("Any"),
+			      C_("Filtering Matcher Menu", "All"), _("Any"),
 			      "From", "To", "Cc", "Reply-To", "Sender", NULL);
 	gtk_box_pack_start(GTK_BOX(match_hbox), header_addr_combo, FALSE, FALSE, 0);
 	header_addr_entry = gtk_bin_get_child(GTK_BIN((header_addr_combo)));
@@ -823,8 +854,12 @@ static void prefs_matcher_create(void)
 #endif
 	matcher.bool_op_combo = bool_op_combo;
 	matcher.test_btn = test_btn;
+#ifndef USE_NEW_ADDRBOOK
 	matcher.addressbook_select_btn = addressbook_select_btn;
+#endif
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	matcher.color_optmenu = color_optmenu;
+#endif
 	matcher.match_label = match_label;
 	matcher.criteria_label2 = criteria_label2;
 	matcher.headers_combo = headers_combo;
@@ -869,6 +904,40 @@ static void prefs_matcher_list_view_set_row(GtkTreeIter *row, MatcherProp *prop)
 	g_free(matcher_str);
 }
 
+static gboolean match_combo2_model_set(void)
+{
+	GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(matcher.match_combo2));
+	if (model == matcher.model_age_units ||
+	    model == matcher.model_found ||
+	    model == matcher.model_partial ||
+	    model == matcher.model_phrase ||
+	    model == matcher.model_set ||
+	    model == matcher.model_size_units ||
+	    model == matcher.model_tags ||
+	    model == matcher.model_thread)
+		return TRUE;
+	else
+		debug_print("match_combo2 model unset.\n");
+
+	return FALSE;
+}
+
+static gboolean match_combo_model_set(void)
+{
+	GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(matcher.match_combo));
+	if (model == matcher.model_age ||
+	    model == matcher.model_contain ||
+	    model == matcher.model_flags ||
+	    model == matcher.model_score ||
+	    model == matcher.model_size ||
+	    model == matcher.model_test)
+		return TRUE;
+	else
+		debug_print("match_combo model unset.\n");
+
+	return FALSE;
+}
+
 /*!
  *\brief	Clears a condition in the list widget
  */
@@ -876,9 +945,13 @@ static void prefs_matcher_reset_condition(void)
 {
 	gtk_combo_box_set_active(GTK_COMBO_BOX(matcher.criteria_combo), MATCH_ALL);
 	gtk_combo_box_set_active(GTK_COMBO_BOX(matcher.criteria_combo2), 0);
-	gtk_combo_box_set_active(GTK_COMBO_BOX(matcher.match_combo), 0);
-	gtk_combo_box_set_active(GTK_COMBO_BOX(matcher.match_combo2), 0);
+	if (match_combo_model_set())
+		gtk_combo_box_set_active(GTK_COMBO_BOX(matcher.match_combo), 0);
+	if (match_combo2_model_set())
+		gtk_combo_box_set_active(GTK_COMBO_BOX(matcher.match_combo2), 0);
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	gtk_cmoption_menu_set_history(GTK_CMOPTION_MENU(matcher.color_optmenu), 0);
+#endif
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(matcher.numeric_entry), 0);
 	gtk_entry_set_text(GTK_ENTRY(matcher.header_entry), "");
 	gtk_entry_set_text(GTK_ENTRY(matcher.header_addr_entry), "");
@@ -1019,9 +1092,11 @@ static gint prefs_matcher_get_criteria_from_matching(gint matching_id)
 	case MATCHCRITERIA_PARTIAL:
 	case MATCHCRITERIA_NOT_PARTIAL:
 		return CRITERIA_PARTIAL;
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	case MATCHCRITERIA_COLORLABEL:
 	case MATCHCRITERIA_NOT_COLORLABEL:
 		return CRITERIA_COLORLABEL;
+#endif
 	case MATCHCRITERIA_IGNORE_THREAD:
 	case MATCHCRITERIA_NOT_IGNORE_THREAD:
 		return CRITERIA_IGNORE_THREAD;
@@ -1070,6 +1145,10 @@ static gint prefs_matcher_get_criteria_from_matching(gint matching_id)
 	case MATCHCRITERIA_NOT_HEADER:
 	case MATCHCRITERIA_HEADER:
 		return CRITERIA_HEADER;
+	case MATCHCRITERIA_AGE_GREATER_HOURS:
+		return CRITERIA_AGE_GREATER_HOURS;
+	case MATCHCRITERIA_AGE_LOWER_HOURS:
+		return CRITERIA_AGE_LOWER_HOURS;
 	case MATCHCRITERIA_AGE_GREATER:
 		return CRITERIA_AGE_GREATER;
 	case MATCHCRITERIA_AGE_LOWER:
@@ -1132,8 +1211,10 @@ static gint prefs_matcher_get_matching_from_criteria(gint criteria_id)
 		return MATCHCRITERIA_SIGNED;
 	case CRITERIA_PARTIAL:
 		return MATCHCRITERIA_PARTIAL;
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	case CRITERIA_COLORLABEL:
 		return MATCHCRITERIA_COLORLABEL;
+#endif
 	case CRITERIA_IGNORE_THREAD:
 		return MATCHCRITERIA_IGNORE_THREAD;
 	case CRITERIA_WATCH_THREAD:
@@ -1162,6 +1243,10 @@ static gint prefs_matcher_get_matching_from_criteria(gint criteria_id)
 		return MATCHCRITERIA_AGE_GREATER;
 	case CRITERIA_AGE_LOWER:
 		return MATCHCRITERIA_AGE_LOWER;
+	case CRITERIA_AGE_GREATER_HOURS:
+		return MATCHCRITERIA_AGE_GREATER_HOURS;
+	case CRITERIA_AGE_LOWER_HOURS:
+		return MATCHCRITERIA_AGE_LOWER_HOURS;
 	case CRITERIA_SCORE_GREATER:
 		return MATCHCRITERIA_SCORE_GREATER;
 	case CRITERIA_SCORE_LOWER:
@@ -1224,8 +1309,10 @@ static gint prefs_matcher_not_criteria(gint matcher_criteria)
 		return MATCHCRITERIA_NOT_SIGNED;
 	case MATCHCRITERIA_PARTIAL:
 		return MATCHCRITERIA_NOT_PARTIAL;
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	case MATCHCRITERIA_COLORLABEL:
 		return MATCHCRITERIA_NOT_COLORLABEL;
+#endif
 	case MATCHCRITERIA_IGNORE_THREAD:
 		return MATCHCRITERIA_NOT_IGNORE_THREAD;
 	case MATCHCRITERIA_WATCH_THREAD:
@@ -1287,8 +1374,10 @@ static gint prefs_matcher_get_criteria(void)
 	case MATCH_HEADER:
 		header = gtk_entry_get_text(GTK_ENTRY(matcher.header_entry));
 		return header_name_to_crit(header);
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	case MATCH_LABEL:
 		return CRITERIA_COLORLABEL;
+#endif
 	case MATCH_PARTIAL:
 		return CRITERIA_PARTIAL;
 	case MATCH_TEST:
@@ -1367,12 +1456,6 @@ static MatcherProp *prefs_matcher_dialog_to_matcher(void)
 	if (value_criteria == -1)
 		return NULL;
 
-	criteria = prefs_matcher_get_matching_from_criteria(value_criteria);
-
-	value_pred = prefs_matcher_get_pred(value_criteria);
-	if(value_pred)
-		criteria = prefs_matcher_not_criteria(criteria);
-
 #ifndef G_OS_WIN32
 	use_regexp = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(matcher.regexp_checkbtn));
 #else
@@ -1451,6 +1534,12 @@ static MatcherProp *prefs_matcher_dialog_to_matcher(void)
 		sel = gtk_combo_box_get_active(GTK_COMBO_BOX(matcher.match_combo2));
 		if(sel == AGE_WEEKS)
 			value *= 7;
+		else if (sel == AGE_HOURS) {
+			if (value_criteria == CRITERIA_AGE_GREATER)
+				value_criteria = CRITERIA_AGE_GREATER_HOURS;
+			else 
+				value_criteria = CRITERIA_AGE_LOWER_HOURS;
+		}
 		break;
 			
 	case CRITERIA_SCORE_GREATER:
@@ -1472,11 +1561,13 @@ static MatcherProp *prefs_matcher_dialog_to_matcher(void)
 			value *= KB_SIZE;
 		break;
 		
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	case CRITERIA_COLORLABEL:
 		value = colorlabel_get_color_menu_active_item
 			(gtk_cmoption_menu_get_menu(GTK_CMOPTION_MENU
 				(matcher.color_optmenu))); 
 		break;
+#endif
 
 	case CRITERIA_HEADER:
 		header = gtk_entry_get_text(GTK_ENTRY(matcher.header_entry));
@@ -1504,7 +1595,7 @@ static MatcherProp *prefs_matcher_dialog_to_matcher(void)
 		if (*expr == '\0') {
 			gchar *tmp;
 
-			if (g_utf8_collate(header, Q_("Filtering Matcher Menu|All")) == 0)
+			if (g_utf8_collate(header, C_("Filtering Matcher Menu", "All")) == 0)
 				tmp = g_strdup(_("all addresses in all headers"));
 			else
 			if (g_utf8_collate(header, _("Any")) == 0)
@@ -1519,7 +1610,7 @@ static MatcherProp *prefs_matcher_dialog_to_matcher(void)
 		    return NULL;
 		}
 		/* store UNtranslated "Any"/"All" in matcher expressions */
-		if (g_utf8_collate(header, Q_("Filtering Matcher Menu|All")) == 0)
+		if (g_utf8_collate(header, C_("Filtering Matcher Menu", "All")) == 0)
 			header = "All";
 		else
 			if (g_utf8_collate(header, _("Any")) == 0)
@@ -1529,7 +1620,13 @@ static MatcherProp *prefs_matcher_dialog_to_matcher(void)
 		break;
 	}
 
-	matcherprop = matcherprop_new_create(criteria, header, matchtype,
+	criteria = prefs_matcher_get_matching_from_criteria(value_criteria);
+
+	value_pred = prefs_matcher_get_pred(value_criteria);
+	if(value_pred)
+		criteria = prefs_matcher_not_criteria(criteria);
+
+	matcherprop = matcherprop_new(criteria, header, matchtype,
 				      expr, value);
 
 	return matcherprop;
@@ -1834,8 +1931,10 @@ static void prefs_matcher_criteria_select(GtkWidget *widget,
 				    (value == MATCH_TEST));
 	prefs_matcher_enable_widget(matcher.addressbook_select_btn,
 				    (value == MATCH_ABOOK));
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	prefs_matcher_enable_widget(matcher.color_optmenu,
 				    (value == MATCH_LABEL));
+#endif
 	prefs_matcher_enable_widget(matcher.upper_filler,
 				    MATCH_CASE_REGEXP(value));
 	prefs_matcher_enable_widget(matcher.lower_filler,
@@ -1859,6 +1958,7 @@ static void prefs_matcher_criteria_select(GtkWidget *widget,
 		gtk_spin_button_set_range(GTK_SPIN_BUTTON(
 				  matcher.numeric_entry), 0, 1000);
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(matcher.numeric_entry), 0);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(matcher.match_combo2), AGE_DAYS);
 		gtk_label_set_text(GTK_LABEL(matcher.match_label), _("Age is"));
 		break;
 	case MATCH_FLAG:
@@ -1877,12 +1977,14 @@ static void prefs_matcher_criteria_select(GtkWidget *widget,
 #endif
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(matcher.case_checkbtn), FALSE);
 		break;
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	case MATCH_LABEL:
 		gtk_cmoption_menu_set_history(GTK_CMOPTION_MENU(matcher.color_optmenu), 0);
 		prefs_matcher_set_model(matcher.match_combo2, matcher.model_set);
 		gtk_label_set_text(GTK_LABEL(matcher.match_label), _("Label"));
 		gtk_label_set_text(GTK_LABEL(matcher.match_label2), _("is"));
 		break;
+#endif
 	case MATCH_PARTIAL:
 		prefs_matcher_set_model(matcher.criteria_combo2, matcher.model_partial);
 		gtk_label_set_text(GTK_LABEL(matcher.criteria_label2), _("Value:"));
@@ -1944,7 +2046,7 @@ static void prefs_matcher_criteria_select(GtkWidget *widget,
 static gboolean prefs_matcher_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				     gpointer data)
 {
-	if (event && event->keyval == GDK_Escape) {
+	if (event && event->keyval == GDK_KEY_Escape) {
 		prefs_matcher_cancel();
 		return TRUE;		
 	}
@@ -2061,6 +2163,7 @@ static gchar *test_desc_strings[] = {
 static DescriptionWindow test_desc_win = { 
 	NULL,
         NULL, 
+	TRUE,
         2,
         N_("Match Type: 'Test'"),
 	N_("'Test' allows you to test a message or message element "
@@ -2081,6 +2184,7 @@ static void prefs_matcher_test_info(GtkWidget *widget, GtkWidget *parent)
 	description_window_create(&test_desc_win);
 }
 
+#ifndef USE_NEW_ADDRBOOK
 static void prefs_matcher_addressbook_select(void)
 {
 	const gchar *folderpath = NULL;
@@ -2093,7 +2197,7 @@ static void prefs_matcher_addressbook_select(void)
 		g_free(new_path);
 	} 
 }
-
+#endif
 
 /*
  * list view
@@ -2181,6 +2285,8 @@ static void prefs_matcher_set_criteria(const gint criteria)
 		break;
 	case CRITERIA_AGE_GREATER:
 	case CRITERIA_AGE_LOWER:
+	case CRITERIA_AGE_GREATER_HOURS:
+	case CRITERIA_AGE_LOWER_HOURS:
 		match_criteria = MATCH_AGE;
 		break;
 	case CRITERIA_SCORE_GREATER:
@@ -2212,9 +2318,11 @@ static void prefs_matcher_set_criteria(const gint criteria)
 	case CRITERIA_TEST:
 		match_criteria = MATCH_TEST;
 		break;
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	case CRITERIA_COLORLABEL:
 		match_criteria = MATCH_LABEL;
 		break;
+#endif
 	case CRITERIA_TAG:
 	case CRITERIA_TAGGED:
 		match_criteria = MATCH_TAGS;
@@ -2384,7 +2492,7 @@ static gboolean prefs_matcher_selected(GtkTreeSelection *selector,
 		/* matcher expressions contain UNtranslated "Any"/"All",
 		  select the relevant translated combo item */
 		if (strcasecmp(prop->header, "All") == 0)
-			header = (gchar*)Q_("Filtering Matcher Menu|All");
+			header = (gchar*)C_("Filtering Matcher Menu", "All");
 		else
 			if (strcasecmp(prop->header, "Any") == 0)
 				header = _("Any");
@@ -2415,6 +2523,14 @@ static gboolean prefs_matcher_selected(GtkTreeSelection *selector,
 		}
 		break;
 		
+	case MATCHCRITERIA_AGE_GREATER_HOURS:
+	case MATCHCRITERIA_AGE_LOWER_HOURS:
+		gtk_combo_box_set_active(GTK_COMBO_BOX(matcher.match_combo2),
+					 AGE_HOURS);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(
+				matcher.numeric_entry), prop->value);
+		break;
+		
 	case MATCHCRITERIA_SCORE_GREATER:
 	case MATCHCRITERIA_SCORE_LOWER:
 	case MATCHCRITERIA_SCORE_EQUAL:
@@ -2443,6 +2559,7 @@ static gboolean prefs_matcher_selected(GtkTreeSelection *selector,
 		}
 		break;
 
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	case MATCHCRITERIA_NOT_COLORLABEL:
 	case MATCHCRITERIA_COLORLABEL:
 		gtk_cmoption_menu_set_history(GTK_CMOPTION_MENU(matcher.color_optmenu),
@@ -2450,6 +2567,7 @@ static gboolean prefs_matcher_selected(GtkTreeSelection *selector,
 		menu = gtk_cmoption_menu_get_menu(GTK_CMOPTION_MENU(matcher.color_optmenu));
 		g_signal_emit_by_name(G_OBJECT(menu), "selection-done", menu);
 		break;
+#endif
 
 	case MATCHCRITERIA_NOT_HEADER:
 	case MATCHCRITERIA_HEADER:
