@@ -1,7 +1,7 @@
 /*
  * Claws Mail templates subsystem 
  * Copyright (C) 2001 Alexander Barinov
- * Copyright (C) 2001-2009 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 2001-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 #include "manual.h"
 #include "gtkutils.h"
 #include "menu.h"
+#include "account.h"
 
 enum {
 	TEMPL_TEXT,
@@ -76,12 +77,12 @@ static struct
 } widgets_table[] = {
 	{N_("Name"),	&templates.entry_name,		FALSE,
 		N_("This name is used as the Menu item")},
-	{N_("From"),	&templates.entry_from,		TRUE,
+	{"From",	&templates.entry_from,		TRUE,
 		N_("Override composing account's From header. This doesn't change the composing account.")},
-	{N_("To"),	&templates.entry_to,		TRUE, 	NULL},
-	{N_("Cc"),	&templates.entry_cc,		TRUE, 	NULL},
-	{N_("Bcc"),	&templates.entry_bcc,		TRUE, 	NULL},
-	{N_("Subject"),	&templates.entry_subject,	FALSE,	NULL},
+	{"To",		&templates.entry_to,		TRUE, 	NULL},
+	{"Cc",		&templates.entry_cc,		TRUE, 	NULL},
+	{"Bcc",		&templates.entry_bcc,		TRUE, 	NULL},
+	{"Subject",	&templates.entry_subject,	FALSE,	NULL},
 	{NULL,		NULL,				FALSE,	NULL}
 };
 
@@ -130,11 +131,11 @@ void prefs_template_open(void)
 {
 	inc_lock();
 
-	if (!templates.window)
-		prefs_template_window_create();
+	prefs_template_window_create();
 
 	prefs_template_window_setup();
 	gtk_widget_show(templates.window);
+	gtk_window_set_modal(GTK_WINDOW(templates.window), TRUE);
 }
 
 /*!
@@ -184,14 +185,12 @@ static void prefs_template_window_create(void)
 	GtkWidget         *ok_btn;
 	static GdkGeometry geometry;
 	gint i;
-	CLAWS_TIP_DECL();
 
 	debug_print("Creating templates configuration window...\n");
 
 	/* main window */
 	window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "prefs_template");
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
-	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 
 	vbox = gtk_vbox_new(FALSE, 8);
@@ -210,6 +209,8 @@ static void prefs_template_window_create(void)
 	gtk_widget_show(vpaned);
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
  					      vpaned);
+	gtk_viewport_set_shadow_type (GTK_VIEWPORT(
+			gtk_bin_get_child(GTK_BIN(scrolled_window))), GTK_SHADOW_NONE);
 
 	/* vbox to handle template name and content */
 	vbox1 = gtk_vbox_new(FALSE, 6);
@@ -228,7 +229,9 @@ static void prefs_template_window_create(void)
 
 		GtkWidget *label;
 
-		label = gtk_label_new(widgets_table[i].label);
+		label = gtk_label_new( (i != 0) ?
+			prefs_common_translated_header_name(widgets_table[i].label) :
+			widgets_table[i].label);
 		gtk_widget_show(label);
 		gtk_table_attach(GTK_TABLE(table), label, 0, 1, i, (i + 1),
 				(GtkAttachOptions) (GTK_FILL),
@@ -328,11 +331,7 @@ static void prefs_template_window_create(void)
 	CLAWS_SET_TIP(clear_btn,
 			_("Clear all the input fields in the dialog"));
 
-#if GTK_CHECK_VERSION(2, 8, 0)
 	desc_btn = gtk_button_new_from_stock(GTK_STOCK_INFO);
-#else
-	desc_btn = gtk_button_new_with_label(_(" Symbols... "));
-#endif
 	gtk_widget_show(desc_btn);
 	gtk_box_pack_end(GTK_BOX(hbox2), desc_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(desc_btn), "clicked",
@@ -511,7 +510,7 @@ static gint prefs_template_deleted_cb(GtkWidget *widget, GdkEventAny *event,
 static gboolean prefs_template_key_pressed_cb(GtkWidget *widget,
 					      GdkEventKey *event, gpointer data)
 {
-	if (event && event->keyval == GDK_Escape)
+	if (event && event->keyval == GDK_KEY_Escape)
 		prefs_template_cancel_cb(NULL, NULL);
 	else {
 		GtkWidget *focused = gtkut_get_focused_child(
@@ -580,7 +579,7 @@ static void prefs_template_ok_cb(gpointer action, gpointer data)
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
 				(templates.list_view)));
 	gtk_list_store_clear(store);
-	gtk_widget_hide(templates.window);
+	gtk_widget_destroy(templates.window);
 	inc_unlock();
 }
 
@@ -607,7 +606,7 @@ static void prefs_template_cancel_cb(gpointer action, gpointer data)
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
 				(templates.list_view)));
 	gtk_list_store_clear(store);
-	gtk_widget_hide(templates.window);
+	gtk_widget_destroy(templates.window);
 	inc_unlock();
 }
 
@@ -681,6 +680,18 @@ gboolean prefs_template_string_is_valid(gchar *string, gint *line, gboolean esca
 			tmp = g_strdup(string);
 		}
 		memset(&dummyinfo, 0, sizeof(MsgInfo));
+		/* init dummy fields, so we can test the result of the parse */
+		dummyinfo.date="Sat, 30 May 2009 01:23:45 +0200";
+		dummyinfo.fromname="John Doe";
+		dummyinfo.from="John Doe <john@example.com>";
+		dummyinfo.to="John Doe <john@example.com>";
+		dummyinfo.cc="John Doe <john@example.com>";
+		dummyinfo.msgid="<1234john@example.com>";
+		dummyinfo.inreplyto="<1234john@example.com>";
+		dummyinfo.newsgroups="alt.test";
+		dummyinfo.subject="subject";
+		
+		
 #ifdef USE_ENCHANT
 		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE, account, FALSE, NULL);
 #else
@@ -723,11 +734,8 @@ static gboolean prefs_template_list_view_set_row(gint row)
 	gchar *value;
 	GtkTextBuffer *buffer;
 	GtkTextIter start, end;
-	GtkTreeModel *model;
 	gint line;
 
-	model = gtk_tree_view_get_model(GTK_TREE_VIEW(templates.list_view));
-
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(templates.text_value));
 	gtk_text_buffer_get_start_iter(buffer, &start);
 	gtk_text_buffer_get_iter_at_offset(buffer, &end, -1);
@@ -738,7 +746,7 @@ static gboolean prefs_template_list_view_set_row(gint row)
 		value = NULL;
 		}
 	if (!prefs_template_string_is_valid(value, &line, TRUE, FALSE)) {
-		alertpanel_error(_("Template body format error at line %d."), line);
+		alertpanel_error(_("The body of the template has an error at line %d."), line);
 		g_free(value);
 		return FALSE;
 	}
@@ -746,7 +754,7 @@ static gboolean prefs_template_list_view_set_row(gint row)
 	name = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_name),
 				      0, -1);
 	if (*name == '\0') {
-		alertpanel_error(_("Template name is not set."));
+		alertpanel_error(_("The template's name is not set."));
 		g_free(value);
 		return FALSE;
 	}
@@ -783,31 +791,31 @@ static gboolean prefs_template_list_view_set_row(gint row)
 	}
 
 	if (!prefs_template_string_is_valid(from, NULL, TRUE, TRUE)) {
-		alertpanel_error(_("Template From format error."));
+		alertpanel_error(_("The \"From\" field of the template contains an invalid email address."));
 		g_free(from);
 		g_free(value);
 		return FALSE;
 	}
 	if (!prefs_template_string_is_valid(to, NULL, TRUE, TRUE)) {
-		alertpanel_error(_("Template To format error."));
+		alertpanel_error(_("The \"To\" field of the template contains an invalid email address."));
 		g_free(to);
 		g_free(value);
 		return FALSE;
 	}
 	if (!prefs_template_string_is_valid(cc, NULL, TRUE, TRUE)) {
-		alertpanel_error(_("Template Cc format error."));	
+		alertpanel_error(_("The \"Cc\" field of the template contains an invalid email address."));	
 		g_free(cc);
 		g_free(value);
 		return FALSE;
 	}
 	if (!prefs_template_string_is_valid(bcc, NULL, TRUE, TRUE)) {
-		alertpanel_error(_("Template Bcc format error."));	
+		alertpanel_error(_("The \"Bcc\" field of the template contains an invalid email address."));	
 		g_free(bcc);
 		g_free(value);
 		return FALSE;
 	}
 	if (!prefs_template_string_is_valid(subject, NULL, TRUE, FALSE)) {
-		alertpanel_error(_("Template subject format error."));	
+		alertpanel_error(_("The \"Subject\" field of the template is invalid."));	
 		g_free(subject);
 		g_free(value);
 		return FALSE;
@@ -884,19 +892,17 @@ static void prefs_template_delete_cb(gpointer action, gpointer data)
 		return;
 
 	gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+	prefs_template_reset_dialog();
 	modified_list = TRUE;	
 }
 
 static void prefs_template_delete_all_cb(gpointer action, gpointer data)
 {
-	GtkListStore *list_store;
-
 	if (alertpanel(_("Delete all templates"),
 			  _("Do you really want to delete all the templates?"),
 			  GTK_STOCK_CANCEL, "+"GTK_STOCK_DELETE, NULL) == G_ALERTDEFAULT)
 	   return;
 
-	list_store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(templates.list_view)));
 	prefs_template_clear_list();
 	modified = FALSE;
 
@@ -1091,6 +1097,21 @@ static GtkActionEntry prefs_template_popup_entries[] =
 	{"PrefsTemplatePopup/Duplicate",	NULL, N_("D_uplicate"), NULL, NULL, G_CALLBACK(prefs_template_duplicate_cb) },
 };
 
+static void prefs_template_row_selected(GtkTreeSelection *selection,
+					GtkTreeView *list_view)
+{
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	
+	if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+		return;
+	
+	path = gtk_tree_model_get_path(model, &iter);
+	prefs_template_select_row(list_view, path);
+	gtk_tree_path_free(path);
+}
+
 static gint prefs_template_list_btn_pressed(GtkWidget *widget, GdkEventButton *event,
 				   GtkTreeView *list_view)
 {
@@ -1184,6 +1205,8 @@ static GtkWidget *prefs_template_list_view_create(void)
 
 	selector = gtk_tree_view_get_selection(list_view);
 	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
+	g_signal_connect(G_OBJECT(selector), "changed",
+			 G_CALLBACK(prefs_template_row_selected), list_view);
 
 	/* create the columns */
 	prefs_template_create_list_view_columns(GTK_WIDGET(list_view));
