@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2011 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -47,6 +48,7 @@
 #include "inputdialog.h"
 #include "alertpanel.h"
 #include "manage_window.h"
+#include "logwindow.h"
 #include "socket.h"
 #include "utils.h"
 #include "gtkutils.h"
@@ -62,6 +64,8 @@ struct _SendProgressDialog
 	gboolean cancelled;
 };
 
+static SendProgressDialog *send_dialog = NULL;
+
 static gint send_recv_message		(Session		*session,
 					 const gchar		*msg,
 					 gpointer		 data);
@@ -76,12 +80,25 @@ static gint send_send_data_finished	(Session		*session,
 static SendProgressDialog *send_progress_dialog_create(void);
 static void send_progress_dialog_destroy(SendProgressDialog *dialog);
 
+static void send_showlog_button_cb	(GtkWidget	*widget,
+					 gpointer	 data);
 static void send_cancel_button_cb	(GtkWidget	*widget,
 					 gpointer	 data);
 
 static void send_put_error		(Session	*session);
 
 
+void send_cancel(void)
+{
+	if (send_dialog)
+		send_cancel_button_cb(NULL, send_dialog);
+}
+
+gboolean send_is_active(void)
+{
+	return (send_dialog != NULL);
+}
+
 gint send_message(const gchar *file, PrefsAccount *ac_prefs, GSList *to_list)
 {
 	FILE *fp;
@@ -196,7 +213,6 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 	Session *session;
 	SMTPSession *smtp_session;
 	gushort port = 0;
-	SendProgressDialog *dialog;
 	gchar buf[BUFFSIZE];
 	gint ret = 0;
 	gboolean was_inited = FALSE;
@@ -239,6 +255,34 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 			smtp_session->hostname = NULL;
 		}
 
+#ifdef USE_GNUTLS
+		port = ac_prefs->set_smtpport ? ac_prefs->smtpport :
+			ac_prefs->ssl_smtp == SSL_TUNNEL ? SSMTP_PORT : SMTP_PORT;
+		session->ssl_type = ac_prefs->ssl_smtp;
+		if (ac_prefs->ssl_smtp != SSL_NONE)
+			session->nonblocking = ac_prefs->use_nonblocking_ssl;
+		if (ac_prefs->set_gnutls_priority && ac_prefs->gnutls_priority &&
+		    strlen(ac_prefs->gnutls_priority))
+			session->gnutls_priority = g_strdup(ac_prefs->gnutls_priority);
+#else
+		if (ac_prefs->ssl_smtp != SSL_NONE) {
+			if (alertpanel_full(_("Insecure connection"),
+				_("This connection is configured to be secured "
+				  "using SSL, but SSL is not available in this "
+				  "build of Claws Mail. \n\n"
+				  "Do you want to continue connecting to this "
+				  "server? The communication would not be "
+				  "secure."),
+				  GTK_STOCK_CANCEL, _("Con_tinue connecting"),
+				  NULL, FALSE, NULL, ALERT_WARNING,
+				  G_ALERTDEFAULT) != G_ALERTALTERNATE) {
+				session_destroy(session);
+				return -1;
+			}
+		}
+		port = ac_prefs->set_smtpport ? ac_prefs->smtpport : SMTP_PORT;
+#endif
+
 		if (ac_prefs->use_smtp_auth) {
 			smtp_session->forced_auth_type = ac_prefs->smtp_auth_type;
 			if (ac_prefs->smtp_userid && strlen(ac_prefs->smtp_userid)) {
@@ -278,36 +322,11 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 			smtp_session->pass = NULL;
 		}
 
-#ifdef USE_GNUTLS
-		port = ac_prefs->set_smtpport ? ac_prefs->smtpport :
-			ac_prefs->ssl_smtp == SSL_TUNNEL ? SSMTP_PORT : SMTP_PORT;
-		session->ssl_type = ac_prefs->ssl_smtp;
-		if (ac_prefs->ssl_smtp != SSL_NONE)
-			session->nonblocking = ac_prefs->use_nonblocking_ssl;
-#else
-		if (ac_prefs->ssl_smtp != SSL_NONE) {
-			if (alertpanel_full(_("Insecure connection"),
-				_("This connection is configured to be secured "
-				  "using SSL, but SSL is not available in this "
-				  "build of Claws Mail. \n\n"
-				  "Do you want to continue connecting to this "
-				  "server? The communication would not be "
-				  "secure."),
-				  GTK_STOCK_CANCEL, _("Con_tinue connecting"),
-				  NULL, FALSE, NULL, ALERT_WARNING,
-				  G_ALERTDEFAULT) != G_ALERTALTERNATE) {
-				session_destroy(session);
-				return -1;
-			}
-		}
-		port = ac_prefs->set_smtpport ? ac_prefs->smtpport : SMTP_PORT;
-#endif
+		send_dialog = send_progress_dialog_create();
+		send_dialog->session = session;
+		smtp_session->dialog = send_dialog;
 
-		dialog = send_progress_dialog_create();
-		dialog->session = session;
-		smtp_session->dialog = dialog;
-
-		progress_dialog_list_set(dialog->dialog, 0, NULL, 
+		progress_dialog_list_set(send_dialog->dialog, 0, NULL, 
 					 ac_prefs->smtp_server, 
 					 _("Connecting"));
 
@@ -316,21 +335,21 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 		    && (time(NULL) - ac_prefs->last_pop_login_time) > (60 * ac_prefs->pop_before_smtp_timeout)) {
 			g_snprintf(buf, sizeof(buf), _("Doing POP before SMTP..."));
 			log_message(LOG_PROTOCOL, "%s\n", buf);
-			progress_dialog_set_label(dialog->dialog, buf);
-			progress_dialog_list_set_status(dialog->dialog, 0, _("POP before SMTP"));
+			progress_dialog_set_label(send_dialog->dialog, buf);
+			progress_dialog_list_set_status(send_dialog->dialog, 0, _("POP before SMTP"));
 			GTK_EVENTS_FLUSH();
 			inc_pop_before_smtp(ac_prefs);
 		}
 
-		g_snprintf(buf, sizeof(buf), _("Account '%s': Connecting to SMTP server: %s ..."),
-				ac_prefs->account_name, ac_prefs->smtp_server);
-		progress_dialog_set_label(dialog->dialog, buf);
+		g_snprintf(buf, sizeof(buf), _("Account '%s': Connecting to SMTP server: %s:%d..."),
+				ac_prefs->account_name, ac_prefs->smtp_server, port);
+		progress_dialog_set_label(send_dialog->dialog, buf);
 		log_message(LOG_PROTOCOL, "%s\n", buf);
 
-		session_set_recv_message_notify(session, send_recv_message, dialog);
+		session_set_recv_message_notify(session, send_recv_message, send_dialog);
 		session_set_send_data_progressive_notify
-			(session, send_send_data_progressive, dialog);
-		session_set_send_data_notify(session, send_send_data_finished, dialog);
+			(session, send_send_data_progressive, send_dialog);
+		session_set_send_data_notify(session, send_send_data_finished, send_dialog);
 
 	} else {
 		/* everything is ready to start at MAIL FROM:, just
@@ -340,7 +359,7 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 		ac_prefs->session = NULL;
 		smtp_session = SMTP_SESSION(session);
 		smtp_session->state = SMTP_HELO;
-		dialog = (SendProgressDialog *)smtp_session->dialog;
+		send_dialog = (SendProgressDialog *)smtp_session->dialog;
 		was_inited = TRUE;
 	}
 
@@ -356,7 +375,7 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 	/* connect if necessary */
 	if (!was_inited && session_connect(session, ac_prefs->smtp_server, port) < 0) {
 		session_destroy(session);
-		send_progress_dialog_destroy(dialog);
+		send_progress_dialog_destroy(send_dialog);
 		ac_prefs->session = NULL;
 		return -1;
 	}
@@ -368,7 +387,7 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 		smtp_from(smtp_session);
 	}
 
-	while (session_is_running(session) && dialog->cancelled == FALSE
+	while (session_is_running(session) && send_dialog->cancelled == FALSE
 		&& SMTP_SESSION(session)->state != SMTP_MAIL_SENT_OK)
 		gtk_main_iteration();
 
@@ -392,13 +411,13 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 		   SMTP_SESSION(session)->state == SMTP_ERROR ||
 		   SMTP_SESSION(session)->error_val != SM_OK)
 		ret = -1;
-	else if (dialog->cancelled == TRUE)
+	else if (send_dialog->cancelled == TRUE)
 		ret = -1;
 
 	if (ret == -1) {
-		manage_window_focus_in(dialog->dialog->window, NULL, NULL);
+		manage_window_focus_in(send_dialog->dialog->window, NULL, NULL);
 		send_put_error(session);
-		manage_window_focus_out(dialog->dialog->window, NULL, NULL);
+		manage_window_focus_out(send_dialog->dialog->window, NULL, NULL);
 	}
 
 	/* if we should close the connection, let's do it.
@@ -408,11 +427,11 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 	if (!keep_session || ret != 0) {
 		if (session_is_connected(session))
 			smtp_quit(smtp_session);
-		while (session_is_connected(session) && !dialog->cancelled)
+		while (session_is_connected(session) && !send_dialog->cancelled)
 			gtk_main_iteration();
 		session_destroy(session);
 		ac_prefs->session = NULL;
-		send_progress_dialog_destroy(dialog);
+		send_progress_dialog_destroy(send_dialog);
 	} else {
 		g_free(smtp_session->from);
 		g_free(smtp_session->send_data);
@@ -509,7 +528,7 @@ static gint send_send_data_progressive(Session *session, guint cur_len,
 		(dialog->dialog, (total_len == 0) ? 0 : (gfloat)cur_len / (gfloat)total_len);
 
 	if (mainwin) {
-		if (!gtkut_widget_get_visible(mainwin->progressbar))	
+		if (!gtk_widget_get_visible(mainwin->progressbar))	
 			gtk_widget_show(mainwin->progressbar);
 		gtk_progress_bar_set_fraction
 			(GTK_PROGRESS_BAR(mainwin->progressbar),
@@ -556,6 +575,8 @@ static SendProgressDialog *send_progress_dialog_create(void)
 	progress = progress_dialog_create();
 	gtk_window_set_title(GTK_WINDOW(progress->window),
 			     _("Sending message"));
+	g_signal_connect(G_OBJECT(progress->showlog_btn), "clicked",
+			 G_CALLBACK(send_showlog_button_cb), dialog);
 	g_signal_connect(G_OBJECT(progress->cancel_btn), "clicked",
 			 G_CALLBACK(send_cancel_button_cb), dialog);
 	g_signal_connect(G_OBJECT(progress->window), "delete_event",
@@ -593,6 +614,14 @@ static void send_progress_dialog_destroy(SendProgressDialog *dialog)
 		progress_dialog_destroy(dialog->dialog);
 	}
 	g_free(dialog);
+	send_dialog = NULL;
+}
+
+static void send_showlog_button_cb(GtkWidget *widget, gpointer data)
+{
+	MainWindow *mainwin = mainwindow_get_mainwindow();
+
+	log_window_show(mainwin->logwin);
 }
 
 static void send_cancel_button_cb(GtkWidget *widget, gpointer data)
