@@ -1,10 +1,10 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,12 +13,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -26,18 +27,10 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkoptionmenu.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkctree.h>
+#include <gtk/gtk.h>
+#if !GTK_CHECK_VERSION(3, 0, 0)
+#include "gtk/gtksctree.h"
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -54,11 +47,13 @@
 #include "prefs_gtk.h"
 #include "manage_window.h"
 #include "alertpanel.h"
+#include "advsearch.h"
 #include "matcher.h"
 #include "matcher_parser.h"
 #include "prefs_matcher.h"
 #include "manual.h"
 #include "prefs_common.h"
+#include "statusbar.h"
 
 static struct SummarySearchWindow {
 	GtkWidget *window;
@@ -86,7 +81,11 @@ static struct SummarySearchWindow {
 
 	SummaryView *summaryview;
 
-	MatcherList			*matcher_list;
+	AdvancedSearch	*advsearch;
+	gboolean	is_fast;
+	gboolean	matcher_is_outdated;
+	gboolean	search_in_progress;
+	GHashTable	*matched_msgnums;
 
 	gboolean is_searching;
 	gboolean from_entry_has_focus;
@@ -96,8 +95,14 @@ static struct SummarySearchWindow {
 	gboolean adv_condition_entry_has_focus;
 } search_window;
 
+static gchar* add_history_get(GtkWidget *from, GList **history);
+
 static void summary_search_create	(void);
 
+static gboolean summary_search_verify_match		(MsgInfo *msg);
+static gboolean summary_search_prepare_matcher		();
+static gboolean summary_search_prereduce_msg_list	();
+
 static void summary_search_execute	(gboolean	 backward,
 					 gboolean	 search_all);
 
@@ -140,19 +145,52 @@ static gboolean adv_condition_entry_focus_evt_in(GtkWidget *widget, GdkEventFocu
 			      	  gpointer data);
 static gboolean adv_condition_entry_focus_evt_out(GtkWidget *widget, GdkEventFocus *event,
 			      	  gpointer data);
-
+#ifndef MAEMO
 static gboolean key_pressed		(GtkWidget	*widget,
 					 GdkEventKey	*event,
 					 gpointer	 data);
+#endif
 
-
-#define GTK_BUTTON_SET_SENSITIVE(widget,sensitive) {	\
+#if !GTK_CHECK_VERSION(2,14,0)
+/* Work around http://bugzilla.gnome.org/show_bug.cgi?id=56070 */
+#define GTK_BUTTON_SET_SENSITIVE(widget,sensitive) {					\
 	gboolean in_btn = FALSE;							\
 	if (GTK_IS_BUTTON(widget))							\
-		in_btn = GTK_BUTTON(widget)->in_button;			\
-	gtk_widget_set_sensitive(widget, sensitive);		\
+		in_btn = GTK_BUTTON(widget)->in_button;					\
+	gtk_widget_set_sensitive(widget, sensitive);					\
 	if (GTK_IS_BUTTON(widget))							\
-		GTK_BUTTON(widget)->in_button = in_btn;			\
+		GTK_BUTTON(widget)->in_button = in_btn;					\
+}
+#else
+#define GTK_BUTTON_SET_SENSITIVE(widget,sensitive) {					\
+	gtk_widget_set_sensitive(widget, sensitive);					\
+}
+#endif
+
+static gchar* add_history_get(GtkWidget *from, GList **history)
+{
+	gchar *result;
+
+#if !GTK_CHECK_VERSION(2, 24, 0)
+	result = gtk_combo_box_get_active_text(GTK_COMBO_BOX(from));
+#else
+	result = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(from));
+#endif
+	if (!result)
+		result = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(from))), 0, -1);
+
+	if (result && result[0] != '\0') {
+		/* add to history */
+
+		combobox_unset_popdown_strings(GTK_COMBO_BOX(from));
+		*history = add_history(*history, result);
+		combobox_set_popdown_strings(GTK_COMBO_BOX(from), *history);
+
+		return result;
+	} else {
+		g_free(result);
+		return NULL;
+	}
 }
 
 void summary_search(SummaryView *summaryview)
@@ -194,8 +232,8 @@ static void summary_search_create(void)
 	GtkWidget *vbox1;
 	GtkWidget *bool_hbox;
 	GtkWidget *bool_optmenu;
-	GtkWidget *bool_menu;
-	GtkWidget *menuitem;
+	GtkListStore *menu;
+	GtkTreeIter iter;
 	GtkWidget *clear_btn;
 
 	GtkWidget *table1;
@@ -222,8 +260,6 @@ static void summary_search_create(void)
 	GtkWidget *next_btn;
 	GtkWidget *close_btn;
 	GtkWidget *stop_btn;
-	GtkTooltips *tooltip;
-
 	gboolean is_searching = FALSE;
 
 	window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "summary_search");
@@ -248,14 +284,14 @@ static void summary_search_create(void)
 	gtk_widget_show(bool_hbox);
 	gtk_box_pack_start(GTK_BOX(vbox1), bool_hbox, FALSE, FALSE, 0);
 
-	bool_optmenu = gtk_option_menu_new();
+	bool_optmenu = gtkut_sc_combobox_create(NULL, FALSE);
+	menu = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(bool_optmenu)));
 	gtk_widget_show(bool_optmenu);
 	gtk_box_pack_start(GTK_BOX(bool_hbox), bool_optmenu, FALSE, FALSE, 0);
 
-	bool_menu = gtk_menu_new();
-	MENUITEM_ADD(bool_menu, menuitem, _("Match any of the following"), 0);
-	MENUITEM_ADD(bool_menu, menuitem, _("Match all of the following"), 1);
-	gtk_option_menu_set_menu(GTK_OPTION_MENU(bool_optmenu), bool_menu);
+	COMBOBOX_ADD(menu, _("Match any of the following"), 0);
+	gtk_combo_box_set_active_iter(GTK_COMBO_BOX(bool_optmenu), &iter);
+	COMBOBOX_ADD(menu, _("Match all of the following"), 1);
 
 	clear_btn = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
 	gtk_widget_show(clear_btn);
@@ -268,7 +304,11 @@ static void summary_search_create(void)
 	gtk_table_set_row_spacings (GTK_TABLE (table1), 8);
 	gtk_table_set_col_spacings (GTK_TABLE (table1), 8);
 
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	from_entry = gtk_combo_box_entry_new_text ();
+#else
+	from_entry = gtk_combo_box_text_new_with_entry ();
+#endif
 	gtk_combo_box_set_active(GTK_COMBO_BOX(from_entry), -1);
 	if (prefs_common.summary_search_from_history)
 		combobox_set_popdown_strings(GTK_COMBO_BOX(from_entry),
@@ -278,12 +318,16 @@ static void summary_search_create(void)
 			  GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	g_signal_connect(G_OBJECT(from_entry), "changed",
 			 G_CALLBACK(from_changed), NULL);
-	g_signal_connect(G_OBJECT(GTK_BIN(from_entry)->child),
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN((from_entry)))),
 			 "focus_in_event", G_CALLBACK(from_entry_focus_evt_in), NULL);
-	g_signal_connect(G_OBJECT(GTK_BIN(from_entry)->child),
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN((from_entry)))),
 			 "focus_out_event", G_CALLBACK(from_entry_focus_evt_out), NULL);
 
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	to_entry = gtk_combo_box_entry_new_text ();
+#else
+	to_entry = gtk_combo_box_text_new_with_entry ();
+#endif
 	gtk_combo_box_set_active(GTK_COMBO_BOX(to_entry), -1);
 	if (prefs_common.summary_search_to_history)
 		combobox_set_popdown_strings(GTK_COMBO_BOX(to_entry),
@@ -293,12 +337,16 @@ static void summary_search_create(void)
 			  GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	g_signal_connect(G_OBJECT(to_entry), "changed",
 			 G_CALLBACK(to_changed), NULL);
-	g_signal_connect(G_OBJECT(GTK_BIN(to_entry)->child),
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN((to_entry)))),
 			 "focus_in_event", G_CALLBACK(to_entry_focus_evt_in), NULL);
-	g_signal_connect(G_OBJECT(GTK_BIN(to_entry)->child),
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN((to_entry)))),
 			 "focus_out_event", G_CALLBACK(to_entry_focus_evt_out), NULL);
 
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	subject_entry = gtk_combo_box_entry_new_text ();
+#else
+	subject_entry = gtk_combo_box_text_new_with_entry ();
+#endif
 	gtk_combo_box_set_active(GTK_COMBO_BOX(subject_entry), -1);
 	if (prefs_common.summary_search_subject_history)
 		combobox_set_popdown_strings(GTK_COMBO_BOX(subject_entry),
@@ -308,12 +356,16 @@ static void summary_search_create(void)
 			  GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	g_signal_connect(G_OBJECT(subject_entry), "changed",
 			 G_CALLBACK(subject_changed), NULL);
-	g_signal_connect(G_OBJECT(GTK_BIN(subject_entry)->child),
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN((subject_entry)))),
 			 "focus_in_event", G_CALLBACK(subject_entry_focus_evt_in), NULL);
-	g_signal_connect(G_OBJECT(GTK_BIN(subject_entry)->child),
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN((subject_entry)))),
 			 "focus_out_event", G_CALLBACK(subject_entry_focus_evt_out), NULL);
 
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	body_entry = gtk_combo_box_entry_new_text ();
+#else
+	body_entry = gtk_combo_box_text_new_with_entry ();
+#endif
 	gtk_combo_box_set_active(GTK_COMBO_BOX(body_entry), -1);
 	if (prefs_common.summary_search_body_history)
 		combobox_set_popdown_strings(GTK_COMBO_BOX(body_entry),
@@ -323,12 +375,16 @@ static void summary_search_create(void)
 			  GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	g_signal_connect(G_OBJECT(body_entry), "changed",
 			 G_CALLBACK(body_changed), NULL);
-	g_signal_connect(G_OBJECT(GTK_BIN(body_entry)->child),
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN((body_entry)))),
 			 "focus_in_event", G_CALLBACK(body_entry_focus_evt_in), NULL);
-	g_signal_connect(G_OBJECT(GTK_BIN(body_entry)->child),
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN((body_entry)))),
 			 "focus_out_event", G_CALLBACK(body_entry_focus_evt_out), NULL);
 
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	adv_condition_entry = gtk_combo_box_entry_new_text ();
+#else
+	adv_condition_entry = gtk_combo_box_text_new_with_entry ();
+#endif
 	gtk_combo_box_set_active(GTK_COMBO_BOX(adv_condition_entry), -1);
 	if (prefs_common.summary_search_adv_condition_history)
 		combobox_set_popdown_strings(GTK_COMBO_BOX(adv_condition_entry),
@@ -338,9 +394,9 @@ static void summary_search_create(void)
 			  GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	g_signal_connect(G_OBJECT(adv_condition_entry), "changed",
 			 G_CALLBACK(adv_condition_changed), NULL);
-	g_signal_connect(G_OBJECT(GTK_BIN(adv_condition_entry)->child),
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN((adv_condition_entry)))),
 			 "focus_in_event", G_CALLBACK(adv_condition_entry_focus_evt_in), NULL);
-	g_signal_connect(G_OBJECT(GTK_BIN(adv_condition_entry)->child),
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN((adv_condition_entry)))),
 			 "focus_out_event", G_CALLBACK(adv_condition_entry_focus_evt_out), NULL);
 
 	adv_condition_btn = gtk_button_new_with_label(" ... ");
@@ -349,9 +405,9 @@ static void summary_search_create(void)
 			  GTK_FILL, 0, 0, 0);
 	g_signal_connect(G_OBJECT (adv_condition_btn), "clicked",
 			 G_CALLBACK(adv_condition_btn_clicked), search_window.window);
-	tooltip = gtk_tooltips_new();
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), adv_condition_btn,
-			     _("Edit search criteria"), NULL);
+
+	CLAWS_SET_TIP(adv_condition_btn,
+			     _("Edit search criteria"));
 
 	from_label = gtk_label_new (_("From:"));
 	gtk_widget_show (from_label);
@@ -412,34 +468,34 @@ static void summary_search_create(void)
 	gtkut_stock_button_add_help(confirm_area, &help_btn);
 
 	all_btn = gtk_button_new_with_mnemonic(_("Find _all"));
-	GTK_WIDGET_SET_FLAGS(all_btn, GTK_CAN_DEFAULT);
+	gtkut_widget_set_can_default(all_btn, TRUE);
 	gtk_box_pack_start(GTK_BOX(confirm_area), all_btn, TRUE, TRUE, 0);
 	gtk_widget_show(all_btn);
 
 	prev_btn = gtk_button_new_from_stock(GTK_STOCK_GO_BACK);
-	GTK_WIDGET_SET_FLAGS(prev_btn, GTK_CAN_DEFAULT);
+	gtkut_widget_set_can_default(prev_btn, TRUE);
 	gtk_box_pack_start(GTK_BOX(confirm_area), prev_btn, TRUE, TRUE, 0);
 	gtk_widget_show(prev_btn);
 
 	next_btn = gtk_button_new_from_stock(GTK_STOCK_GO_FORWARD);
-	GTK_WIDGET_SET_FLAGS(next_btn, GTK_CAN_DEFAULT);
+	gtkut_widget_set_can_default(next_btn, TRUE);
 	gtk_box_pack_start(GTK_BOX(confirm_area), next_btn, TRUE, TRUE, 0);
 	gtk_widget_show(next_btn);
 
 	close_btn = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
-	GTK_WIDGET_SET_FLAGS(close_btn, GTK_CAN_DEFAULT);
+	gtkut_widget_set_can_default(close_btn, TRUE);
 	gtk_box_pack_start(GTK_BOX(confirm_area), close_btn, TRUE, TRUE, 0);
 	gtk_widget_show(close_btn);
 
 	/* stop button hidden */
 	stop_btn = gtk_button_new_from_stock(GTK_STOCK_STOP);
-	GTK_WIDGET_SET_FLAGS(stop_btn, GTK_CAN_DEFAULT);
+	gtkut_widget_set_can_default(stop_btn, TRUE);
 	gtk_box_pack_start(GTK_BOX(confirm_area), stop_btn, TRUE, TRUE, 0);
 
 	gtk_box_pack_start (GTK_BOX (vbox1), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(next_btn);
 
-	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, bool_menu)
+	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, bool_optmenu)
 	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, from_entry)
 	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, to_entry)
 	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, subject_entry)
@@ -484,31 +540,179 @@ static void summary_search_create(void)
 	search_window.next_btn = next_btn;
 	search_window.close_btn = close_btn;
 	search_window.stop_btn = stop_btn;
-	search_window.matcher_list = NULL;
+	search_window.advsearch = NULL;
+	search_window.matcher_is_outdated = TRUE;
+	search_window.search_in_progress = FALSE;
+	search_window.matched_msgnums = NULL;
 	search_window.is_searching = is_searching;
 #ifdef MAEMO
 	maemo_window_full_screen_if_needed(GTK_WINDOW(search_window.window));
 #endif
 }
 
-static void summary_search_execute(gboolean backward, gboolean search_all)
+static gboolean summary_search_verify_match(MsgInfo *msg)
+{
+	gpointer msgnum = GUINT_TO_POINTER(msg->msgnum);
+
+	if (g_hash_table_lookup(search_window.matched_msgnums, msgnum) != NULL)
+		return TRUE;
+	else
+		return FALSE;
+}
+
+static gboolean summary_search_progress_cb(gpointer data, guint at, guint matched, guint total)
+{
+	if (!search_window.is_searching) {
+		search_window.matcher_is_outdated = TRUE;
+		return FALSE;
+	}
+
+	return summaryview_search_root_progress(search_window.summaryview, at, matched, total);
+}
+
+static gboolean summary_search_prepare_matcher()
 {
-	SummaryView *summaryview = search_window.summaryview;
-	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
-	GtkCTreeNode *node;
-	MsgInfo *msginfo;
 	gboolean adv_search;
 	gboolean bool_and = FALSE;
 	gboolean case_sens = FALSE;
-	gboolean all_searched = FALSE;
-	gboolean matched = FALSE;
-	gboolean body_matched = FALSE;
+	gchar *matcher_str;
+	gint match_type;
 	gchar *from_str = NULL, *to_str = NULL, *subject_str = NULL;
 	gchar *body_str = NULL;
-	gchar *adv_condition = NULL;
-	StrFindFunc str_find_func = NULL;
-	gboolean is_fast = TRUE;
-	gint interval = 1000;
+	GSList *matchers = NULL;
+
+	if (!search_window.matcher_is_outdated)
+		return TRUE;
+
+	if (search_window.advsearch == NULL) {
+		search_window.advsearch = advsearch_new();
+		advsearch_set_on_error_cb(search_window.advsearch, NULL, NULL); /* TODO */
+		advsearch_set_on_progress_cb(search_window.advsearch, 
+			summary_search_progress_cb, NULL);
+	}
+
+	adv_search = gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON(search_window.adv_search_checkbtn));
+
+	if (adv_search) {
+		matcher_str = add_history_get(search_window.adv_condition_entry, &prefs_common.summary_search_adv_condition_history);
+	} else {
+		MatcherList *matcher_list;
+		bool_and = combobox_get_active_data(GTK_COMBO_BOX(search_window.bool_optmenu));
+		case_sens = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(search_window.case_checkbtn));
+
+		from_str    = add_history_get(search_window.from_entry, &prefs_common.summary_search_from_history);
+		to_str      = add_history_get(search_window.to_entry, &prefs_common.summary_search_to_history);
+		subject_str = add_history_get(search_window.subject_entry, &prefs_common.summary_search_subject_history);
+		body_str    = add_history_get(search_window.body_entry, &prefs_common.summary_search_body_history);
+
+		if (!from_str && !to_str && !subject_str && !body_str) {
+			/* TODO: warn if no search criteria? (or make buttons enabled only when
+ 			 * at least one search criteria has been set */
+			return FALSE;
+		}
+
+		match_type = case_sens ? MATCHTYPE_MATCH : MATCHTYPE_MATCHCASE;
+
+		if (from_str) {
+			MatcherProp *prop = matcherprop_new(MATCHCRITERIA_FROM, NULL, match_type, from_str, 0);
+			matchers = g_slist_append(matchers, prop);
+		}
+		if (to_str) {
+			MatcherProp *prop = matcherprop_new(MATCHCRITERIA_TO, NULL, match_type, to_str, 0);
+			matchers = g_slist_append(matchers, prop);
+		}
+		if (subject_str) {
+			MatcherProp *prop = matcherprop_new(MATCHCRITERIA_SUBJECT, NULL, match_type, subject_str, 0);
+			matchers = g_slist_append(matchers, prop);
+		}
+		if (body_str) {
+			MatcherProp *prop = matcherprop_new(MATCHCRITERIA_BODY_PART, NULL, match_type, body_str, 0);
+			matchers = g_slist_append(matchers, prop);
+		}
+		g_free(from_str);
+		g_free(to_str);
+		g_free(subject_str);
+		g_free(body_str);
+
+		matcher_list = matcherlist_new(matchers, bool_and);
+		if (!matcher_list)
+			return FALSE;
+		matcher_str = matcherlist_to_string(matcher_list);
+		matcherlist_free(matcher_list);
+	}
+	if (!matcher_str)
+		return FALSE;
+
+	advsearch_set(search_window.advsearch, ADVANCED_SEARCH_EXTENDED,
+		      matcher_str);
+
+	debug_print("Advsearch set: %s\n", matcher_str);
+	g_free(matcher_str);
+
+	if (!advsearch_has_proper_predicate(search_window.advsearch))
+		return FALSE;
+
+	search_window.matcher_is_outdated = FALSE;
+
+	return TRUE;
+}
+
+static gboolean summary_search_prereduce_msg_list()
+{
+	MsgInfoList *msglist = NULL;
+	MsgNumberList *msgnums = NULL;
+	MsgNumberList *cur;
+	SummaryView *summaryview = search_window.summaryview;
+	gboolean result;
+	FolderItem *item = summaryview->folder_item;
+	static GdkCursor *watch_cursor = NULL;
+	if (!watch_cursor)
+		watch_cursor = gdk_cursor_new(GDK_WATCH);
+
+	if (search_window.matcher_is_outdated && !summary_search_prepare_matcher()) {
+		return FALSE;
+	}
+
+	main_window_cursor_wait(mainwindow_get_mainwindow());
+	gdk_window_set_cursor(gtk_widget_get_window(search_window.window), watch_cursor);
+	statusbar_print_all(_("Searching in %s... \n"),
+		item->path ? item->path : "(null)");
+
+	result = advsearch_search_msgs_in_folders(search_window.advsearch,
+			&msglist, item, FALSE);
+	statusbar_pop_all();
+	statusbar_progress_all(0, 0, 0);
+	gdk_window_set_cursor(gtk_widget_get_window(search_window.window), NULL);
+	main_window_cursor_normal(mainwindow_get_mainwindow());
+
+	if (!result)
+		return FALSE;
+	msgnums = procmsg_get_number_list_for_msgs(msglist);
+	procmsg_msg_list_free(msglist);
+
+	if (search_window.matched_msgnums == NULL)
+		search_window.matched_msgnums = g_hash_table_new(g_direct_hash, NULL);
+
+	g_hash_table_remove_all(search_window.matched_msgnums);
+
+	for (cur = msgnums; cur != NULL; cur = cur->next) {
+		g_hash_table_insert(search_window.matched_msgnums, cur->data, GINT_TO_POINTER(1));
+	}
+
+	g_slist_free(msgnums);
+
+	return TRUE;
+}
+
+static void summary_search_execute(gboolean backward, gboolean search_all)
+{
+	SummaryView *summaryview = search_window.summaryview;
+	GtkCMCTree *ctree = GTK_CMCTREE(summaryview->ctree);
+	GtkCMCTreeNode *node;
+	MsgInfo *msginfo;
+	gboolean all_searched = FALSE;
+	gboolean matched = FALSE;
 	gint i = 0;
 
 	if (summary_is_locked(summaryview)) {
@@ -516,126 +720,24 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 	}
 	summary_lock(summaryview);
 
-	adv_search = gtk_toggle_button_get_active
-		(GTK_TOGGLE_BUTTON(search_window.adv_search_checkbtn));
-
-	if (adv_search) {
-		if (search_window.matcher_list != NULL) {
-			matcherlist_free(search_window.matcher_list);
-			search_window.matcher_list = NULL;
-		}
-		adv_condition = gtk_combo_box_get_active_text(GTK_COMBO_BOX(search_window.adv_condition_entry));
-		if (!adv_condition)
-			adv_condition = gtk_editable_get_chars(
-					GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(search_window.adv_condition_entry))),0,-1);
-		if (adv_condition && adv_condition[0] != '\0') {
-
-			/* add to history */
-			combobox_unset_popdown_strings(GTK_COMBO_BOX(search_window.adv_condition_entry));
-			prefs_common.summary_search_adv_condition_history = add_history(
-					prefs_common.summary_search_adv_condition_history, adv_condition);
-			combobox_set_popdown_strings(GTK_COMBO_BOX(search_window.adv_condition_entry),
-					prefs_common.summary_search_adv_condition_history);
-
-			search_window.matcher_list = matcher_parser_get_cond((gchar*)adv_condition, &is_fast);
-			if (!is_fast)
-				interval = 100;
-			/* TODO: check for condition parsing error and show an error dialog */
-			g_free(adv_condition);
-		} else {
-			/* TODO: warn if no search condition? (or make buttons enabled only when
-				at least one search condition has been set */
-			summary_unlock(summaryview);
-			return;
-		}
-	} else {
-		bool_and = GPOINTER_TO_INT
-			(menu_get_option_menu_active_user_data
-				(GTK_OPTION_MENU(search_window.bool_optmenu)));
-		case_sens = gtk_toggle_button_get_active
-			(GTK_TOGGLE_BUTTON(search_window.case_checkbtn));
-
-		if (case_sens) {
-			str_find_func = str_find;
-		} else {
-			str_find_func = str_case_find;
-		}
-
-		from_str    = gtk_combo_box_get_active_text(GTK_COMBO_BOX(search_window.from_entry));
-		to_str      = gtk_combo_box_get_active_text(GTK_COMBO_BOX(search_window.to_entry));
-		subject_str = gtk_combo_box_get_active_text(GTK_COMBO_BOX(search_window.subject_entry));
-		body_str    = gtk_combo_box_get_active_text(GTK_COMBO_BOX(search_window.body_entry));
-
-		if (!from_str)
-			from_str = gtk_editable_get_chars(
-					GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(search_window.from_entry))),0,-1);
-		if (!to_str)
-			to_str = gtk_editable_get_chars(
-					GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(search_window.to_entry))),0,-1);
-		if (!subject_str)
-			subject_str = gtk_editable_get_chars(
-					GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(search_window.subject_entry))),0,-1);
-		if (!body_str)
-			body_str = gtk_editable_get_chars(
-					GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(search_window.body_entry))),0,-1);
-
-		if (!from_str || !to_str || !subject_str || !body_str) {
-			/* TODO: warn if no search criteria? (or make buttons enabled only when
- 			 * at least one search criteria has been set */
-			summary_unlock(summaryview);
-			return;
-		}
-		if (	(from_str[0] == '\0') &&
-				(to_str[0] == '\0') &&
-				(subject_str[0] == '\0') &&
-				(body_str[0] == '\0')) {
-			/* TODO: warn if no search criteria? (or make buttons enabled only when
-				at least one search criteria has been set */
-			summary_unlock(summaryview);
-			return;
-		}
-
-		/* add to history */
-		if (from_str[0] != '\0')
-			combobox_unset_popdown_strings(GTK_COMBO_BOX(search_window.from_entry));
-			prefs_common.summary_search_from_history = add_history(
-					prefs_common.summary_search_from_history, from_str);
-			combobox_set_popdown_strings(GTK_COMBO_BOX(search_window.from_entry),
-					prefs_common.summary_search_from_history);
-		if (to_str[0] != '\0')
-			combobox_unset_popdown_strings(GTK_COMBO_BOX(search_window.to_entry));
-			prefs_common.summary_search_to_history = add_history(
-					prefs_common.summary_search_to_history, to_str);
-			combobox_set_popdown_strings(GTK_COMBO_BOX(search_window.to_entry),
-					prefs_common.summary_search_to_history);
-		if (subject_str[0] != '\0')
-			combobox_unset_popdown_strings(GTK_COMBO_BOX(search_window.subject_entry));
-			prefs_common.summary_search_subject_history = add_history(
-					prefs_common.summary_search_subject_history, subject_str);
-			combobox_set_popdown_strings(GTK_COMBO_BOX(search_window.subject_entry),
-					prefs_common.summary_search_subject_history);
-		if (body_str[0] != '\0')
-			combobox_unset_popdown_strings(GTK_COMBO_BOX(search_window.body_entry));
-			prefs_common.summary_search_body_history = add_history(
-					prefs_common.summary_search_body_history, body_str);
-			combobox_set_popdown_strings(GTK_COMBO_BOX(search_window.body_entry),
-					prefs_common.summary_search_body_history);
-	}
-
 	search_window.is_searching = TRUE;
 	main_window_cursor_wait(summaryview->mainwin);
 	summary_show_stop_button();
 
+	if (search_window.matcher_is_outdated && !summary_search_prereduce_msg_list()) {
+		goto exit;
+	}
+
 	if (search_all) {
 		summary_freeze(summaryview);
 		summary_unselect_all(summaryview);
-		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+		node = GTK_CMCTREE_NODE(GTK_CMCLIST(ctree)->row_list);
 		backward = FALSE;
 	} else if (!summaryview->selected) {
 		if (backward) {
-			node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list_end);
+			node = GTK_CMCTREE_NODE(GTK_CMCLIST(ctree)->row_list_end);
 		} else {
-			node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+			node = GTK_CMCTREE_NODE(GTK_CMCLIST(ctree)->row_list);
 		}
 
 		if (!node) {
@@ -647,11 +749,9 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 		}
 	} else {
 		if (backward) {
-			node = gtkut_ctree_node_prev
-				(ctree, summaryview->selected);
+			node = gtkut_ctree_node_prev(ctree, summaryview->selected);
 		} else {
-			node = gtkut_ctree_node_next
-				(ctree, summaryview->selected);
+			node = gtkut_ctree_node_next(ctree, summaryview->selected);
 		}
 	}
 
@@ -681,11 +781,9 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 					 GTK_STOCK_NO, "+" GTK_STOCK_YES, NULL);
 			if (G_ALERTALTERNATE == val) {
 				if (backward) {
-					node = GTK_CTREE_NODE
-						(GTK_CLIST(ctree)->row_list_end);
+					node = GTK_CMCTREE_NODE(GTK_CMCLIST(ctree)->row_list_end);
 				} else {
-					node = GTK_CTREE_NODE
-						(GTK_CLIST(ctree)->row_list);
+					node = GTK_CMCTREE_NODE(GTK_CMCLIST(ctree)->row_list);
 				}
 
 				all_searched = TRUE;
@@ -696,105 +794,34 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 			}
 		}
 
-		msginfo = gtk_ctree_node_get_row_data(ctree, node);
-		body_matched = FALSE;
+		msginfo = gtk_cmctree_node_get_row_data(ctree, node);
 
-		if (adv_search) {
-			matched = matcherlist_match(search_window.matcher_list, msginfo);
-		} else {
-			if (bool_and) {
-				matched = TRUE;
-				if (*from_str) {
-					if (!msginfo->from ||
-				    	!str_find_func(msginfo->from, from_str)) {
-						matched = FALSE;
-					}
-				}
-				if (matched && *to_str) {
-					if (!msginfo->to ||
-				    	!str_find_func(msginfo->to, to_str)) {
-						matched = FALSE;
-					}
-				}
-				if (matched && *subject_str) {
-					if (!msginfo->subject ||
-				    	!str_find_func(msginfo->subject, subject_str)) {
-						matched = FALSE;
-					}
-				}
-				if (matched && *body_str) {
-					if (procmime_find_string(msginfo, body_str,
-								 str_find_func)) {
-						body_matched = TRUE;
-					} else {
-						matched = FALSE;
-					}
-				}
-				if (matched && !*from_str && !*to_str &&
-			    	!*subject_str && !*body_str) {
-					matched = FALSE;
-				}
-			} else {
-				matched = FALSE;
-				if (*from_str && msginfo->from) {
-					if (str_find_func(msginfo->from, from_str)) {
-						matched = TRUE;
-					}
-				}
-				if (!matched && *to_str && msginfo->to) {
-					if (str_find_func(msginfo->to, to_str)) {
-						matched = TRUE;
-					}
-				}
-				if (!matched && *subject_str && msginfo->subject) {
-					if (str_find_func(msginfo->subject, subject_str)) {
-						matched = TRUE;
-					}
-				}
-				if (!matched && *body_str) {
-					if (procmime_find_string(msginfo, body_str,
-								 str_find_func)) {
-						matched = TRUE;
-						body_matched = TRUE;
-					}
-				}
-			}
-		}
+		matched = summary_search_verify_match(msginfo);
 
 		if (matched) {
 			if (search_all) {
-				gtk_ctree_select(ctree, node);
+				gtk_cmctree_select(ctree, node);
 			} else {
-				if (messageview_is_visible
-					(summaryview->messageview)) {
+				if (messageview_is_visible(summaryview->messageview)) {
 					summary_unlock(summaryview);
-					summary_select_node
-						(summaryview, node, TRUE, TRUE);
+					summary_select_node(summaryview, node, TRUE, TRUE);
 					summary_lock(summaryview);
-					if (body_matched) {
-						messageview_search_string
-							(summaryview->messageview,
-							 body_str, case_sens);
-					}
 				} else {
-					summary_select_node
-						(summaryview, node, FALSE, TRUE);
+					summary_select_node(summaryview, node, FALSE, TRUE);
 				}
 				break;
 			}
 		}
 
+		if (i % (search_window.is_fast ? 1000 : 100) == 0) {
+			GTK_EVENTS_FLUSH();
+		}
+
 		node = backward ? gtkut_ctree_node_prev(ctree, node)
 				: gtkut_ctree_node_next(ctree, node);
-		if (i % interval == 0)
-			GTK_EVENTS_FLUSH();
 	}
 
-	g_free(from_str);
-	g_free(to_str);
-	g_free(subject_str);
-	g_free(body_str);
-
+exit:
 	search_window.is_searching = FALSE;
 	summary_hide_stop_button();
 	main_window_cursor_normal(summaryview->mainwin);
@@ -808,17 +835,18 @@ static void summary_search_clear(GtkButton *button, gpointer data)
 {
 	if (gtk_toggle_button_get_active
 		(GTK_TOGGLE_BUTTON(search_window.adv_search_checkbtn))) {
-		gtk_entry_set_text(GTK_ENTRY(GTK_BIN(search_window.adv_condition_entry)->child), "");
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN((search_window.adv_condition_entry)))), "");
 	} else {
-		gtk_entry_set_text(GTK_ENTRY(GTK_BIN(search_window.from_entry)->child), "");
-		gtk_entry_set_text(GTK_ENTRY(GTK_BIN(search_window.to_entry)->child), "");
-		gtk_entry_set_text(GTK_ENTRY(GTK_BIN(search_window.subject_entry)->child), "");
-		gtk_entry_set_text(GTK_ENTRY(GTK_BIN(search_window.body_entry)->child), "");
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN((search_window.from_entry)))), "");
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN((search_window.to_entry)))), "");
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN((search_window.subject_entry)))), "");
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN((search_window.body_entry)))), "");
 	}
 	/* stop searching */
 	if (search_window.is_searching) {
 		search_window.is_searching = FALSE;
 	}
+	search_window.matcher_is_outdated = TRUE;
 }
 
 static void summary_search_prev_clicked(GtkButton *button, gpointer data)
@@ -840,7 +868,7 @@ static void adv_condition_btn_done(MatcherList * matchers)
 {
 	gchar *str;
 
-	g_return_if_fail(
+	cm_return_if_fail(
 			mainwindow_get_mainwindow()->summaryview->quicksearch != NULL);
 
 	if (matchers == NULL) {
@@ -848,9 +876,10 @@ static void adv_condition_btn_done(MatcherList * matchers)
 	}
 
 	str = matcherlist_to_string(matchers);
+	search_window.matcher_is_outdated = TRUE;
 
 	if (str != NULL) {
-		gtk_entry_set_text(GTK_ENTRY(GTK_BIN(search_window.adv_condition_entry)->child), str);
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN((search_window.adv_condition_entry)))), str);
 		g_free(str);
 	}
 }
@@ -865,11 +894,15 @@ static void adv_condition_btn_clicked(GtkButton *button, gpointer data)
 	const gchar * cond_str;
 	MatcherList * matchers = NULL;
 
-	g_return_if_fail( search_window.window != NULL );
+	cm_return_if_fail( search_window.window != NULL );
 
 	/* re-use the current search value if it's a condition expression,
 	   otherwise ignore it silently */
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	cond_str = gtk_combo_box_get_active_text(GTK_COMBO_BOX(search_window.adv_condition_entry));
+#else
+	cond_str = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(search_window.adv_condition_entry));
+#endif
 	if (cond_str && *cond_str != '\0') {
 		matchers = matcher_parser_get_cond((gchar*)cond_str, NULL);
 	}
@@ -885,30 +918,35 @@ static void from_changed(void)
 {
 	if (!search_window.from_entry_has_focus)
 		gtk_widget_grab_focus(search_window.from_entry);
+	search_window.matcher_is_outdated = TRUE;
 }
 
 static void to_changed(void)
 {
 	if (!search_window.to_entry_has_focus)
 		gtk_widget_grab_focus(search_window.to_entry);
+	search_window.matcher_is_outdated = TRUE;
 }
 
 static void subject_changed(void)
 {
 	if (!search_window.subject_entry_has_focus)
 		gtk_widget_grab_focus(search_window.subject_entry);
+	search_window.matcher_is_outdated = TRUE;
 }
 
 static void body_changed(void)
 {
 	if (!search_window.body_entry_has_focus)
 		gtk_widget_grab_focus(search_window.body_entry);
+	search_window.matcher_is_outdated = TRUE;
 }
 
 static void adv_condition_changed(void)
 {
 	if (!search_window.adv_condition_entry_has_focus)
 		gtk_widget_grab_focus(search_window.adv_condition_entry);
+	search_window.matcher_is_outdated = TRUE;
 }
 
 static gboolean from_entry_focus_evt_in(GtkWidget *widget, GdkEventFocus *event,
@@ -980,11 +1018,11 @@ static gboolean adv_condition_entry_focus_evt_out(GtkWidget *widget, GdkEventFoc
 	search_window.adv_condition_entry_has_focus = FALSE;
 	return FALSE;
 }
-
+#ifndef MAEMO
 static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
 			    gpointer data)
 {
-	if (event && (event->keyval == GDK_Escape)) {
+	if (event && (event->keyval == GDK_KEY_Escape)) {
 		/* ESC key will:
 			- stop a running search
 			- close the search window if no search is running
@@ -996,13 +1034,13 @@ static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
 		}
 	}
 
-	if (event && (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter)) {
+	if (event && (event->keyval == GDK_KEY_Return || event->keyval == GDK_KEY_KP_Enter)) {
 		if (!search_window.is_searching) {
 			summary_search_execute(FALSE, FALSE);
 		}
 	}
 
-	if (event && (event->keyval == GDK_Down || event->keyval == GDK_Up)) {
+	if (event && (event->keyval == GDK_KEY_Down || event->keyval == GDK_KEY_Up)) {
 		if (search_window.from_entry_has_focus) {
 			combobox_set_value_from_arrow_key(
 					GTK_COMBO_BOX(search_window.from_entry),
@@ -1037,3 +1075,4 @@ static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
 
 	return FALSE;
 }
+#endif
