@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2001-2009 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 2001-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -57,6 +58,12 @@
 #include "prefs_toolbar.h"
 #include "alertpanel.h"
 #include "imap.h"
+#include "account.h"
+#ifndef USE_NEW_ADDRBOOK
+	#include "addressbook.h"
+#else
+	#include "addressbook-dbus.h"
+#endif
 
 /* elements */
 #define TOOLBAR_TAG_INDEX        "toolbar"
@@ -136,6 +143,8 @@ static void toolbar_print_cb			(GtkWidget	*widget,
 
 static void toolbar_actions_execute_cb	   	(GtkWidget     	*widget,
 				  	    	 gpointer      	 data);
+static void toolbar_plugins_execute_cb      (GtkWidget      *widget,
+                             gpointer        data);
 
 
 static void toolbar_send_cb			(GtkWidget	*widget,
@@ -170,13 +179,17 @@ static void toolbar_check_spelling_cb  		(GtkWidget   	*widget,
 #endif
 static void toolbar_cancel_inc_cb		(GtkWidget	*widget,
 						 gpointer	 data);
+static void toolbar_cancel_send_cb		(GtkWidget	*widget,
+						 gpointer	 data);
+static void toolbar_cancel_all_cb		(GtkWidget	*widget,
+						 gpointer	 data);
 
 struct {
 	gchar *index_str;
 	const gchar *descr;
 } toolbar_text [] = {
-	{ "A_RECEIVE_ALL",   	N_("Receive Mail on all Accounts")         },
-	{ "A_RECEIVE_CUR",   	N_("Receive Mail on current Account")      },
+	{ "A_RECEIVE_ALL",   	N_("Receive Mail from all Accounts")       },
+	{ "A_RECEIVE_CUR",   	N_("Receive Mail from current Account")    },
 	{ "A_SEND_QUEUED",   	N_("Send Queued Messages")                 },
 	{ "A_COMPOSE_EMAIL", 	N_("Compose Email")                        },
 	{ "A_COMPOSE_NEWS",  	N_("Compose News")                         },
@@ -184,19 +197,19 @@ struct {
 	{ "A_REPLY_SENDER",  	N_("Reply to Sender")                      },
 	{ "A_REPLY_ALL",     	N_("Reply to All")                         },
 	{ "A_REPLY_ML",      	N_("Reply to Mailing-list")                },
-	{ "A_OPEN_MAIL",	N_("Open email")			   },
+	{ "A_OPEN_MAIL",        N_("Open email")                           },
 	{ "A_FORWARD",       	N_("Forward Message")                      }, 
 	{ "A_TRASH",        	N_("Trash Message")   	                   },
 	{ "A_DELETE_REAL",    	N_("Delete Message")                       },
 	{ "A_EXECUTE",       	N_("Execute")                              },
 	{ "A_GOTO_PREV",     	N_("Go to Previous Unread Message")        },
 	{ "A_GOTO_NEXT",     	N_("Go to Next Unread Message")            },
-	{ "A_IGNORE_THREAD", 	N_("Ignore thread")			   },
-	{ "A_WATCH_THREAD", 	N_("Watch thread")			   },
-	{ "A_PRINT",	     	N_("Print")				   },
-	{ "A_LEARN_SPAM",	N_("Learn Spam or Ham")			   },
+	{ "A_IGNORE_THREAD", 	N_("Ignore thread")                        },
+	{ "A_WATCH_THREAD", 	N_("Watch thread")                         },
+	{ "A_PRINT",	     	N_("Print")                                },
+	{ "A_LEARN_SPAM",       N_("Learn Spam or Ham")                    },
 	{ "A_GO_FOLDERS",   	N_("Open folder/Go to folder list")        },
-	{ "A_PREFERENCES",	N_("Preferences")			   },
+	{ "A_PREFERENCES",      N_("Preferences")                          },
 
 	{ "A_SEND",          	N_("Send Message")                         },
 	{ "A_SENDL",         	N_("Put into queue folder and send later") },
@@ -211,10 +224,13 @@ struct {
 #ifdef USE_ENCHANT
 	{ "A_CHECK_SPELLING",	N_("Check spelling")                       },
 #endif
-	{ "A_CLAWS_ACTIONS",   	N_("Claws Mail Actions Feature")	   }, 
-	{ "A_CANCEL_INC",	N_("Cancel receiving")			   },
-	{ "A_CLOSE",		N_("Close window")			   },
-	{ "A_SEPARATOR",     	N_("Separator")				}
+	{ "A_CLAWS_ACTIONS",   	N_("Claws Mail Actions Feature")           }, 
+	{ "A_CANCEL_INC",       N_("Cancel receiving")                     },
+	{ "A_CANCEL_SEND",      N_("Cancel sending")                       },
+	{ "A_CANCEL_ALL",       N_("Cancel receiving/sending")             },
+	{ "A_CLOSE",            N_("Close window")                         },
+	{ "A_SEPARATOR",     	N_("Separator")                            },
+	{ "A_CLAWS_PLUGINS",    N_("Claws Mail Plugins")                   },
 };
 
 /* struct holds configuration files and a list of
@@ -273,7 +289,7 @@ static gboolean toolbar_is_duplicate(gint action, ToolbarType source)
 {
 	GSList *cur;
 
-	if ((action == A_SEPARATOR) || (action == A_CLAWS_ACTIONS)) 
+	if ((action == A_SEPARATOR) || (action == A_CLAWS_ACTIONS) || (action == A_CLAWS_PLUGINS))
 		return FALSE;
 
 	for (cur = toolbar_config[source].item_list; cur != NULL; cur = cur->next) {
@@ -295,13 +311,13 @@ GList *toolbar_get_action_items(ToolbarType source)
 	gint i = 0;
 	
 	if (source == TOOLBAR_MAIN) {
-		gint main_items[]   = { A_RECEIVE_ALL,   A_RECEIVE_CUR,   A_SEND_QUEUED,
+		gint main_items[] = { A_RECEIVE_ALL,   A_RECEIVE_CUR,   A_SEND_QUEUED,
 					A_COMPOSE_EMAIL, A_REPLY_MESSAGE, A_REPLY_SENDER, 
-					A_REPLY_ALL,     A_REPLY_ML,      A_OPEN_MAIL, 	A_FORWARD, 
-					A_TRASH , A_DELETE_REAL,       A_EXECUTE,       A_GOTO_PREV, 
-					A_GOTO_NEXT,	A_IGNORE_THREAD,  A_WATCH_THREAD,	A_PRINT,
-					A_ADDRBOOK, 	A_LEARN_SPAM, A_GO_FOLDERS, 
-					A_CANCEL_INC,   A_PREFERENCES };
+					A_REPLY_ALL,     A_REPLY_ML,      A_OPEN_MAIL,     A_FORWARD, 
+					A_TRASH,         A_DELETE_REAL,   A_EXECUTE,       A_GOTO_PREV, 
+					A_GOTO_NEXT,     A_IGNORE_THREAD, A_WATCH_THREAD,  A_PRINT,
+					A_ADDRBOOK,      A_LEARN_SPAM,    A_GO_FOLDERS, 
+					A_CANCEL_INC,    A_CANCEL_SEND,   A_CANCEL_ALL,    A_PREFERENCES };
 
 		for (i = 0; i < sizeof main_items / sizeof main_items[0]; i++)  {
 			items = g_list_append(items, gettext(toolbar_text[main_items[i]].descr));
@@ -349,7 +365,7 @@ static void toolbar_parse_item(XMLFile *file, ToolbarType source)
 		if (g_utf8_collate(name, TOOLBAR_ICON_FILE) == 0) 
 			item->file = g_strdup (value);
 		else if (g_utf8_collate(name, TOOLBAR_ICON_TEXT) == 0)
-			item->text = g_strdup (gettext(value));
+			item->text = g_strdup (*value ? gettext(value):"");
 		else if (g_utf8_collate(name, TOOLBAR_ICON_ACTION) == 0)
 			item->index = toolbar_ret_val_from_text(value);
 		if (item->index == -1 && !strcmp(value, "A_DELETE")) {
@@ -358,7 +374,7 @@ static void toolbar_parse_item(XMLFile *file, ToolbarType source)
 			g_free(item->file);
 			item->file = g_strdup("trash_btn");
 			g_free(item->text);
-			item->text = g_strdup(_("Trash"));
+			item->text = g_strdup(C_("Toolbar", "Trash"));
 			rewrite = TRUE;
 		}
 		if (item->index == -1 && !strcmp(value, "A_SYL_ACTIONS")) {
@@ -386,14 +402,14 @@ const gchar *toolbar_get_short_text(int action) {
 	case A_RECEIVE_ALL: 	return _("Get Mail");
 	case A_RECEIVE_CUR: 	return _("Get");
 	case A_SEND_QUEUED: 	return _("Send");
-	case A_COMPOSE_EMAIL: 	return Q_("Toolbar|Compose");
-	case A_COMPOSE_NEWS: 	return Q_("Toolbar|Compose");
+	case A_COMPOSE_EMAIL: 	return C_("Toolbar", "Compose");
+	case A_COMPOSE_NEWS: 	return C_("Toolbar", "Compose");
 	case A_REPLY_MESSAGE: 	return _("Reply");
 	case A_REPLY_ALL: 	return _("All");
-	case A_REPLY_SENDER: 	return _("Sender");
+	case A_REPLY_SENDER: 	return C_("Toolbar", "Sender");
 	case A_REPLY_ML: 	return _("List");
 	case A_FORWARD: 	return _("Forward");
-	case A_TRASH: 		return _("Trash");
+	case A_TRASH: 		return C_("Toolbar", "Trash");
 	case A_DELETE_REAL:	return _("Delete");
 	case A_LEARN_SPAM: 	return _("Spam");
 	case A_GOTO_PREV: 	return _("Prev");
@@ -414,6 +430,8 @@ const gchar *toolbar_get_short_text(int action) {
 	case A_LINEWRAP_ALL:	return _("Wrap all");
 	case A_ADDRBOOK: 	return _("Address");
 	case A_CANCEL_INC:	return _("Stop");
+	case A_CANCEL_SEND:	return _("Stop");
+	case A_CANCEL_ALL:	return _("Stop all");
 	case A_EXECUTE:		return _("Execute");
 	#ifdef USE_ENCHANT
 	case A_CHECK_SPELLING:	return _("Check spelling");
@@ -457,6 +475,8 @@ gint toolbar_get_icon(int action) {
 	case A_LINEWRAP_ALL:	return STOCK_PIXMAP_LINEWRAP_ALL;
 	case A_ADDRBOOK: 	return STOCK_PIXMAP_ADDRESS_BOOK;
 	case A_CANCEL_INC:	return STOCK_PIXMAP_CANCEL;
+	case A_CANCEL_SEND:	return STOCK_PIXMAP_CANCEL;
+	case A_CANCEL_ALL:	return STOCK_PIXMAP_CANCEL;
 	case A_EXECUTE:		return STOCK_PIXMAP_EXEC;
 	#ifdef USE_ENCHANT
 	case A_CHECK_SPELLING:	return STOCK_PIXMAP_CHECK_SPELLING;
@@ -489,9 +509,7 @@ static void toolbar_set_default_main(void)
 		{ A_SEPARATOR},
 		{ A_TRASH},
 #ifndef GENERIC_UMPC
-#if (defined(USE_SPAMASSASSIN_PLUGIN) || defined(USE_BOGOFILTER_PLUGIN))
 		{ A_LEARN_SPAM},
-#endif
 #endif
 		{ A_SEPARATOR},
 		{ A_GOTO_NEXT}
@@ -593,9 +611,7 @@ static void toolbar_set_default_msgview(void)
 		{ A_SEPARATOR},
 		{ A_TRASH},
 #ifndef GENERIC_UMPC
-#if (defined(USE_SPAMASSASSIN_PLUGIN) || defined(USE_BOGOFILTER_PLUGIN))
 		{ A_LEARN_SPAM},
-#endif
 #endif
 		{ A_GOTO_NEXT}
 	};
@@ -699,8 +715,6 @@ void toolbar_read_config_file(ToolbarType source)
 {
 	XMLFile *file   = NULL;
 	gchar *fileSpec = NULL;
-	GList *attr;
-	gboolean retVal;
 	jmp_buf    jumper;
 
 	debug_print("read Toolbar Configuration from %s\n", toolbar_config[source].conf_file);
@@ -720,9 +734,6 @@ void toolbar_read_config_file(ToolbarType source)
 			return;
 		}
 
-		attr = xml_get_current_tag_attr(file);
-		
-		retVal = TRUE;
 		for (;;) {
 			if (!file->level) 
 				break;
@@ -813,40 +824,52 @@ static void toolbar_action_execute(GtkWidget    *widget,
 			    gpointer     data,
 			    gint         source) 
 {
-	GSList *cur, *lop;
-	gchar *action, *action_p;
-	gboolean found = FALSE;
+	GSList *cur;
 	gint i = 0;
 
 	for (cur = action_list; cur != NULL;  cur = cur->next) {
 		ToolbarClawsActions *act = (ToolbarClawsActions*)cur->data;
 
 		if (widget == act->widget) {
-			
-			for (lop = prefs_common.actions_list; lop != NULL; lop = lop->next) {
-				action = g_strdup((gchar*)lop->data);
+			i = prefs_actions_find_by_name(act->name);
 
-				action_p = strstr(action, ": ");
-				action_p[0] = 0x00;
-				if (g_utf8_collate(act->name, action) == 0) {
-					found = TRUE;
-					g_free(action);
-					break;
-				} else 
-					i++;
-				g_free(action);
-			}
-			if (found) 
+			if (i != -1) 
 				break;
 		}
 	}
 
-	if (found) 
+	if (i != -1) 
 		actions_execute(data, i, widget, source);
 	else
 		g_warning ("Error: did not find Claws Action to execute");
 }
 
+gboolean toolbar_check_action_btns(ToolbarType type)
+{
+	GSList *temp, *curr, *list = toolbar_config[type].item_list;
+	gboolean modified = FALSE;
+	
+	curr = list;
+	while (curr != NULL) {
+		ToolbarItem *toolbar_item = (ToolbarItem *) curr->data;
+		temp = curr;
+		curr = curr->next;
+		
+		if (toolbar_item->index != A_CLAWS_ACTIONS)
+			continue;
+
+		if (prefs_actions_find_by_name(toolbar_item->text) == -1) {
+			list = g_slist_delete_link(list, temp);
+			g_free(toolbar_item->file);
+			g_free(toolbar_item->text);
+			g_free(toolbar_item);
+			modified = TRUE;
+		}
+	}
+	
+	return modified;
+}
+
 #if !(GTK_CHECK_VERSION(2,12,0))
 #define CLAWS_SET_TOOL_ITEM_TIP(widget,tip) { \
 	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(widget), GTK_TOOLTIPS(toolbar_tips),	\
@@ -1304,7 +1327,17 @@ static void toolbar_addrbook_cb(GtkWidget *widget, gpointer data)
 	default:
 		return;
 	}
+#ifndef USE_NEW_ADDRBOOK
 	addressbook_open(compose);
+#else
+	GError* error = NULL;
+	addressbook_connect_signals(compose);
+	addressbook_dbus_open(TRUE, &error);
+	if (error) {
+		g_warning("%s", error->message);
+		g_error_free(error);
+	}
+#endif
 }
 
 
@@ -1463,6 +1496,19 @@ static void toolbar_cancel_inc_cb(GtkWidget *widget, gpointer data)
 	imap_cancel_all();
 }
 
+static void toolbar_cancel_send_cb(GtkWidget *widget, gpointer data)
+{
+	ToolbarItem *toolbar_item = (ToolbarItem*)data;
+
+	cm_return_if_fail(toolbar_item != NULL);
+	send_cancel();
+}
+
+static void toolbar_cancel_all_cb(GtkWidget *widget, gpointer data)
+{
+	toolbar_cancel_inc_cb(widget, data);
+	toolbar_cancel_send_cb(widget, data);
+}
 
 static void toolbar_print_cb(GtkWidget *widget, gpointer data)
 {
@@ -1622,6 +1668,12 @@ static void toolbar_actions_execute_cb(GtkWidget *widget, gpointer data)
 	toolbar_action_execute(widget, action_list, parent, toolbar_item->type);	
 }
 
+static void toolbar_plugins_execute_cb(GtkWidget *widget, gpointer data)
+{
+	ToolbarItem *toolbar_item = data;
+	prefs_toolbar_execute_plugin_item(toolbar_item->parent, toolbar_item->type, toolbar_item->text);
+}
+
 static MainWindow *get_mainwin(gpointer data)
 {
 	ToolbarItem *toolbar_item = (ToolbarItem*)data;
@@ -1716,7 +1768,10 @@ static void toolbar_buttons_cb(GtkWidget   *widget,
 		{ A_CHECK_SPELLING,     toolbar_check_spelling_cb       },
 #endif
 		{ A_CLAWS_ACTIONS,	toolbar_actions_execute_cb	},
-		{ A_CANCEL_INC,		toolbar_cancel_inc_cb		}
+		{ A_CANCEL_INC,		toolbar_cancel_inc_cb		},
+		{ A_CANCEL_SEND,	toolbar_cancel_send_cb		},
+		{ A_CANCEL_ALL,		toolbar_cancel_all_cb		},
+		{ A_CLAWS_PLUGINS,  toolbar_plugins_execute_cb  },
 	};
 
 	num_items = sizeof(callbacks)/sizeof(callbacks[0]);
@@ -1731,6 +1786,7 @@ static void toolbar_buttons_cb(GtkWidget   *widget,
 #ifndef GENERIC_UMPC
 #define TOOLBAR_ITEM(item,icon,text,tooltip) {								\
 	item = GTK_WIDGET(gtk_tool_button_new(icon, text));						\
+	gtkut_widget_set_can_focus(gtk_bin_get_child(GTK_BIN(item)), FALSE);				\
 	gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(item), FALSE);					\
 	gtk_tool_item_set_is_important(GTK_TOOL_ITEM(item), TRUE);					\
 	g_signal_connect (G_OBJECT(item), "clicked", G_CALLBACK(toolbar_buttons_cb), toolbar_item);	\
@@ -1754,7 +1810,9 @@ static void toolbar_buttons_cb(GtkWidget   *widget,
 	gchild = gtk_container_get_children(								\
 			GTK_CONTAINER(child)); 								\
 	btn = (GtkWidget *)gchild->data;								\
+	gtkut_widget_set_can_focus(btn, FALSE);								\
 	arr = (GtkWidget *)(gchild->next?gchild->next->data:NULL);					\
+	gtkut_widget_set_can_focus(arr, FALSE);								\
 	g_list_free(gchild);										\
 	gchild = gtk_container_get_children(GTK_CONTAINER(arr));					\
 	gtk_widget_set_size_request(GTK_WIDGET(gchild->data), 9, -1);					\
@@ -1763,6 +1821,7 @@ static void toolbar_buttons_cb(GtkWidget   *widget,
 #else
 #define TOOLBAR_ITEM(item,icon,text,tooltip) {								\
 	item = GTK_WIDGET(gtk_tool_button_new(icon, text));						\
+	gtkut_widget_set_can_focus(gtk_bin_get_child(GTK_BIN(item)), FALSE);				\
 	gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(item), FALSE);					\
 	gtk_tool_item_set_is_important(GTK_TOOL_ITEM(item), TRUE);					\
 	g_signal_connect (G_OBJECT(item), "clicked", G_CALLBACK(toolbar_buttons_cb), toolbar_item);	\
@@ -1781,7 +1840,9 @@ static void toolbar_buttons_cb(GtkWidget   *widget,
 	gchild = gtk_container_get_children(								\
 			GTK_CONTAINER(child)); 								\
 	btn = (GtkWidget *)gchild->data;								\
+	gtkut_widget_set_can_focus(btn, FALSE);								\
 	arr = (GtkWidget *)(gchild->next?gchild->next->data:NULL);					\
+	gtkut_widget_set_can_focus(arr, FALSE);								\
 	g_list_free(gchild);										\
 	gchild = gtk_container_get_children(GTK_CONTAINER(arr));					\
 	gtk_widget_set_size_request(GTK_WIDGET(gchild->data), 9, -1);					\
@@ -1848,7 +1909,11 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 
 	toolbar = gtk_toolbar_new();
 
+#if (GTK_CHECK_VERSION(2,16,0))
+	gtk_orientable_set_orientation(GTK_ORIENTABLE(toolbar), GTK_ORIENTATION_HORIZONTAL);
+#else
 	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar), GTK_ORIENTATION_HORIZONTAL);
+#endif
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_BOTH);
 	gtk_toolbar_set_show_arrow(GTK_TOOLBAR(toolbar), TRUE);
 	
@@ -1880,12 +1945,12 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 			break;
 		case A_RECEIVE_ALL:
 			TOOLBAR_MENUITEM(item,icon_wid,toolbar_item->text,
-				_("Receive Mail on all Accounts"),
-				_("Receive Mail on selected Account"));
+				_("Receive Mail from all Accounts"),
+				_("Receive Mail from selected Account"));
 			toolbar_data->getall_btn = item;
 			break;
 		case A_RECEIVE_CUR:
-			TOOLBAR_ITEM(item,icon_wid,toolbar_item->text, _("Receive Mail on current Account"));
+			TOOLBAR_ITEM(item,icon_wid,toolbar_item->text, _("Receive Mail from current Account"));
 			toolbar_data->get_btn = item;
 			break;
 		case A_SEND_QUEUED:
@@ -2112,6 +2177,17 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 			TOOLBAR_ITEM(item,icon_wid,toolbar_item->text,_("Cancel receiving"));
 			toolbar_data->cancel_inc_btn = item;
 			break;
+		case A_CANCEL_SEND:
+			TOOLBAR_ITEM(item,icon_wid,toolbar_item->text,_("Cancel sending"));
+			toolbar_data->cancel_send_btn = item;
+			break;
+		case A_CANCEL_ALL:
+			TOOLBAR_ITEM(item,icon_wid,toolbar_item->text,_("Cancel receiving/sending"));
+			toolbar_data->cancel_all_btn = item;
+			break;
+		case A_CLAWS_PLUGINS:
+			TOOLBAR_ITEM(item,icon_wid,toolbar_item->text, toolbar_item->text);
+			break;
 		default:
 			TOOLBAR_ITEM(item,icon_wid,toolbar_item->text,
 				toolbar_ret_descr_from_val(toolbar_item->index));
@@ -2135,8 +2211,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 		item = GTK_WIDGET(gtk_tool_item_new());
 		gtk_container_add (GTK_CONTAINER (item), progressbar);
 		gtk_widget_show(item);
-		gtk_widget_show(progressbar);
-		gtk_widget_set_size_request(progressbar, 70, -1);
+		gtk_widget_set_size_request(progressbar, 84, -1);
 		gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(item), -1);
 		mainwin->progressbar = progressbar;
 #endif
@@ -2150,7 +2225,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 	
 #ifndef MAEMO
 	gtk_container_add(GTK_CONTAINER(container), toolbar);
-	gtk_container_set_border_width(GTK_CONTAINER(container), 2);
+	gtk_container_set_border_width(GTK_CONTAINER(container), 0);
 #else
 	if ( GTK_IS_WINDOW(container) ) {
 		hildon_window_add_toolbar (HILDON_WINDOW(container), GTK_TOOLBAR(toolbar));
@@ -2305,7 +2380,7 @@ void toolbar_update(ToolbarType type, gpointer data)
 
 void toolbar_main_set_sensitive(gpointer data)
 {
-	SensitiveCond state;
+	SensitiveCondMask state;
 	gboolean sensitive;
 	MainWindow *mainwin = (MainWindow*)data;
 	Toolbar *toolbar = mainwin->toolbar;
@@ -2315,26 +2390,27 @@ void toolbar_main_set_sensitive(gpointer data)
 	typedef struct _Entry Entry;
 	struct _Entry {
 		GtkWidget *widget;
-		SensitiveCond cond;
+		SensitiveCondMask cond;
 		gboolean empty;
 	};
 
-#define SET_WIDGET_COND(w, c)     \
-{ \
+#define SET_WIDGET_COND(w, ...)     \
+do { \
 	Entry *e = g_new0(Entry, 1); \
 	e->widget = w; \
-	e->cond   = c; \
+	e->cond = main_window_get_mask(__VA_ARGS__, -1); \
 	entry_list = g_slist_append(entry_list, e); \
-}
-
+} while (0)
 	
+	/* match all bit flags */
+
 	if (toolbar->get_btn)
 		SET_WIDGET_COND(toolbar->get_btn, 
-			M_HAVE_ACCOUNT|M_UNLOCKED);
+			M_HAVE_ACCOUNT, M_UNLOCKED, M_HAVE_RETRIEVABLE_ACCOUNT);
 
 	if (toolbar->getall_btn) {
 		SET_WIDGET_COND(toolbar->getall_btn, 
-			M_HAVE_ACCOUNT|M_UNLOCKED);
+			M_HAVE_ACCOUNT, M_UNLOCKED, M_HAVE_ANY_RETRIEVABLE_ACCOUNT);
 	}
 	if (toolbar->send_btn) {
 		SET_WIDGET_COND(toolbar->send_btn,
@@ -2350,42 +2426,42 @@ void toolbar_main_set_sensitive(gpointer data)
 	}
 	if (toolbar->open_mail_btn) {
 		SET_WIDGET_COND(toolbar->open_mail_btn, 
-			M_TARGET_EXIST|M_SUMMARY_ISLIST);
+			M_TARGET_EXIST, M_SUMMARY_ISLIST);
 	}
 	if (toolbar->reply_btn) {
 		SET_WIDGET_COND(toolbar->reply_btn,
-			M_HAVE_ACCOUNT|M_TARGET_EXIST|M_SUMMARY_ISLIST);
+			M_HAVE_ACCOUNT, M_TARGET_EXIST, M_SUMMARY_ISLIST);
 	}
 	if (toolbar->replyall_btn) {
 		SET_WIDGET_COND(toolbar->replyall_btn,
-			M_HAVE_ACCOUNT|M_TARGET_EXIST|M_SUMMARY_ISLIST);
+			M_HAVE_ACCOUNT, M_TARGET_EXIST, M_SUMMARY_ISLIST);
 	}
 	if (toolbar->replylist_btn) {
 		SET_WIDGET_COND(toolbar->replylist_btn,
-			M_HAVE_ACCOUNT|M_TARGET_EXIST|M_SUMMARY_ISLIST);
+			M_HAVE_ACCOUNT, M_TARGET_EXIST, M_SUMMARY_ISLIST);
 	}
 	if (toolbar->replysender_btn) {
 		SET_WIDGET_COND(toolbar->replysender_btn,
-			M_HAVE_ACCOUNT|M_TARGET_EXIST|M_SUMMARY_ISLIST);
+			M_HAVE_ACCOUNT, M_TARGET_EXIST, M_SUMMARY_ISLIST);
 	}
 	if (toolbar->fwd_btn) {
 		SET_WIDGET_COND(toolbar->fwd_btn, 
-			M_HAVE_ACCOUNT|M_TARGET_EXIST|M_SUMMARY_ISLIST);
+			M_HAVE_ACCOUNT, M_TARGET_EXIST, M_SUMMARY_ISLIST);
 	}
 
 	if (prefs_common.next_unread_msg_dialog == NEXTUNREADMSGDIALOG_ASSUME_NO) {
-		SET_WIDGET_COND(toolbar->next_btn, M_MSG_EXIST|M_SUMMARY_ISLIST);
+		SET_WIDGET_COND(toolbar->next_btn, M_MSG_EXIST, M_SUMMARY_ISLIST);
 	} else {
-		SET_WIDGET_COND(toolbar->next_btn, 0);
+		SET_WIDGET_COND(toolbar->next_btn, -1);
 	}
 	
 	if (toolbar->trash_btn)
 		SET_WIDGET_COND(toolbar->trash_btn,
-			M_TARGET_EXIST|M_ALLOW_DELETE|M_NOT_NEWS);
+			M_TARGET_EXIST, M_ALLOW_DELETE, M_NOT_NEWS);
 
 	if (toolbar->delete_btn)
 		SET_WIDGET_COND(toolbar->delete_btn,
-			M_TARGET_EXIST|M_ALLOW_DELETE);
+			M_TARGET_EXIST, M_ALLOW_DELETE);
 
 	if (toolbar->exec_btn)
 		SET_WIDGET_COND(toolbar->exec_btn, 
@@ -2393,20 +2469,22 @@ void toolbar_main_set_sensitive(gpointer data)
 	
 	if (toolbar->learn_spam_btn)
 		SET_WIDGET_COND(toolbar->learn_spam_btn, 
-			M_TARGET_EXIST|M_CAN_LEARN_SPAM|M_SUMMARY_ISLIST);
+			M_TARGET_EXIST, M_CAN_LEARN_SPAM, M_SUMMARY_ISLIST);
 
 	if (toolbar->cancel_inc_btn)
 		SET_WIDGET_COND(toolbar->cancel_inc_btn,
 				M_INC_ACTIVE);
 
+	if (toolbar->cancel_send_btn)
+		SET_WIDGET_COND(toolbar->cancel_send_btn,
+				M_SEND_ACTIVE);
+
 	for (cur = toolbar->action_list; cur != NULL;  cur = cur->next) {
 		ToolbarClawsActions *act = (ToolbarClawsActions*)cur->data;
 		
-		SET_WIDGET_COND(act->widget, M_TARGET_EXIST|M_UNLOCKED);
+		SET_WIDGET_COND(act->widget, M_TARGET_EXIST, M_UNLOCKED);
 	}
 
-#undef SET_WIDGET_COND
-
 	state = main_window_get_current_state(mainwin);
 
 	for (cur = entry_list; cur != NULL; cur = cur->next) {
@@ -2425,12 +2503,35 @@ void toolbar_main_set_sensitive(gpointer data)
 		entry_list = g_slist_remove(entry_list, e);
 	}
 
+	/* match any bit flags */
+
+	if (toolbar->cancel_all_btn)
+		SET_WIDGET_COND(toolbar->cancel_all_btn,
+				M_INC_ACTIVE, M_SEND_ACTIVE);
+
+	for (cur = entry_list; cur != NULL; cur = cur->next) {
+		Entry *e = (Entry*) cur->data;
+
+		if (e->widget != NULL) {
+			sensitive = ((e->cond & state) != 0);
+			GTK_BUTTON_SET_SENSITIVE(e->widget, sensitive);	
+		}
+	}
+
+	while (entry_list != NULL) {
+		Entry *e = (Entry*) entry_list->data;
+
+		g_free(e);
+		entry_list = g_slist_remove(entry_list, e);
+	}
+
 	g_slist_free(entry_list);
 
 	activate_compose_button(toolbar, 
 				prefs_common.toolbar_style,
 				toolbar->compose_btn_type);
 	
+#undef SET_WIDGET_COND
 }
 
 void toolbar_comp_set_sensitive(gpointer data, gboolean sensitive)
@@ -2474,38 +2575,53 @@ void toolbar_comp_set_sensitive(gpointer data, gboolean sensitive)
 static void toolbar_init(Toolbar * toolbar)
 {
 
-	toolbar->toolbar          	= NULL;
-	toolbar->folders_btn		= NULL;
-	toolbar->get_btn          	= NULL;
-	toolbar->getall_btn       	= NULL;
-	toolbar->send_btn         	= NULL;
-	toolbar->compose_mail_btn 	= NULL;
-	toolbar->compose_mail_icon 	= NULL;
-	toolbar->compose_news_icon 	= NULL;
-	toolbar->reply_btn        	= NULL;
-	toolbar->replysender_btn  	= NULL;
-	toolbar->replyall_btn     	= NULL;
-	toolbar->replylist_btn    	= NULL;
-	toolbar->fwd_btn          	= NULL;
-	toolbar->trash_btn       	= NULL;
-	toolbar->delete_btn       	= NULL;
-	toolbar->prev_btn         	= NULL;
-	toolbar->next_btn         	= NULL;
-	toolbar->exec_btn         	= NULL;
-	toolbar->open_mail_btn		= NULL;
-	toolbar->close_window_btn	= NULL;
+	toolbar->toolbar           = NULL;
+	toolbar->folders_btn       = NULL;
+	toolbar->get_btn           = NULL;
+	toolbar->getall_btn        = NULL;
+	toolbar->send_btn          = NULL;
+	toolbar->compose_mail_btn  = NULL;
+	toolbar->compose_mail_icon = NULL;
+	toolbar->compose_news_icon = NULL;
+	toolbar->reply_btn         = NULL;
+	toolbar->replysender_btn   = NULL;
+	toolbar->replyall_btn      = NULL;
+	toolbar->replylist_btn     = NULL;
+	toolbar->fwd_btn           = NULL;
+	toolbar->trash_btn         = NULL;
+	toolbar->delete_btn        = NULL;
+	toolbar->prev_btn          = NULL;
+	toolbar->next_btn          = NULL;
+	toolbar->exec_btn          = NULL;
+	toolbar->separator         = NULL;
+	toolbar->learn_spam_btn    = NULL;
+	toolbar->learn_spam_icon   = NULL;
+	toolbar->learn_ham_icon    = NULL;
+	toolbar->cancel_inc_btn    = NULL;
+	toolbar->cancel_send_btn   = NULL;
+	toolbar->cancel_all_btn    = NULL;
+
 	/* compose buttons */ 
-	toolbar->sendl_btn        	= NULL;
-	toolbar->draft_btn        	= NULL;
-	toolbar->insert_btn       	= NULL;
-	toolbar->attach_btn       	= NULL;
-	toolbar->sig_btn          	= NULL;	
-	toolbar->exteditor_btn    	= NULL;	
-	toolbar->linewrap_current_btn	= NULL;	
-	toolbar->linewrap_all_btn     	= NULL;	
-	toolbar->addrbook_btn     	= NULL;	
+	toolbar->sendl_btn         = NULL;
+	toolbar->draft_btn         = NULL;
+	toolbar->insert_btn        = NULL;
+	toolbar->attach_btn        = NULL;
+	toolbar->sig_btn           = NULL; 
+	toolbar->exteditor_btn     = NULL; 
+	toolbar->linewrap_current_btn = NULL;	
+	toolbar->linewrap_all_btn  = NULL;	
+	toolbar->addrbook_btn      = NULL; 
+
+	toolbar->open_mail_btn     = NULL;
+	toolbar->close_window_btn  = NULL;
+	toolbar->preferences_btn   = NULL;
+	toolbar->action_list       = NULL;
+	toolbar->item_list         = NULL;
 #ifdef USE_ENCHANT
-	toolbar->spellcheck_btn   	= NULL;
+	toolbar->spellcheck_btn    = NULL;
+#endif
+#if !GTK_CHECK_VERSION(2,12,0)
+	toolbar->tooltips          = NULL;
 #endif
 
 	toolbar_destroy(toolbar);
