@@ -1,6 +1,6 @@
 /*
  * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Colin Leroy <colin@colino.net> 
+ * Copyright (C) 1999-2012 Colin Leroy <colin@colino.net> 
  * and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -36,6 +37,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "utils.h"
@@ -450,7 +452,7 @@ static void initialize_fonts(WizardWindow *wizard)
 {
 	GtkWidget *widget = wizard->email;
 	gint size = pango_font_description_get_size(
-			widget->style->font_desc)
+			gtk_widget_get_style(widget)->font_desc)
 		      /PANGO_SCALE;
 	gchar *tmp, *new;
 #ifdef G_OS_WIN32
@@ -534,7 +536,7 @@ static void write_welcome_email(WizardWindow *wizard)
 	get_rfc822_date(buf_date, sizeof(buf_date));
 
 	conv_encode_header_full(enc_subject, sizeof(enc_subject), 
-			Q_("Welcome Mail Subject|Welcome to Claws Mail"),
+			C_("Welcome Mail Subject", "Welcome to Claws Mail"),
 			strlen("Subject: "), FALSE, CS_INTERNAL);
 	conv_encode_header_full(enc_to_name, sizeof(enc_to_name), 
 			gtk_entry_get_text(GTK_ENTRY(wizard->full_name)),
@@ -992,7 +994,8 @@ static GtkWidget* user_page (WizardWindow * wizard)
 {
 	GtkWidget *table = gtk_table_new(1,1, FALSE);
 	GtkWidget *vbox;
-	GtkWidget *hbox;
+	GtkWidget *label;
+	GtkWidget *user_table;
 	
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
@@ -1003,25 +1006,38 @@ static GtkWidget* user_page (WizardWindow * wizard)
 	gtk_table_attach(GTK_TABLE(table), vbox, 0,1,1,2, 
 			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
-	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	user_table = gtk_table_new(3, 2, FALSE);
+	gtk_table_set_row_spacings(GTK_TABLE(user_table), VSPACING_NARROW);
+	gtk_box_pack_start(GTK_BOX(vbox), user_table, FALSE, FALSE, 0);
+
+	label = gtk_label_new(_("<span weight=\"bold\">Your name:</span>"));
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+	gtk_table_attach(GTK_TABLE(user_table), label, 0,1,0,1, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->full_name = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(wizard->full_name), tmpl.name?tmpl.name:"");
-	PACK_BOX(hbox, _("<span weight=\"bold\">Your name:</span>"),
-		 wizard->full_name);
-	
-	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_table_attach(GTK_TABLE(user_table), wizard->full_name, 1,2,0,1, 
+			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
+
+	label = gtk_label_new(_("<span weight=\"bold\">Your email address:</span>"));
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+	gtk_table_attach(GTK_TABLE(user_table), label, 0,1,1,2, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->email = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(wizard->email), tmpl.email?tmpl.email:"");
-	PACK_BOX(hbox, _("<span weight=\"bold\">Your email address:</span>"),
-		 wizard->email);
-	
-	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);	
+	gtk_table_attach(GTK_TABLE(user_table), wizard->email, 1,2,1,2, 
+			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
+
+	label = gtk_label_new(_("Your organization:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+	gtk_table_attach(GTK_TABLE(user_table), label, 0,1,2,3, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->organization = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(wizard->organization), tmpl.organization?tmpl.organization:"");
-	PACK_BOX(hbox, _("Your organization:"), wizard->organization);
+	gtk_table_attach(GTK_TABLE(user_table), wizard->organization, 1,2,2,3, 
+			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	
 	g_signal_connect(G_OBJECT(wizard->email), "changed",
 			 G_CALLBACK(wizard_email_changed),
@@ -1109,7 +1125,6 @@ static GtkWidget* mailbox_page (WizardWindow * wizard)
 	gchar *uri, *mount_path;
 #endif
 	GtkWidget *hbox;
-	CLAWS_TIP_DECL();
 
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
@@ -1195,8 +1210,7 @@ static GtkWidget* mailbox_page (WizardWindow * wizard)
 
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
 	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-	PACK_BOX(hbox, _("<span weight=\"bold\">Store data</span>"),
-		 vbox2);
+	PACK_BOX(hbox, _("<span weight=\"bold\">Store data</span>"), vbox2);
 #endif
 
 	return table;
@@ -1238,13 +1252,14 @@ static GtkWidget* smtp_page (WizardWindow * wizard)
 	GtkWidget *table = gtk_table_new(1, 1, FALSE);
 	GtkWidget *vbox;
 	GtkWidget *hbox;
-#ifdef USE_GNUTLS
+	GtkWidget *hbox_spc;
+	GtkWidget *smtp_auth_table;
 	GtkWidget *label;
+#ifdef USE_GNUTLS
 	GtkWidget *button;
 	GtkWidget *smtp_cert_table;
 #endif
 	gchar *text;
-	CLAWS_TIP_DECL();
 	
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
@@ -1269,7 +1284,6 @@ static GtkWidget* smtp_page (WizardWindow * wizard)
 	PACK_BOX(hbox, _("<span weight=\"bold\">SMTP server address:</span>"),
 		 wizard->smtp_server);
 
-
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
 	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 	wizard->smtp_auth = gtk_check_button_new_with_label(
@@ -1281,23 +1295,35 @@ static GtkWidget* smtp_page (WizardWindow * wizard)
 			 wizard);
 	gtk_box_pack_start(GTK_BOX(hbox), wizard->smtp_auth, FALSE, FALSE, 0);
 
-	text = get_default_smtp_account(wizard);
+	label = gtk_label_new(_("<span size=\"small\">(empty to use the same as receive)</span>"));
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+	SET_TOGGLE_SENSITIVITY (wizard->smtp_auth, label);	
+	gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
 	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
+	smtp_auth_table = gtk_table_new(2, 2, FALSE);
+	SET_TOGGLE_SENSITIVITY (wizard->smtp_auth, smtp_auth_table);
+	gtk_box_pack_start(GTK_BOX(hbox), smtp_auth_table, TRUE, TRUE, 0);
+
+	wizard->smtp_username_label = gtk_label_new(_("SMTP username:"));
+	gtk_misc_set_alignment(GTK_MISC(wizard->smtp_username_label), 1, 0.5);	      
+	gtk_table_attach(GTK_TABLE(smtp_auth_table), wizard->smtp_username_label, 0,1,0,1, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
+	text = get_default_smtp_account(wizard);
 	wizard->smtp_username = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(wizard->smtp_username), text);
 	g_free(text);
-	wizard->smtp_username_label = gtk_label_new(_("SMTP username:\n"
-					"<span size=\"small\">(empty to use the same as receive)</span>"));
-	gtk_label_set_use_markup(GTK_LABEL(wizard->smtp_username_label), TRUE);
-	if (GTK_IS_MISC(wizard->smtp_username_label))						      
-		gtk_misc_set_alignment(GTK_MISC(wizard->smtp_username_label), 1, 0.5);	      
-	gtk_box_pack_start(GTK_BOX(hbox), wizard->smtp_username_label, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), wizard->smtp_username, TRUE, TRUE, 0);
+	gtk_table_attach(GTK_TABLE(smtp_auth_table), wizard->smtp_username, 1,2,0,1, 
+			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
-	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	wizard->smtp_password_label = gtk_label_new(_("SMTP password:"));
+	gtk_misc_set_alignment(GTK_MISC(wizard->smtp_password_label), 1, 0.5);	      
+	gtk_table_attach(GTK_TABLE(smtp_auth_table), wizard->smtp_password_label, 0,1,1,2, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->smtp_password = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(wizard->smtp_password), tmpl.smtppass?tmpl.smtppass:""); 
 	gtk_entry_set_visibility(GTK_ENTRY(wizard->smtp_password), FALSE);
@@ -1305,13 +1331,8 @@ static GtkWidget* smtp_page (WizardWindow * wizard)
 	hildon_gtk_entry_set_input_mode(GTK_ENTRY(wizard->smtp_password), 
 		HILDON_GTK_INPUT_MODE_FULL | HILDON_GTK_INPUT_MODE_INVISIBLE);
 #endif
-	wizard->smtp_password_label = gtk_label_new(_("SMTP password:\n"
-					"<span size=\"small\">(empty to use the same as receive)</span>"));
-	gtk_label_set_use_markup(GTK_LABEL(wizard->smtp_password_label), TRUE);
-	if (GTK_IS_MISC(wizard->smtp_password_label))						      
-		gtk_misc_set_alignment(GTK_MISC(wizard->smtp_password_label), 1, 0.5);	      
-	gtk_box_pack_start(GTK_BOX(hbox), wizard->smtp_password_label, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), wizard->smtp_password, TRUE, TRUE, 0);
+	gtk_table_attach(GTK_TABLE(smtp_auth_table), wizard->smtp_password, 1,2,1,2, 
+			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
 #ifdef USE_GNUTLS
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
 	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
@@ -1323,6 +1344,9 @@ static GtkWidget* smtp_page (WizardWindow * wizard)
 
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
 	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 	wizard->smtp_use_tls = gtk_check_button_new_with_label(
 					_("Use SSL via STARTTLS"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wizard->smtp_use_tls),
@@ -1331,36 +1355,47 @@ static GtkWidget* smtp_page (WizardWindow * wizard)
 	SET_TOGGLE_SENSITIVITY (wizard->smtp_use_ssl, wizard->smtp_use_tls);
 	
 	smtp_cert_table = gtk_table_new(3,3, FALSE);
-	gtk_container_set_border_width(GTK_CONTAINER(smtp_cert_table), 8);
-	gtk_box_pack_start (GTK_BOX(vbox), smtp_cert_table, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX(vbox), smtp_cert_table, FALSE, FALSE, 4);
+	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
 	label = gtk_label_new(_("Client SSL certificate (optional)"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_table_attach(GTK_TABLE(smtp_cert_table), label, 0, 3, 0, 1, GTK_FILL, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_table_attach(GTK_TABLE(smtp_cert_table), hbox, 0, 3, 0, 1, GTK_FILL, 0, 0, 0);
+	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
 	label = gtk_label_new(_("File"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach(GTK_TABLE(smtp_cert_table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+	gtk_table_attach(GTK_TABLE(smtp_cert_table), hbox, 0, 1, 1, 2, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->smtp_ssl_cert_file = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(wizard->smtp_ssl_cert_file), tmpl.smtpssl_cert?tmpl.smtpssl_cert:"");
 	gtk_table_attach(GTK_TABLE(smtp_cert_table), wizard->smtp_ssl_cert_file, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
 	button = gtkut_get_browse_file_btn(_("Browse"));
-	gtk_table_attach(GTK_TABLE(smtp_cert_table), button, 2, 3, 1, 2, GTK_FILL, 0, 0, 0);
-	SET_TOGGLE_SENSITIVITY (wizard->smtp_use_ssl, label);
-	SET_TOGGLE_SENSITIVITY (wizard->smtp_use_ssl, wizard->smtp_ssl_cert_file);
-	SET_TOGGLE_SENSITIVITY (wizard->smtp_use_ssl, button);
+	gtk_table_attach(GTK_TABLE(smtp_cert_table), button, 2, 3, 1, 2, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(cert_browse_cb), wizard->smtp_ssl_cert_file);
 
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);	
 	label = gtk_label_new(_("Password"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach(GTK_TABLE(smtp_cert_table), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+	gtk_table_attach(GTK_TABLE(smtp_cert_table), hbox, 0, 1, 2, 3, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->smtp_ssl_cert_pass = gtk_entry_new();
 	gtk_entry_set_visibility(GTK_ENTRY(wizard->smtp_ssl_cert_pass), FALSE);
 	gtk_entry_set_text(GTK_ENTRY(wizard->smtp_ssl_cert_pass), tmpl.smtpssl_cert_pass?tmpl.smtpssl_cert_pass:"");
 	gtk_table_attach(GTK_TABLE(smtp_cert_table), wizard->smtp_ssl_cert_pass, 1, 2, 2, 3, GTK_FILL, 0, 0, 0);
-	SET_TOGGLE_SENSITIVITY (wizard->smtp_use_ssl, label);
-	SET_TOGGLE_SENSITIVITY (wizard->smtp_use_ssl, wizard->smtp_ssl_cert_pass);
+	SET_TOGGLE_SENSITIVITY (wizard->smtp_use_ssl, smtp_cert_table);
 	wizard->smtp_cert_table = smtp_cert_table;
 #endif
 	smtp_auth_changed(NULL, wizard);
@@ -1478,8 +1513,10 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	GtkWidget *table = gtk_table_new(1,1, FALSE);
 	GtkWidget *vbox;
 	GtkWidget *hbox;
-#ifdef USE_GNUTLS
+	GtkWidget *hbox_spc;	
+	GtkWidget *recv_table;
 	GtkWidget *label;
+#ifdef USE_GNUTLS
 	GtkWidget *button;
 	GtkWidget *recv_cert_table;
 #endif
@@ -1487,7 +1524,6 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	GtkTreeIter iter;
 	gchar *text;
 	gint index = 0;
-	CLAWS_TIP_DECL();
 
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
@@ -1498,8 +1534,14 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	gtk_table_attach(GTK_TABLE(table), vbox, 0,1,1,2, 
 			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
-	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	recv_table = gtk_table_new(4, 2, FALSE); 
+	gtk_box_pack_start(GTK_BOX(vbox), recv_table, FALSE, FALSE, 0);
+
+	label = gtk_label_new(_("<span weight=\"bold\">Server type:</span>"));
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+	gtk_table_attach(GTK_TABLE(recv_table), label, 0,1,0,1, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->recv_type = gtkut_sc_combobox_create(NULL, FALSE);
 	store = GTK_LIST_STORE(gtk_combo_box_get_model(
 			GTK_COMBO_BOX(wizard->recv_type)));
@@ -1525,11 +1567,14 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	g_signal_connect(G_OBJECT(wizard->recv_type), "changed",
 			 G_CALLBACK(wizard_protocol_changed),
 			 wizard);
-	PACK_BOX(hbox, _("<span weight=\"bold\">Server type:</span>"),
-		 wizard->recv_type);
+	gtk_table_attach(GTK_TABLE(recv_table), wizard->recv_type, 1,2,0,1, 
+			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
-	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	wizard->recv_label = gtk_label_new(_("<span weight=\"bold\">Server address:</span>"));
+	gtk_misc_set_alignment(GTK_MISC(wizard->recv_label), 1, 0.5);
+	gtk_label_set_use_markup(GTK_LABEL(wizard->recv_label), TRUE);
+	gtk_table_attach(GTK_TABLE(recv_table), wizard->recv_label, 0,1,1,2, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->recv_server = gtk_entry_new();
 	text = get_default_server(wizard, "pop");
 	gtk_entry_set_text(GTK_ENTRY(wizard->recv_server), text);
@@ -1538,42 +1583,34 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	CLAWS_SET_TIP(wizard->recv_server,
 			     _("You can specify the port number by appending it at the end: "
 			       "\"mail.example.com:110\""));
-
-	wizard->recv_label = gtk_label_new(_("<span weight=\"bold\">Server address:</span>"));
-	gtk_label_set_use_markup(GTK_LABEL(wizard->recv_label), TRUE);
-	if (GTK_IS_MISC(wizard->recv_label))						      
-		gtk_misc_set_alignment(GTK_MISC(wizard->recv_label), 1, 0.5);	      
-	gtk_box_pack_start(GTK_BOX(hbox), wizard->recv_label, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), wizard->recv_server, TRUE, TRUE, 0);
+	gtk_table_attach(GTK_TABLE(recv_table), wizard->recv_server, 1,2,1,2, 
+			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	
-	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-	wizard->recv_username = gtk_entry_new();
 	wizard->recv_username_label = gtk_label_new(_("<span weight=\"bold\">Username:</span>"));
+	gtk_misc_set_alignment(GTK_MISC(wizard->recv_username_label), 1, 0.5);
 	gtk_label_set_use_markup(GTK_LABEL(wizard->recv_username_label), TRUE);
-	if (GTK_IS_MISC(wizard->recv_username_label))						      
-		gtk_misc_set_alignment(GTK_MISC(wizard->recv_username_label), 1, 0.5);	      
-	gtk_box_pack_start(GTK_BOX(hbox), wizard->recv_username_label, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), wizard->recv_username, TRUE, TRUE, 0);
-	
+	gtk_table_attach(GTK_TABLE(recv_table), wizard->recv_username_label, 0,1,2,3, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
+	wizard->recv_username = gtk_entry_new();
 	text = get_default_account(wizard);
 	gtk_entry_set_text(GTK_ENTRY(wizard->recv_username), text);
 	g_free(text);
-
-	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_table_attach(GTK_TABLE(recv_table), wizard->recv_username, 1,2,2,3, 
+			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
+			 
+	wizard->recv_password_label = gtk_label_new(_("Password:"));
+	gtk_misc_set_alignment(GTK_MISC(wizard->recv_password_label), 1, 0.5);
+	gtk_table_attach(GTK_TABLE(recv_table), wizard->recv_password_label, 0,1,3,4, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->recv_password = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(wizard->recv_password), tmpl.recvpass?tmpl.recvpass:"");
-	wizard->recv_password_label = gtk_label_new(_("Password:"));
-	if (GTK_IS_MISC(wizard->recv_password_label))						      
-		gtk_misc_set_alignment(GTK_MISC(wizard->recv_password_label), 1, 0.5);	      
 	gtk_entry_set_visibility(GTK_ENTRY(wizard->recv_password), FALSE);
+	gtk_table_attach(GTK_TABLE(recv_table), wizard->recv_password, 1,2,3,4, 
+			 GTK_EXPAND|GTK_FILL, 0, 0, 0);
 #ifdef MAEMO
 	hildon_gtk_entry_set_input_mode(GTK_ENTRY(wizard->recv_password), 
 		HILDON_GTK_INPUT_MODE_FULL | HILDON_GTK_INPUT_MODE_INVISIBLE);
 #endif
-	gtk_box_pack_start(GTK_BOX(hbox), wizard->recv_password_label, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), wizard->recv_password, TRUE, TRUE, 0);
 	
 #ifdef USE_GNUTLS
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
@@ -1586,6 +1623,9 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
 	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);	
 	wizard->recv_use_tls = gtk_check_button_new_with_label(
 					_("Use SSL via STARTTLS"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wizard->recv_use_tls),
@@ -1594,36 +1634,47 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	SET_TOGGLE_SENSITIVITY (wizard->recv_use_ssl, wizard->recv_use_tls);
 
 	recv_cert_table = gtk_table_new(3,3, FALSE);
-	gtk_container_set_border_width(GTK_CONTAINER(recv_cert_table), 8);
-	gtk_box_pack_start (GTK_BOX(vbox), recv_cert_table, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX(vbox), recv_cert_table, FALSE, FALSE, 4);
+	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
 	label = gtk_label_new(_("Client SSL certificate (optional)"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_table_attach(GTK_TABLE(recv_cert_table), label, 0, 3, 0, 1, GTK_FILL, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);	
+	gtk_table_attach(GTK_TABLE(recv_cert_table), hbox, 0, 3, 0, 1, GTK_FILL, 0, 0, 0);
+	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
 	label = gtk_label_new(_("File"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach(GTK_TABLE(recv_cert_table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);	
+	gtk_table_attach(GTK_TABLE(recv_cert_table), hbox, 0, 1, 1, 2, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->recv_ssl_cert_file = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(wizard->recv_ssl_cert_file), tmpl.recvssl_cert?tmpl.recvssl_cert:"");
 	gtk_table_attach(GTK_TABLE(recv_cert_table), wizard->recv_ssl_cert_file, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
 	button = gtkut_get_browse_file_btn(_("Browse"));
-	gtk_table_attach(GTK_TABLE(recv_cert_table), button, 2, 3, 1, 2, GTK_FILL, 0, 0, 0);
-	SET_TOGGLE_SENSITIVITY (wizard->recv_use_ssl, label);
-	SET_TOGGLE_SENSITIVITY (wizard->recv_use_ssl, wizard->recv_ssl_cert_file);
-	SET_TOGGLE_SENSITIVITY (wizard->recv_use_ssl, button);
+	gtk_table_attach(GTK_TABLE(recv_cert_table), button, 2, 3, 1, 2, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(cert_browse_cb), wizard->recv_ssl_cert_file);
 
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
 	label = gtk_label_new(_("Password"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach(GTK_TABLE(recv_cert_table), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+	gtk_table_attach(GTK_TABLE(recv_cert_table), hbox, 0, 1, 2, 3, 
+			 GTK_FILL, 0, VSPACING_NARROW, 0);
 	wizard->recv_ssl_cert_pass = gtk_entry_new();
 	gtk_entry_set_visibility(GTK_ENTRY(wizard->recv_ssl_cert_pass), FALSE);
 	gtk_entry_set_text(GTK_ENTRY(wizard->recv_ssl_cert_pass), tmpl.recvssl_cert_pass?tmpl.recvssl_cert_pass:"");
 	gtk_table_attach(GTK_TABLE(recv_cert_table), wizard->recv_ssl_cert_pass, 1, 2, 2, 3, GTK_FILL, 0, 0, 0);
-	SET_TOGGLE_SENSITIVITY (wizard->recv_use_ssl, label);
-	SET_TOGGLE_SENSITIVITY (wizard->recv_use_ssl, wizard->recv_ssl_cert_pass);
+	SET_TOGGLE_SENSITIVITY (wizard->recv_use_ssl, recv_cert_table);	
 	wizard->recv_cert_table = recv_cert_table;
 #endif	
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
@@ -1631,13 +1682,15 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	wizard->recv_imap_subdir = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(wizard->recv_imap_subdir), tmpl.imapdir?tmpl.imapdir:"");
 	wizard->recv_imap_label = gtk_label_new(_("IMAP server directory:"));
-	if (GTK_IS_MISC(wizard->recv_imap_label))						      
-		gtk_misc_set_alignment(GTK_MISC(wizard->recv_imap_label), 1, 0.5);	      
+	gtk_misc_set_alignment(GTK_MISC(wizard->recv_imap_label), 1, 0.5);	      
 	gtk_box_pack_start(GTK_BOX(hbox), wizard->recv_imap_label, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), wizard->recv_imap_subdir, TRUE, TRUE, 0);
 	
 	hbox = gtk_hbox_new(FALSE, VSPACING_NARROW);
 	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
 	wizard->subsonly_checkbtn = gtk_check_button_new_with_label(
 			_("Show only subscribed folders"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wizard->subsonly_checkbtn),
@@ -1775,6 +1828,7 @@ gboolean run_wizard(MainWindow *mainwin, gboolean create_mailbox) {
 			GTK_STOCK_CANCEL, CANCEL,
 			NULL);
 	gtk_widget_set_size_request(wizard->window, -1, 480);
+	gtk_window_set_position(GTK_WINDOW(wizard->window), GTK_WIN_POS_CENTER);
 
 	g_signal_connect(wizard->window, "response", 
 			  G_CALLBACK(wizard_response_cb), wizard);
@@ -1793,7 +1847,7 @@ gboolean run_wizard(MainWindow *mainwin, gboolean create_mailbox) {
 	wizard->notebook = gtk_notebook_new();
 	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(wizard->notebook), FALSE);
 	gtk_notebook_set_show_border(GTK_NOTEBOOK(wizard->notebook), FALSE);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(wizard->window)->vbox), 
+	gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(wizard->window))), 
 			    wizard->notebook, TRUE, TRUE, 0);
 	
 	wizard->pages = NULL;
